classdef (ConstructOnLoad = true) CBR < handle
    %Objecte CBR, aquest objecte s el que s'encarregara de mitjanant
    %objectes PCA i CBR mantenir un model i una base de casos.
    
    properties (Access = private)
        %Model i Case_Base seran 2 objectes de tipus PCA que aniran lligats
        %a l'existencia de l'objecte actual, un cop els haguem assignat no
        %podran canviar-se. En el cas del model no es podra modificar.
        Model;%sera un objecte de tipus PCA i sera el nostre model descriptor del problema
        Case_Base;%PCA no model - La base de casos del problema
        
        %Aqui guardarem els parametres necesaris per les funcionalitats propies
        %del CBR. Un com s'han entrat no es nessesari entrar -les de nou a
        %menys que es bulgui canviar els seus valors.
        vk; %elements que agafem a cada nivell d'ordenaci
        dc; %metode de calcular les distncies
        method; %metode d'elecci de la classe segons les classes dels veins
        
        %Aqui indicarem que s'ha fet una optimitzaci a la base de casos.
        %Si fem una optimitzaci els parametres vk, dc, i method queden
        %fixats per sempre!
        block;
        
        %Aqui emmagatzemarem temporalment els veins i els elements a
        %testejar per no haver de passar-los per parametre a cada funci.
        test;%Objecte de tipus PCA temporal on guardarem l'ultim set de dades tractat o a tractar en breu
        veins;%Aqui guardarem kNN veins
        classepredit;%variable per emmagatzemar la classe predita
        
        param;%parametre extre per les distancies 17 i 18 indica el tan per 1 a partir del qual considerem 1.
        
        treuoutliers;%indica si bolem treure els outliers o no
    end
    
    methods
        %constructor
        function obj=CBR(varargin)
            %Metode constructor de la classe CBR.
            %
            %Sense parametres d'entrada 'x = CBR();' creara un objecte
            %buit, tots els cams s'inicialitzen a buit.
            %
            %Amb parametres, Podem tenir 2 o 3 parametres d'entrada, els 2
            %primers correspondaran a un Model PCA i una Base de casos. El
            %3er parametre indicar si bolem eliminar outliers o no.
            %
            %Parametres d'entrada:
            %
            %   varargin{1} (opcional) - Model PCA que s'utilitzara com a
            %                            model per projectar la base de
            %                            casos del Problema.
            %
            %   varargin{2} (opcional) - Objecte DATA que contindra la base
            %                            de casos per al nostre CBR.
            %
            %   varargin{3} (opcional) - 'YES' o 'NO' indica si es vol
            %                            treure outliers, si no s'indica no
            %                            es treuran.
            %                            Un cop contestada la pregunta es
            %                            seguira el mateix comportament
            %                            mentres existeixi l'objecte.
            %                            Indicant '' o qualsevol altre
            %                            valor no reconegut es preguntara a
            %                            l'usuari que fer.
            %
            %Parametres de sortida:
            %
            %    obj - El nou objecte CBR
            %
            
            %inicialitzara un objecte CBR.
            if nargin == 0
                obj.Case_Base=[];%fem una copia avans de projectar per evitar problemes
                
                %inicialitzem tots els atributs
                obj.vk=[];
                obj.dc=[];
                obj.method=[];
                
                obj.block =0; %la base de casos no esta bloquejada
                
                obj.test = [];
                obj.veins=[];
                obj.classepredit=[];%encara no hem predit cap classe per aquestes dades
                obj.param=[];
            else
                
                model = varargin{1};%Recupero el model
                C_B = varargin{2};%i la Base de Casos
                
                if nargin == 2
                    resp = 'NO'; %per defecte no treurem outlaiers si no ens ho indiquen!
                else
                    resp = varargin{3};
                end
                z=whos('model');
                if(not(strcmp(z.class,'PCA')))%si el tipus es correcte
                    %ens han passat alguna cosa que no era del tipus correcte
                    error('CBR\CBR: model parameter must be an object of type PCA');
                end
                
                z=whos('C_B');
                if(not(strcmp(z.class,'DATA')))%si el tipus es correcte
                    %ens han passat alguna cosa que no era del tipus correcte
                    error('CBR\CBR: C_B parameter must be an object of type DATA');
                end
                
                obj.Model=model.copy();%fem una copia del model
                
                z=whos('resp');
                if strcmp(z.class,'char')
                    obj.treuoutliers = resp;
                else
                    error('CBR\CBR: resp parameter must be char type');
                end
                
                outs = obj.Model.getOutLiers();
                
                while(sum(outs) > 0)
                    if isempty(obj.treuoutliers)
                        obj.treuoutliers = questdlg('There are outliers do you want to remove them?','Attention!','YES','NO','PLOT','YES');
                    end
                    switch obj.treuoutliers
                        case ''
                            outs = 1;
                            obj.treuoutliers=[];
                        case 'YES'
                            obj.Model.deleteBatch(outs);
                            outs=obj.Model.getOutLiers();
                        case 'NO'
                            outs = 0;
                        case 'PLOT'
                            obj.Model.plotOutLiers();
                            outs = 1;
                            obj.treuoutliers=[];
                        otherwise
                            error('CBR\CBR: resp parameter must be char type and valid ones are: YES, NO, PLOT');
                    end
                    
                end
                
                
                
                %Projectem la base de casos!
                obj.Case_Base=obj.Model.project(C_B.copy());%fem una copia avans de projectar per evitar problemes
                
                %inicialitzem tots els atributs
                obj.vk=[];
                obj.dc=[];
                obj.method=[];
                
                obj.block =0; %la base de casos no esta bloquejada
                
                obj.test = [];
                obj.veins=[];
                obj.classepredit=[];%encara no hem predit cap classe per aquestes dades
                obj.param=[];
                
            end
        end

        function objo=copy(obj)
            %Metode constructor de Copia de la classe CBR. Creara un CBR
            %Copia de l'objecte actual.
            %
            %Parametres d'entrada:
            %
            %    no en te (obj es per defecte de la OO en matlab i es
            %    l'objecte actual) 
            %
            %Parametres de sortida:
            %
            %    objo - El nou objecte CBR
            %
            
            %creem un CBR buit
            objo = CBR();
            
            %inicialitzem tots els parametres fen una copia dels de
            %l'objecte actual
            objo.Model = obj.Model.copy();
            objo.Case_Base = obj.Case_Base.copy();
            
            objo.vk = obj.vk;
            objo.dc = obj.dc;
            objo.method = obj.method;
            
            objo.block = obj.block;
            
            objo.test = obj.test;
            objo.veins = obj.veins;
            objo.classepredit = obj.classepredit;
            
            objo.param = obj.param;
            
            objo.treuoutliers = obj.treuoutliers;
        end
        
        %getters
        function m=getCaseBase(obj)
            %metode que retorna la base de casos que s'esta fent servir en
            %aquest moment, la retoorna com a objecte de tipus PCA, tal i
            %com el guardem dins el nostre objecte CBR.
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   m - Retorna l'objecte PCA que fa de Case_Base al CBR
            %   actual.
            %
            m=obj.Case_Base;%Retornem les dades
        end
        function model=getModel(obj)
            %metode que retorna l'objecte PCA que fa de model en el nostre
            %CBR
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   model - Retorna El model del nostre CBR
            %
            model=obj.Model.copy;%retornem el model
        end
        function out = isOptimized(obj)
            %metode que retorna Un boolea que indica si el nostre Objecte
            %CBR ha estat optimitzat o no
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   out - Retorna cert si el CBR ha estat optimitzat i fals si
            %         no ha estat optimitzat
            %
            out=obj.block;
        end
        function out = getVK(obj)
            %metode que retorna el parametre VK actual del nostre objecte.
            %Per defecte el CBR si no se li indica la configuraci
            %utilitzar la ultima coneguda, aquest metode indica l'ultim VK
            %utilitzat.
            %
            %VK indica el nombre de veins que es seleccionen a cada un dels
            %nivells de distancia, depenent de la distancia fa falta 1 o 2
            %nivells. En cas de tenir indicat 2 nivells i nomes necesitar 1
            %s'utilitzara com a vk el minim dels 2 valors indicats.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   out - VK que s'utilitzaria en cas de no indicar-se
            %
            out = obj.vk;
        end
        function out = getDC(obj)
            %metode que retorna el parametre DC actual del nostre objecte.
            %Per defecte el CBR si no se li indica la configuraci
            %utilitzar la ultima coneguda, aquest metode indica l'ultim DC
            %utilitzat.
            %
            % DC indica el criteri de distancia que utilitzem per calcular
            % les distancies entre els casos.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   out - DC que s'utilitzaria en cas de no indicar-se
            %
           out=obj.dc; 
        end
        function out = getMethod(obj)
            %metode que retorna el parametre method actual del nostre
            %objecte. Per defecte el CBR si no se li indica la configuraci
            %utilitzar la ultima coneguda, aquest metode indica l'ultim
            %method utilitzat.
            %
            %Method indica el metode de voting que s'utilitzara per decidir
            %la classe d'un cas a partir dels seus veins mes propers.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   out - method que s'utilitzaria en cas de no indicar-se
            %
            out=obj.method;        
        end
        function out = getParam(obj)
            %metode que retorna el parametre param actual del nostre
            %objecte. Per defecte el CBR si no se li indica la configuraci
            %utilitzar la ultima coneguda, aquest metode indica l'ultim
            %param utilitzat.
            %
            %param s el parametre extre per les distancies 17 i 18, indica
            %el tan per 1 a partir del qual considerem 1. l'agrupaci de
            %contribucions.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   out - param que s'utilitzaria en cas de no indicar-se
            %
            out=obj.param;        
        end
        
        %Posar i treure casos
        function addCase(obj,dad,class,nom)
            %metode per afegir un cas a la nostre base de casos.
            %
            %Parametres d'entrada:
            %
            %   dad - matriu vidimencional de doubles amb les dades del
            %         cas. Les dimencions han de ser concistents amb les
            %         dels casos existens a la base de casos.
            %
            %   class - Double que ens indica la classe del cas
            %
            %   nom - String que indica l'identificador del lot. 
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            
            obj.Case_Base.addBatch(dad,class,nom);%l'afegim amb el metode de PCA per no haver de reprojectar
            
            %han modificat la base de casos si estavem a mig retreave els
            %resultats canviaran per tant obligarem que tornin a fer-lo
            obj.test = [];
            obj.veins=[];
            obj.classepredit=[];
        end
        function deleteCase(obj,i)
            %metode per eliminar un o diversos casos de la nostra base de
            %casos.
            %
            %Parametres d'entrada:
            %
            %   i - index que ens indica el lot o lots que bolem eliminar
            %       de la nostra base de casos
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            
            obj.Case_Base.deleteBatch(i);%el/s treiem amb el metode de PCA per no haver de reprojectar
            
            %han modificat la base de casos si estavem a mig retreave els
            %resultats canviaran per tant obligarem que tornin a fer-lo
            obj.test = [];
            obj.veins=[];
            obj.classepredit=[];
        end
        function [dad clas nom] = getCase(obj,i)
            %metode per recuperar les dades d'un o diversos casos de la
            %nostra base de casos.
            %
            %Parametres d'entrada:
            %
            %   i - index que ens indica el lot o lots que bolem recuperar
            %       de la nostra base de casos
            %
            %Parametres de sortida:
            %
            %   dad - Matriu vidimencional amb les dades del lot
            %
            %   clas - La classe del lot
            %
            %   nom - L'identificador del lot.
            %
            
            %simplement invoquem el metode de PCA
            [dad clas nom] = obj.Case_Base.getBatch(i);
        end
        
        %Posar i treure lots del model
        function addCaseToModel(obj,dad,class,nom)
            %Metode per afegir un cas al nostre model
            %
            %Parametres d'entrada:
            %
            %   dad - matriu vidimencional de doubles amb les dades del
            %         cas. Les dimencions han de ser concistents amb les
            %         dels casos existens al model.
            %
            %   class - Double que ens indica la classe del cas
            %
            %   nom - String que indica l'identificador del lot. 
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            
            if(obj.block == 0)
                obj.Model.addBatch(dad,class,nom);
                %han modificat el model si estavem a mig retreave els
                %resultats canviaran per tant obligarem que tornin a fer-lo
                obj.test = [];
                obj.veins=[];
                obj.classepredit=[];
                
                %Projectem la base de casos de nou
                obj.Case_Base=obj.Model.project(obj.Case_Base.getDATA());
                
                %Afegint casos pot ser que posem outliers!
                outs = obj.Model.getOutLiers();
                
                while(sum(outs) > 0)
                    if isempty(obj.treuoutliers)
                        obj.treuoutliers = questdlg('There are outliers do you want to remove them?','Attention!','YES','NO','PLOT','YES');
                    end
                    switch obj.treuoutliers
                        case ''
                            outs = 1;
                            obj.treuoutliers=[];
                        case 'YES'
                            obj.Model.deleteBatch(outs);
                            outs=obj.Model.getOutLiers();
                        case 'NO'
                            outs = 1;
                            obj.treuoutliers=[];
                        case 'PLOT'
                            obj.Model.plotOutLiers();
                    end
                    
                end
                
            else
                error('CBR\addCaseToModel: You can''t change your model if you made an optimitzation .');
            end
        end
        function deleteCaseFromModel(obj,i)
            %metode per eliminar un o diversos casos del nostra model
            %
            %Parametres d'entrada:
            %
            %   i - index que ens indica el lot o lots que bolem eliminar
            %       del nostre model
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            if(obj.block == 0)
                obj.Model.deleteBatch(i);
                
                %han modificat el model si estavem a mig retreave els
                %resultats canviaran per tant obligarem que tornin a fer-lo
                obj.test = [];
                obj.veins=[];
                obj.classepredit=[];
                
                %Projectem la base de casos de nou
                obj.Case_Base=obj.Model.project(obj.Case_Base.getDATA());
                
                outs = obj.Model.getOutLiers();
                
                while(sum(outs) > 0)
                    if isempty(obj.treuoutliers)
                        obj.treuoutliers = questdlg('There are outliers do you want to remove them?','Attention!','YES','NO','PLOT','YES');
                    end
                    switch obj.treuoutliers
                        case ''
                            outs = 1;
                            obj.treuoutliers=[];
                        case 'YES'
                            obj.Model.deleteBatch(outs);
                            outs=obj.Model.getOutLiers();
                        case 'NO'
                            outs = 0;
                        case 'PLOT'
                            obj.Model.plotOutLiers();
                            outs = 1;
                            obj.treuoutliers=[];
                    end
                    
                end
            else
                error('CBR\addCaseToModel: You can''t change your model if you made an optimitzation .');
            end
            
        end
        
        
        %Retreave, reuse, revise i retain (control de parametres i retorns)
        function [index Test_set]= retrieve(obj,dades,ivk,idc,iparam)
            %metode de retrieve, s'encarrega de seleccinar els casos ms
            %propers al cas/ casos indicats a dades. ivk indicara quans
            %casos volem recuperar i idc la distancia a utilitzar iparam es
            %per les distancies 17 i 18.
            %
            %Parametres d'entrada:
            %
            %    dades - Objecte DATA amb casos als quals buscarem veins a
            %            la nostre base de casos.
            %
            %    ivk (opcional si ja s'ha indicat previament) 
            %            indica el nombre de veins a recuperar en cada un
            %            dels nivells de la distancia.
            %
            %    idc (opcional si ja s'ha indicat previament)
            %            indica el metode de distancia que s'utilitzara per
            %            calcular les distancies.
            %
            %    iparam (opcional si ja s'ha indicat previament)
            %            Parametre auxiliar per les distancies 17 i 18 que
            %            indicara el tan per 1 de casos que es necesiten
            %            per considerar un grup de contribucions com a
            %            falla (1) o no (0).
            %
            %Parametres de sortida:
            %
            %   index - l'index dels casos mes propers. 
            %
            %   Test_set - L'objecte dades projectat a sobre el model del
            %              problema 
            %
            %
            %Apunts D'optimitzaci de codi:
            %
            %      - Si no es necesiten els parametres de sortida no
            %        demanar-los i es podran evitar calculs. 
            %
            %      - A l'hora de fer retrieve de diversos casos es ms
            %        eficient fer-los de cop.
            %
            
            %el parametre dades es sempre obligatori i ha de ser un objecte
            %DATA si no retornem error.
            z=whos('dades');
            if(not(strcmp(z.class,'DATA')))%comprovem que sigui del tipus correcte
                %ens han passat unes dades de tipus incorrecte.
                error('CBR\retrieve: dades parameter must be an object of type DATA');
            end
            
            %la resta de parametres son opcionals (sempre que els hagin entrat algun cop)
            if nargin >2
                %Comprobem que el ivk que ens passen es del tipus correcte
                z=whos('ivk');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\retrieve: The parameter ivk must be of type Double');
                end
                %ara comprobem que no s'hagi bloquejat la base de casos
                if (obj.block ==1) && sum(obj.vk ~= ivk)~=0
                    error('CBR\retrieve: vk parameter must not be changed when the case base has been optimited. Remember, is not necesary to call the method retrieve with all the parameters.');
                else
                    obj.vk=ivk;
                end
                
                %comprobem el parametre dc
                z=whos('idc');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\retrieve: dc parameter must be of type Double');
                end
                if (obj.block ==1) && (obj.dc~=idc)
                    error('CBR\retrieve: dc parameter must not be changed when the case base has been optimized. Remember, is not necesary to call the method retrieve with all the parameters.');
                else
                    obj.dc=idc;
                end
                
                if obj.dc ==17 || obj.dc == 18
                    if obj.block == 1 && obj.param~=iparam
                        error('CBR\reuse: This parameter must not be changed when the case base has been optimzed. Remember that is not necessary to call the method reuse with all the parameters.');
                    else
                        if exist('iparam','var')
                            obj.param=iparam;
                        else
                            error('CBR\reuse:This distance needs an extra parameter!');
                        end
                    end
                end
            end
            %comprovem que els hagin entrat algun cop
            if isempty(obj.vk)
                error('CBR\retrieve: vk parameter never had a value. Remember, you cannot use the short form.');
            end
            if isempty(obj.dc)
                error('CBR\retrieve: dc parameter never had a value. Remember, you cannot use the short form.');
            end
            
            %mirem si hi ha el parametre extra de la distancia
            if obj.dc ==17 || obj.dc == 18
                if isempty(obj.param)
                    %no s'ha fet un retrieve
                    error('CBR\reuse:This distance needs an extra parameter!');
                end
            end
            
            obj.classepredit=[];%encara no hem predit cap classe per aquestes dades
            
            %segons que ens demani l'usuari farem uns calculs o uns altres
            if nargout ~= 0
                %Bolen parametres de sortida!!
                obj.test = obj.Model.project(dades);%projectem les dades que ens passen al nostre model.
                Test_set = obj.test;%preparem el retorn per l'usuari
                
                %si ens demanen mes elements dels que tenim, retornem tots els
                %que poguem.
                obj.vk(obj.vk>obj.Case_Base.getBatchCount())=obj.Case_Base.getBatchCount();%les components del vector que siguin mes grans que el nombre de lots que tenim passen a ser el nombre de lots.
                
                CBR_retrieve(obj);%cridem el retreave privat (el que fara realment el retreave)
                
                %Preparem el retorn per l'usuari on:
                %dades son els "lots" veins
                %class son les classes d'aquests veins
                %noms son els noms d'aquests veins
 %               [dades class noms]=obj.Case_Base.getBatch(obj.veins.indexs_k2);%Per poder indexar sobre dades
                %es ms util disposar se l'index del lot vei k del lot!!
                index = obj.veins.indexs_k2;
            else
                %no hi ha retorn
                obj.test = obj.Model.project(dades);%projectem les dades que ens passen al nostre model.
                
                %si ens demanen mes elements dels que tenim, retornem tots els
                %que poguem.
                obj.vk(obj.vk>obj.Case_Base.getBatchCount())=obj.Case_Base.getBatchCount();%les components del vector que siguin mes grans que el nombre de lots que tenim passen a ser el nombre de lots.
                
                CBR_retrieve(obj);%cridem el retreave privat
            end
            
        end
        function classe = reuse(obj,imethod)
            %Metode per fer un reuse sobre la base de casos actual. Per
            %poder utilitzar aquest metode s'ha d'haver fet avans un
            %retrieve.
            %
            %Parametres d'entrada:
            %
            %
            %   imethod (opcional si ja s'ha indicat previament) 
            %             Metode que s'utilitzara per escollir la nova
            %             classe dels casos entrats a dades de la funcio
            %             retrieve
            %
            %Parametres de sortida:
            %
            %   classe - Classe assignada a cadascun dels lots de l'objecte
            %            dades sobre el qual s'ha fet l'ultim retrieve
            %
            %
            %Apunts D'optimitzaci de codi:
            %
            %      - Si no es necesiten els parametres de sortida no
            %        demanar-los i es podran evitar calculs. 
            %
            
            %el parametremethod es opcional
            if nargin > 1
                %comprobem correctesa de les dades
                z=whos('imethod');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\reuse: imethod parameter must be of type Double.');
                end
                if obj.block ==1 && obj.method~=imethod
                    error('CBR\reuse: method parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method reuse with all the parameters.');
                else
                    obj.method=imethod;
                end
                
            end
            
            %comprovem que l'hagin entrat algun cop
            if isempty(obj.method)
                %no s'ha fet un retrieve
                error('CBR\retrieve: method parameter never had a value. Remember, you can not use the short form.');
            end
            
            %mirem si tenim dades d'un retrieve
            if isempty(obj.test)
                %no s'ha fet un retrieve
                error('CBR\reuse:You can''t use reuse. Use retrieve first!!');
            end
            
            
            
            %cridem el rehuse privat
            CBR_reuse(obj);
            
            %retornem les classes si les bolen
            if nargout >0
                classe=obj.classepredit;
            end
        end
        function [matri fallen]=revise(obj)
            %metode perque l'usuari comprovi si les dades s'han classificat
            %be o no.
            %Parametres d'entrada:
            %
            %   No en te, Cal que els casos entrats a la funci retrieve
            %   tinguin classe asignada per poder fer automaticament aquest
            %   pas. Si no tenen classes es demanara a l'usuari que les
            %   proporcioni mitjanant un dialeg.
            %
            %Parametres de sortida:
            %
            %   matri - matriu de confusio
            %   fallen - indexs dels casos que fallen
            %
            
            %fem les comprovacions necessaries per assegurar que podem fer
            %un revise.
            if isempty(obj.test)
                %no han fet el retrieve
                error('CBR\revise: You can''t use revise. Use retrieve first!');
            end
            if isempty(obj.classepredit)
                %no han fet el reuse
                error('CBR\revise: You can''t use revise. Use reuse first!');
            end
            
            
            %cridem al revise intern
            [matri fallen]=CBR_revise(obj);
        end
        function retain(obj,fallen)
            %metode que rete tots els casos que han fallat en la seva
            %classigicaci, l'usuari ens ha d'indicar un vector amb els
            %casos que han fallat o no
            %
            %Parametres d'entrada:
            %
            %       fallen  - vector de logicals que indica si el CBR ha
            %                 fallat en classficar el cas o no
            %
            %Parametres de sortida:
            %       No en te
            %
            
            obj.CBR_retain(fallen);
        end
        
        %Metodes per optimitzar la base de casos (Control de parametres)
        function IBUdG(obj,nelem,descarta,ivk,idc,imethod,iparam)
            %Mtode per netejar la base de casos. Aquest mtode intentara
            %definir les fronteres de les classes i despoblar els centres.
            %Per fer-ho, es busca el cas amb una classe diferent ms proper
            %a cada cas de la base de casos, anomenat enemic. Lenemic
            %safegeix a la base de casos optimitzada, sempre i quant no
            %estiguin ja inclosos a la base de casos.
            %
            %Tal com succea en altres mtodes, els parmetres vk,dc i
            %"method" no sn necessaris si sha cridat amb anterioritat el
            %mtode amb tots els parmetres dentrada. En aquest cas,
            %prendrem lltim valor entrant dels parmetres
            %
            %El parmetre descarta serveix per separar les fronteres,
            %en problemes on el soroll s considerable aquest mtode
            %tendeix a afegir masses casos a les fronteres. Per corregir
            %aquest defecte proposem descartar els X enemics ms propers de
            %cada cas on X s el valor que es dona a descarta.
            %
            %Finalment, noms indicar que el nombre denemics s crtic a
            %lhora de filtrar la base de casos. El valor recomanat s el
            %nombre mnim delements que es recuperen per un nou problema.
            %Si triem un nombre denemics inferior a aquest valor,
            %obtindrem una base de casos menor, per que al mateix temps,
            %representa un major nombre de classificacions incorrectes. Per
            %altra banda, si el nombre denemics s superior al nombre
            %mnim de casos que recuperem, llavors tindrem una base de
            %casos ms gran, per que al mateix temps, tindr un nombre de
            %classificacions incorrectes ms baix.
            %
            %
            %Parametres d'entrada:
            %
            %       (Parametres opcionals si ja s'han indicat anteriorment)
            %
            %       ivk - Tupla que indica el nombre de vens a retenir a
            %             cada nivell del clcul de distncia.
            %
            %       idc - Double que indica que indica el criteri per
            %             calcular la distncia entre 2 casos.
            %
            %       imethod - Double que indica el mtode a utilitzar per
            %                 assignar la classe als nous problemes.
            %
            %       iparam - Parametre per a les distancies 17 i 18
            %
            %
            %       (Parametres obligatoris)
            %
            %       nelem - Double que indica el nombre denemics a afegir
            %               per cada cas de la base de casos.
            %
            %       descarta - Double que indica quants dels enemics ms
            %                  propers shan dignorar (no afegir a la base
            %                  de casos)
            %
            %Parametres de sortida:
            %
            %       No en te
            %


            if nargin >3
                z=whos('ivk');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IBUdG: vk parameter must be of type Double.');
                end
                if (obj.block ==1) && sum(obj.vk ~= ivk)~=0
                    error('CBR\IBUdG: vk parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.vk=ivk;
                end
                %comprovar mides double o [double double] ??
                z=whos('idc');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IBUdG: dc parameter must be of type Double.');
                end
                if obj.block ==1 && obj.dc~=idc
                    error('CBR\IBUdG: dc parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.dc=idc;
                end
               
                z=whos('imethod');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IBUdG: method parameter must be of type Double.');
                end
                if obj.block ==1 && obj.method~=imethod
                    error('CBR\IBUdG: method parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.method=imethod;
                end
                
                if obj.dc ==17 || obj.dc == 18
                    if obj.block == 1 && obj.param~=iparam
                        error('CBR\IBUdG: This parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method reuse with all the parameters.');
                    else
                        if exist('iparam','var')
                            obj.param=iparam;
                        else
                            error('CBR\IBUdG:This distance needs an extra parameter!');
                        end
                    end
                end
            end
            
            %mirem si hi ha el parametre extra de la distancia
            if obj.dc ==17 || obj.dc == 18
                if isempty(obj.param)
                    %no s'ha fet un retrieve
                    error('CBR\IBUdG:This distance needs an extra parameter!');
                end
            end
            
            if isempty(obj.vk) || isempty(obj.dc)  || isempty(obj.method)
                error('CBR\IBUdG: At least 1 of them (vk , dc , method) never had a value. You can''t use the short form.');
            end
            
            obj.block=1;%ja no es poden canviar els parametres
            Calcula_IB_UdG(obj,nelem,descarta);
        end
        function IB3(obj,ivk,idc,imethod,iparam)
            %Mtode per netejar la base de casos proposat per David W. Aha.
            %Aquest mtode intenta definir les fronteres de les classes
            %lo ms allunyades possible. La frontera entre 2 classes s la
            %zona de lespai de cerca del problema en que dues classes
            %presenten una gran confluncia. 
            %             
            % Algorisme base proposat per  David W. Aha.
            % The IB3 algorithm (CD = Concept Description)
            %   CD = []
            %   For each x in TrainingSet do
            %      For each y in CD do
            %         Sim[y] = Similarity(x, y)
            %      end
            %      if Exists y in CD   acceptable(y) then
            %         ymax   some acceptable y in CD with maximal Sim[y]
            %      else
            %         i   randomly selected value in CD
            %         ymax   some y in CD that is the i th most similar instance to x
            %      end
            %      if class(x)   class(ymax) then
            %         classification - correct
            %      else
            %         classification - incorrect
            %         CD - CD U [x]
            %      end
            %      For each y in CD do
            %         if Sim[y]   Sim[ymax] then
            %            Update ys classification record
            %            if ys record is significantly poor then
            %               CD - C   [y]
            %            end
            %         end
            %      end
            %   end 
            %
            %Parametres d'entrada:
            %
            %       (Parametres opcionals si ja s'han indicat anteriorment)
            %
            %       ivk - Tupla que indica el nombre de vens a retenir a
            %             cada nivell del clcul de distncia.
            %
            %       idc - Double que indica que indica el criteri per
            %             calcular la distncia entre 2 casos.
            %
            %       imethod - Double que indica el mtode a utilitzar per
            %                 assignar la classe als nous problemes.
            %
            %       iparam - Parametre per a les distancies 17 i 18
            %
            %
            %
            %Parametres de sortida:
            %
            %       No en te
            %

            if nargin >1
                z=whos('ivk');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IB3: vk parameter must be of type Double.');
                end
                if (obj.block ==1) && sum(obj.vk ~= ivk)~=0
                    error('CBR\IB3: vk parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.vk=ivk;
                end
                %comprovar mides double o [double double] ??
                z=whos('idc');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IB3: dc parameter must be of type Double.');
                end
                if obj.block ==1 && obj.dc~=idc
                    error('CBR\IB3: dc parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.dc=idc;
                end
                
                z=whos('imethod');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IB3: method parameter must be of type Double.');
                end
                if obj.block ==1 && obj.method~=imethod
                    error('CBR\IB3: method parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.method=imethod;
                end
                
               if obj.dc ==17 || obj.dc == 18
                    if obj.block == 1 && obj.param~=iparam
                        error('CBR\IB3: This parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method reuse with all the parameters.');
                    else
                        if exist('iparam','var')
                            obj.param=iparam;
                        else
                            error('CBR\IB3:This distance needs an extra parameter!');
                        end
                    end
                end
            end
            
            if isempty(obj.vk) || isempty(obj.dc)  || isempty(obj.method)
                error('CBR\IB3: At least 1 of them (vk , dc , method) never had a value. You can''t use the short form.');
            end
            
            %mirem si hi ha el parametre extra de la distancia
            if obj.dc ==17 || obj.dc == 18
                if isempty(obj.param)
                    %no s'ha fet un retrieve
                    error('CBR\IB2:This distance needs an extra parameter!');
                end
            end
            obj.block=1;%ja no es poden canviar els parametres
            
            Calcula_IB3(obj)
        end
        function IB2(obj,ivk,idc,imethod,iparam)
            %Mtode per netejar la base de casos proposat per David W. Aha.
            %
            % IB2 s un algorisme incremental, va provant de classificar
            % casos, en cas que es falli en classificar-lo el cas ser
            % afegit a la base de casos, si per contra el cas es classifica
            % be no safegeix.
            % 
            % Algorisme base proposat per  David W. Aha.
            % The IB2 algorithm (CD = ConceptDescription)
            %   CD = []
            %   for each x in Training Set do
            %     for each y in CD do
            %       Sim[y] = Similarity(x, y)
            %     end
            %     ymax = Max(Sim[])
            %     if class(x) = class(ymax)
            %       classification - correct
            %     else
            %       classification - incorrect
            %        CD = CD U [x]
            %     end
            %   end  
            %
            %
            %Parametres d'entrada:
            %
            %       (Parametres opcionals si ja s'han indicat anteriorment)
            %
            %       ivk - Tupla que indica el nombre de vens a retenir a
            %             cada nivell del clcul de distncia.
            %
            %       idc - Double que indica que indica el criteri per
            %             calcular la distncia entre 2 casos.
            %
            %       imethod - Double que indica el mtode a utilitzar per
            %                 assignar la classe als nous problemes.
            %
            %       iparam - Parametre per a les distancies 17 i 18
            %
            %
            %
            %Parametres de sortida:
            %
            %       No en te
            %
            
            if nargin >1
                z=whos('ivk');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IB2: vk parameter must be of type Double.');
                end
                if (obj.block ==1) && sum(obj.vk ~= ivk)~=0
                    error('CBR\IB2: vk parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.vk=ivk;
                end
                %comprovar mides double o [double double] ??
                z=whos('idc');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IB2: dc parameter must be of type Double.');
                end
                if obj.block ==1 && obj.dc~=idc
                    error('CBR\IB2: dc parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.dc=idc;
                end
                z=whos('imethod');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\IB2: method parameter must be of type Double.');
                end
                if obj.block ==1 && obj.method~=imethod
                    error('CBR\IB2: method parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.method=imethod;
                end
                
                if obj.dc ==17 || obj.dc == 18
                    if obj.block == 1 && obj.param~=iparam
                        error('CBR\IB2: This parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method reuse with all the parameters.');
                    else
                        if exist('iparam','var')
                            obj.param=iparam;
                        else
                            error('CBR\IB2:This distance needs an extra parameter!');
                        end
                    end
                end
            end
            
            if isempty(obj.vk) || isempty(obj.dc)  || isempty(obj.method)
                error('CBR\IB2: At least 1 of them (vk , dc , method) never had a value. You can''t use the short form.');
            end
            %mirem si hi ha el parametre extra de la distancia
            if obj.dc ==17 || obj.dc == 18
                if isempty(obj.param)
                    %no s'ha fet un retrieve
                    error('CBR\IB2:This distance needs an extra parameter!');
                end
            end
            
            Calcula_IB2(obj);
            obj.block=1;%ja no es poden canviar els parametres
        end
        function DROP4(obj,ivk,idc,imethod,iparam)
            %Mtode per netejar la base de casos proposat per Wilson i
            %Martinez. s un procs decremental, comena amb la base de
            %casos completa i en treu els que no sn necessaris. A
            %continuaci tenim lalgorisme original proposat per Wilson i 
            %Martinez, el qual sha adaptat per tal de treballar de la
            %forma ms eficient possible sobre Matlab.
            % 
            %
            %DROP4(Training Set = T): Instance set S
            %   Let SI = T
            %   //inicialize the lists of neighbours and associates
            %   For each instance i in S
            %     Make sure we know the neighbors of i in S
            %     Add i to each of its neihbors lists of associates
            %  //Do careful noise-filtering pass  For each instance i in S
            %    if i is misclassified by its neighbors
            %      removeifnothelping(i,S)
            %  //Do more aggresive reduction pass  Sort instances by distance to nearest enemy (furthest ones first)
            %  For each instance i in S(starting with those furthest from their nearest enemy)
            %  removeifnothelping(i,S)
            %  Return S
            %
            %Parametres d'entrada:
            %
            %       (Parametres opcionals si ja s'han indicat anteriorment)
            %
            %       ivk - Tupla que indica el nombre de vens a retenir a
            %             cada nivell del clcul de distncia.
            %
            %       idc - Double que indica que indica el criteri per
            %             calcular la distncia entre 2 casos.
            %
            %       imethod - Double que indica el mtode a utilitzar per
            %                 assignar la classe als nous problemes.
            %
            %       iparam - Parametre per a les distancies 17 i 18
            %
            %
            %
            %Parametres de sortida:
            %
            %       No en te
            %
            
            if nargin >1
                z=whos('ivk');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\DROP4: vk parameter must be of type Double.');
                end
                if (obj.block ==1) && sum(obj.vk ~= ivk)~=0
                    error('CBR\DROP4: vk parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.vk=ivk;
                end
                %comprovar mides double o [double double] ??
                z=whos('idc');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\DROP4: dc parameter must be of type Double.');
                end
                if obj.block ==1 && obj.dc~=idc
                    error('CBR\DROP4: dc parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.dc=idc;
                end
                z=whos('imethod');
                if(not(strcmp(z.class,'double')))%comprovem que sigui del tipus correcte
                    %ens han passat unes dades de tipus incorrecte.
                    error('CBR\DROP4: method parameter must be of type Double.');
                end
                if obj.block ==1 && obj.method~=imethod
                    error('CBR\DROP4: method parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method IBUdG with all the parameters, you can use the short form.');
                else
                    obj.method=imethod;
                end
                
                if obj.dc ==17 || obj.dc == 18
                    if obj.block == 1 && obj.param~=iparam
                        error('CBR\DROP4: This parameter must not be changed when the case base has been optimited. Remember that is not necessary to call the method reuse with all the parameters.');
                    else
                        if exist('iparam','var')
                            obj.param=iparam;
                        else
                            error('CBR\DROP4:This distance needs an extra parameter!');
                        end
                    end
                end
            end
            
            if isempty(obj.vk) || isempty(obj.dc) || isempty(obj.method)
                error('CBR\DROP4: At least 1 of them (vk , dc , method) never had a value. You can''t use the short form.');
            end
            
                        %mirem si hi ha el parametre extra de la distancia
            if obj.dc ==17 || obj.dc == 18
                if isempty(obj.param)
                    %no s'ha fet un retrieve
                    error('CBR\DROP4:This distance needs an extra parameter!');
                end
            end
            obj.block=1;%ja no es poden canviar els parametres

            ordena(obj);
            calcula_DROP4(obj);
        end
        
%-------------------------------------------------------------------------%
%% -------------------- Visualitzaci de l'objecte -----------------------
%-------------------------------------------------------------------------%    

        %que fer quan fan doble click subre un objecte
        function openvar(nom,~) %l'imput ignorat es l'objecte actual!
            %Aquesta es una funci que sobreescriu l'event per defecte de
            %doble click sobre un objecte de tipus CBR en el workspace.
            %sobreescrivim la funci de visualitzaci per defecte,
            %el parametre ignorat s l'objecte actual el nom ens el passen
            %en format text. Aqui invocarem una pantalla especifica per
            %visualitzar l'objecte.
            %
            %!!ATENCIO!! -> Aquest metode no hauria de ser invocat per
            %l'usuari directament a menys que sapiga molt b que esta fent!
            
            x=whos('nom');
            if strcmp(x.class,'CBR')
                %no puc saber el nom de la variable -> no fem res!
            else
                if(evalin('base',['size(' nom ');']) == [1 1])
                    Menu_CBR(nom)
                 end
            end
        end
        %com es mostra a consola
        function disp(obj)
            %funci que sobreescriu la visualitzaci de l'objecte per
            %consola i en alguns menus desplegables de matlab.
            %Mostrarem informaci util per l'usuari (tamany de les dades)
            %en comptes de la informaci per defecte.
            %
            % Aquesta funci s'invoca automaticament per Matlab quan cal
            % l'usuari no necesita invocarla en cap moment.
            
            %fem els displays que faci falta
            disp('Model')
            obj.Model.disp()
            
            disp(' ')
            
            disp('Case Base')
            obj.Case_Base.disp()
            
        end
        
        function s = saveobj(obj)
            %Sobreescripture del metode Save de matlab perque funcioni
            %correctament amb objectes de tipus CBR.
            %
            % Parametres de sortida:
            % s - l'estructura que guardarem en memoria
            
            %retorna l'objecte en format Structura
            s.Model =  obj.Model;
            s.Case_Base =  obj.Case_Base;
            
            s.vk =  obj.vk;
            s.dc =  obj.dc;
            s.method =  obj.method;
            
            s.block =  obj.block;
            
            s.test =  obj.test;
            s.veins =  obj.veins;
            s.classepredit =  obj.classepredit;
            
            s.param =  obj.param;
            
            s.treuoutliers =  obj.treuoutliers;
        end
    end
    
    methods (Static)
        function obj = loadobj(obj)
            %Sobre escriptura del metode load de matlab perque funcioni
            %correctament amb objectes de tipus CBR.
            %
            %Parametres 
            % obj - entra com a structura i surt com a objecte CBR
            
            if isstruct(obj)
                % Call default constructor
                newObj = CBR();
                % Assign property values from struct
                newObj.Model =  obj.Model;
                newObj.Case_Base =  obj.Case_Base;
                
                newObj.vk =  obj.vk;
                newObj.dc =  obj.dc;
                newObj.method =  obj.method;
                
                newObj.block =  obj.block;
                
                newObj.test =  obj.test;
                newObj.veins =  obj.veins;
                newObj.classepredit =  obj.classepredit;
                
                newObj.param =  obj.param;
                
                newObj.treuoutliers =  obj.treuoutliers;
                obj = newObj;
            end
        end
    end
    
    methods (Access = private)
        
        function CBR_retrieve(obj)
            %Calculs del metode retrieve
            
            switch (obj.dc)
                case 1 % Distncia sobre les components principals
                    % Inicialitzacions
                    testgT=obj.test.getT();
                    CBgT=obj.Case_Base.getT();
                   
                    
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    distancia = zeros(size(CBgT,1),size(testgT,1));
                   
                     rep = repmat(obj.Model.getLambda()',size(CBgT,1),1);
                    % Clcul de les distncies
                    for i = 1:size(testgT,1)
                        distancia(:,i) = sqrt(sum(((CBgT - repmat(testgT(i,:),size(CBgT,1),1) ).^2 ./ rep),2));
                    end
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;

                case 2 % Distncia sobre T^2
                    % Inicialitzacions
                    tegT2=obj.test.getT2();
                    CBgT2=obj.Case_Base.getT2();
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    
                    distancia = zeros(length(CBgT2),length(tegT2));
                    % Clcul de les distncies
                    for i = 1:length(tegT2)
                        distancia(:,i) = abs(CBgT2 - repmat(tegT2(i),length(CBgT2),1));
                    end
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 3 % Distncia sobre Q
                    % Inicialitzacions
                    tegQ=obj.test.getQ();
                    CBgQ=obj.Case_Base.getQ;
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    distancia = zeros(length(CBgQ),length(tegQ));
                    % Clcul de les distncies
                    for i = 1:length(tegQ)
                        distancia(:,i) = abs(CBgQ - repmat(tegQ(i),length(CBgQ),1));
                    end
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 4 % Distncia combinada sobre Q i T^2
                    % Inicialitzacions
                    tegQ=obj.test.getQ();
                    CBgQ=obj.Case_Base.getQ;
                    tegT2=obj.test.getT2();
                    CBgT2=obj.Case_Base.getT2();
                    
                    % Inicialitzem el valor de k1 i k2
                    k1 = obj.vk(1); k2 = obj.vk(2);
                    if k2 > k1
                        k2=k1;
                    end
                    % Inicialitzem els vectors on es guardaran les distncies
                    distancia1 = zeros(length(CBgQ),length(tegQ));
                    distancia2 = zeros(k1,length(tegQ));
                    % Clcul del primer nivell de distncies (Q)
                    for i = 1:length(tegQ)
                        distancia1(:,i) = abs(CBgQ - repmat(tegQ(i),length(CBgQ),1));
                    end
                    % Ordenaci dels casos segons la primera distncia
                    [ordenats_k1,indexs_k1] = sort(distancia1,1);
                    % Selecci dels vens ms propers del primer nivell
                    obj.veins.ordenats_k1 = ordenats_k1(1:k1,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k1,:);
                    
                    % Clcul del segon nivell de distncies (T^2)
                    for i = 1:length(tegT2)
                        casos_restants = CBgT2(indexs_k1(1:k1,i));
                        distancia2(:,i) = abs(casos_restants - repmat(tegT2(i),k1,1));
                    end
                    % Ordenaci dels casos segons la segona distncia
                    [ordenats_k2,indexs_k2] = sort(distancia2,1);
                    % Selecci dels vens ms propers del segon nivell
                    obj.veins.indexs_k2 = zeros(k2,length(tegT2));
                    for i = 1:length(tegT2)
                        obj.veins.indexs_k2(:,i) = indexs_k1(indexs_k2(1:k2,i),i);
                    end
                    obj.veins.ordenats_k2 = ordenats_k2(1:k2,:);
                    
                case 5 % Distncia combinada sobre Q i les principals components
                    % Inicialitzacions
                    CBgQ=obj.Case_Base.getQ;
                    CBgT=obj.Case_Base.getT();
                    tegT=obj.test.getT;
                    tegQ=obj.test.getQ();
                    % Inicialitzem el valor de k1 i k2
                    k1 = obj.vk(1); k2 = obj.vk(2);
                    if k2 > k1
                        k2=k1;
                    end
                    % Inicialitzem els vectors on es guardaran les distncies
                    distancia1 = zeros(length(CBgQ),length(tegQ));
                    distancia2 = zeros(k1,size(tegT,1));
                    % Clcul del primer nivell de distncies (Q)
                    for i = 1:length(tegQ)
                        distancia1(:,i) = abs(CBgQ - repmat(tegQ(i),length(CBgQ),1));
                    end
                    % Ordenaci dels casos segons la primera distncia
                    [ordenats_k1,indexs_k1] = sort(distancia1,1);
                    % Selecci dels vens ms propers del primer nivell
                    obj.veins.ordenats_k1 = ordenats_k1(1:k1,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k1,:);
                    % Clcul del segon nivell de distncies (components principals)
                    rep=repmat(obj.Model.getLambda()',k1,1);
                    for i = 1:size(tegT,1)
                        casos_restants = CBgT(indexs_k1(1:k1,i),:);
                        distancia2(:,i) = sqrt(sum(((casos_restants - repmat(tegT(i,:),k1,1)).^2) ./ rep ,2));
                    end
                    % Ordenaci dels casos segons la segona distncia
                    [ordenats_k2,indexs_k2] = sort(distancia2,1);
                    % Selecci dels vens ms propers del segon nivell
                    obj.veins.indexs_k2 = zeros(k2,length(tegQ));
                    for i = 1:size(tegT,1)
                        obj.veins.indexs_k2(:,i) = indexs_k1(indexs_k2(1:k2,i),i);
                    end
                    obj.veins.ordenats_k2 = ordenats_k2(1:k2,:);
                    
                case 6 % Distncia combinada sobre T^2 i les principals components
                    % Inicialitzacions
                    CBgT2=obj.Case_Base.getT2;
                    tegT2=obj.test.getT2;
                    % Inicialitzem el valor de k1 i k2
                    k1 = obj.vk(1); k2 = obj.vk(2);
                    if k2 > k1
                        k2=k1;
                    end
                    % Inicialitzem els vectors on es guardaran les distncies
                    distancia1 = zeros(length(CBgT2),length(tegT2));
                    distancia2 = zeros(k1,size(tegT2,1));
                    % Clcul del primer nivell de distncies (T^2)
                    for i = 1:length(tegT2)
                        distancia1(:,i) = abs(CBgT2 - repmat(tegT2(i),length(CBgT2),1));
                    end
                    % Ordenaci dels casos segons la primera distncia
                    [ordenats_k1,indexs_k1] = sort(distancia1,1);
                    % Selecci dels vens ms propers del primer nivell
                    obj.veins.ordenats_k1 = ordenats_k1(1:k1,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k1,:);
                    % Clcul del segon nivell de distncies (components principals)
                    CBgT=obj.Case_Base.getT();
                    tegT=obj.test.getT;
                    
                    aux=CBgT(indexs_k1,:);
                    rep=repmat(obj.Model.getLambda()',k1,1);
                    for i = 1:size(tegT,1)
                        casos_restants = aux(indexs_k1(1:k1,i),:);
                        distancia2(:,i) = sqrt(sum((casos_restants - repmat(tegT(i,:),k1,1)).^2 ./ rep ,2) );
                    end
                    % Ordenaci dels casos segons la segona distncia
                    [ordenats_k2,indexs_k2] = sort(distancia2,1);
                    % Selecci dels vens ms propers del segon nivell
                    obj.veins.indexs_k2 = zeros(k2,length(tegT2));
                    for i = 1:size(tegT,1)
                        obj.veins.indexs_k2(:,i) = indexs_k1(indexs_k2(1:k2,i),i);
                    end
                    obj.veins.ordenats_k2 = ordenats_k2(1:k2,:);
                    
                case 7 % Distncia sobre Contribucions de Q en distancia euclidiana
                    % Inicialitzacions
                    tegcQ=obj.test.getQContribution();
                    CBgcQ=obj.Case_Base.getQContribution();
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                    % Clcul de les distncies
                    for i = 1:size(tegcQ,1)
                        distancia(:,i) = sqrt( sum( (CBgcQ - repmat(tegcQ(i,:),[d1 1]) ).^2, 2));
                    end
                    
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 8 % Distncia sobre Contribucions de T^2 en distancia euclidiana
                    % Inicialitzacions
                    tegcT2=obj.test.getT2Contribution();
                    CBgcT2=obj.Case_Base.getT2Contribution();
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcT2,1);
                    distancia = zeros(d1,size(tegcT2,1));
                    
                    % Clcul de les distncies
                    for i = 1:size(tegcT2,1)
                        distancia(:,i) = sqrt( sum( (CBgcT2 - repmat(tegcT2(i,:),[d1 1]) ).^2, 2));
                    end
                    
                    
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 9 % Distncia sobre Contribucions de Q en distancia mahalanobis (STD)
                    % Inicialitzacions
                    tegcQ=obj.test.getQContribution();
                    CBgcQ=obj.Case_Base.getQContribution();
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                    %si Tenim nomes un cas a la base de casos da STD es 0!!
                    if(obj.Case_Base.getBatchCount()<2)
                        %no podem fer mahalanobis, farem distancia
                        %euclidiana normal
                        for i = 1:size(tegcQ,1)
                            distancia(:,i) = sqrt(sum(CBgcQ.^2 - repmat(tegcQ(i,:).^2,[d1 1]),2));
                        end
                    else %podem fer mahalanobis
                        % Clcul de les distncies
                        rep = (repmat(obj.Case_Base.getQContributionLimitStd(),[d1 1])).^2;
                        for i = 1:size(tegcQ,1)
                            distancia(:,i) =sqrt( sum( (CBgcQ -  repmat(tegcQ(i,:), [d1 1])).^2 ./ rep, 2) );
                            % distancia(:,i) = sqrt(sum((obj.Case_Base.getQContribution())./obj.Case_Base.getStd() - repmat((aux(i,:))/obj.Case_Base.getStd(),[d1 1])).^2,2);
                        end
                    end
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                 case 10 % Distncia sobre Contribucions de T^2 en distancia mahalanobis (STD)
                    % Inicialitzacions
                    tegcQ=obj.test.getT2Contribution();
                    CBgcQ=obj.Case_Base.getT2Contribution();
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                    %si Tenim nomes un cas a la base de casos da STD es 0!!
                    if(obj.Case_Base.getBatchCount()<2)
                        %no podem fer mahalanobis, farem distancia
                        %euclidiana normal
                        for i = 1:size(tegcQ,1)
                            distancia(:,i) = sqrt(sum(CBgcQ.^2 - repmat(tegcQ(i,:).^2,[d1 1]),2));
                        end
                    else %podem fer mahalanobis
                        % Clcul de les distncies
                        rep = (repmat(obj.Case_Base.getT2ContributionLimitStd(),[d1 1])).^2;
                        for i = 1:size(tegcQ,1)
                            distancia(:,i) =sqrt( sum( (CBgcQ -  repmat(tegcQ(i,:), [d1 1])).^2 ./ rep, 2) );
                            % distancia(:,i) = sqrt(sum((obj.Case_Base.getQContribution())./obj.Case_Base.getStd() - repmat((aux(i,:))/obj.Case_Base.getStd(),[d1 1])).^2,2);
                        end
                    end
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 11% Distncia combinada sobre Q i T^2
                    % Inicialitzacions
                    tegT2=obj.test.getT2();
                    CBgT2=obj.Case_Base.getT2();
                    tegQ=obj.test.getQ();
                    CBgQ=obj.Case_Base.getQ();
                    % Inicialitzem el valor de k1 i k2
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    distancia1 = zeros(length(CBgT2),length(tegT2));
                    
                    % Clcul de les distncies per T^2
                    for i = 1:length(tegT2)
                        distancia1(:,i) = abs(CBgT2 - repmat(tegT2(i),length(CBgT2),1));
                    end
                    
                    % Distncia sobre Q
                    % Inicialitzem el vector on es guardaran les distncies
                    distancia2 = zeros(length(CBgQ),length(tegQ));
                    % Clcul de les distncies
                    for i = 1:length(tegQ)
                        distancia2(:,i) = abs(CBgQ - repmat(tegQ(i),length(CBgQ),1) );
                    end
                    
                    distancia=sqrt(distancia1.^2+distancia2.^2);
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 12% Distncia sobre Signatures de Q
                    % Inicialitzacions
                    tegcQ=obj.test.getQContribution();
                    tegcQ = tegcQ > repmat((obj.test.getQContributionLimitMean + obj.test.getQContributionLimitStd * 3),[size(tegcQ,1) 1] ); %pasem a binari
                    CBgcQ=obj.Case_Base.getQContribution();
                    CBgcQ= CBgcQ> repmat((obj.Case_Base.getQContributionLimitMean + obj.Case_Base.getQContributionLimitStd * 3),[size(CBgcQ,1) 1] ); %pasem a binari
                    
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                    % Clcul de les distncies
                    for i = 1:size(tegcQ,1)
                        distancia(:,i)=sum(repmat(tegcQ(i,:),[d1 1]) ~= CBgcQ,2);
                    end
                    distancia =  distancia / size(tegcQ,2);
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 13% Distncia sobre Signatures de T^2
                    % Inicialitzacions
                    tegcT2=obj.test.getT2Contribution();
                    tegcT2 = tegcT2 > repmat((obj.test.getT2ContributionLimitMean + obj.test.getT2ContributionLimitStd * 3),[size(tegcT2,1) 1] ); %pasem a binari
                    CBgcT2=obj.Case_Base.getT2Contribution();
                    CBgcT2= CBgcT2> repmat((obj.Case_Base.getT2ContributionLimitMean + obj.Case_Base.getT2ContributionLimitStd * 3),[size(CBgcT2,1) 1] ); %pasem a binari
                    
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcT2,1);
                    distancia = zeros(d1,size(tegcT2,1));
                    
                    % Clcul de les distncies
                    for i = 1:size(tegcT2,1)
                        distancia(:,i)=sum(repmat(tegcT2(i,:),[d1 1]) ~= CBgcT2,2);
                    end
                    distancia =  distancia / size(tegcT2,2);
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 14 % Signatures de Q agrupades per fases
                    % Inicialitzacions
                    tegcQ=obj.test.getQContribution();
                    tegcQ = tegcQ > repmat((obj.test.getQContributionLimitMean + obj.test.getQContributionLimitStd * 3),[size(tegcQ,1) 1] ); %pasem a binari
                    CBgcQ=obj.Case_Base.getQContribution();
                    CBgcQ= CBgcQ> repmat((obj.Case_Base.getQContributionLimitMean + obj.Case_Base.getQContributionLimitStd * 3),[size(CBgcQ,1) 1] ); %pasem a binari
                    dfases=repmat(obj.test.getDATA.getStagesDurations(),1,obj.test.getVariableCount);
                    if isempty(dfases)
                       error('There is no value in Fases. You can use this distance method without Fases value.') 
                    end
                    index = 1;
                    cdfases=cumsum(dfases);
                    tegcQ=double(tegcQ);
                    CBgcQ = double(CBgcQ);
                    for i=1:length(dfases)
                       tegcQ(:,i) = sum(tegcQ(:,index:cdfases(i)),2);
                       CBgcQ(:,i) = sum(CBgcQ(:,index:cdfases(i)),2);
                       index = index + dfases(i);
                    end
                    tegcQ(:,(length(dfases)+1):end) = [];
                    CBgcQ(:,(length(dfases)+1):end) = [];
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                    % Clcul de les distncies
                    for i = 1:size(tegcQ,1)
                        distancia(:,i)=abs(sum(repmat(tegcQ(i,:),[d1 1]) - CBgcQ,2));
                    end
                    distancia =  distancia / max(cdfases);
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 15 % Signatures de T2 agrupades per fases
                    % Inicialitzacions
                    tegcQ=obj.test.getT2Contribution();
                    tegcQ = tegcQ > repmat((obj.test.getT2ContributionLimitMean + obj.test.getT2ContributionLimitStd * 3),[size(tegcQ,1) 1] ); %pasem a binari
                    CBgcQ=obj.Case_Base.getT2Contribution();
                    CBgcQ= CBgcQ> repmat((obj.Case_Base.getT2ContributionLimitMean + obj.Case_Base.getT2ContributionLimitStd * 3),[size(CBgcQ,1) 1] ); %pasem a binari
                    dfases=repmat(obj.test.getDATA.getStagesDurations(),1,obj.test.getVariableCount);
                    if isempty(dfases)
                       error('There is no value in Fases. You can use this distance method without Fases value.') 
                    end
                    index = 1;
                    cdfases=cumsum(dfases);
                    tegcQ=double(tegcQ);
                    CBgcQ = double(CBgcQ);
                    for i=1:length(dfases)
                       tegcQ(:,i) = sum(tegcQ(:,index:cdfases(i)),2);
                       CBgcQ(:,i) = sum(CBgcQ(:,index:cdfases(i)),2);
                       index = index + dfases(i);
                    end
                    tegcQ(:,(length(dfases)+1):end) = [];
                    CBgcQ(:,(length(dfases)+1):end) = [];
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                    % Clcul de les distncies
                    for i = 1:size(tegcQ,1)
                        distancia(:,i)=abs(sum(repmat(tegcQ(i,:),[d1 1]) - CBgcQ,2));
                    end
                    distancia =  distancia / max(cdfases);
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 16 % Distncia sobre Contribucions de Q en distancia mahalanobis (Lambda)
                                     % Inicialitzacions
                    testgT=obj.test.getT();
                    CBgT=obj.Case_Base.getT();
                   
                    
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    distancia = zeros(size(CBgT,1),size(testgT,1));
                   

                    % Clcul de les distncies
                    for i = 1:size(testgT,1)
                        distancia(:,i) = sqrt(sum(((CBgT - repmat(testgT(i,:),size(CBgT,1),1) ).^2 ),2));
                    end
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 17  % Signatures de Q agrupades per fases binaritzades
                    % Inicialitzacions
                    tegcQ=obj.test.getQContribution();
                    tegcQ = tegcQ > repmat((obj.test.getQContributionLimitMean + obj.test.getQContributionLimitStd * 3),[size(tegcQ,1) 1] ); %pasem a binari
                    CBgcQ=obj.Case_Base.getQContribution();
                    CBgcQ= CBgcQ> repmat((obj.Case_Base.getQContributionLimitMean + obj.Case_Base.getQContributionLimitStd * 3),[size(CBgcQ,1) 1] ); %pasem a binari
                    dfases=repmat(obj.test.getDATA.getStagesDurations(),1,obj.test.getVariableCount);
                    if isempty(dfases)
                       error('There is no value in Fases. You can use this distance method without Fases value.') 
                    end
                    index = 1;
                    cdfases=cumsum(dfases);
                    tegcQ=double(tegcQ);
                    CBgcQ = double(CBgcQ);
                    for i=1:length(dfases)
                       tegcQ(:,i) = sum(tegcQ(:,index:cdfases(i)),2);
                       CBgcQ(:,i) = sum(CBgcQ(:,index:cdfases(i)),2);
                       index = index + dfases(i);
                    end
                    tegcQ(:,(length(dfases)+1):end) = [];
                    CBgcQ(:,(length(dfases)+1):end) = [];
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                    % Clcul de les distncies      
                    tegcQ = tegcQ > repmat((dfases*obj.param),size(tegcQ,1),1 );
                    CBgcQ = CBgcQ > repmat((dfases*obj.param),size(CBgcQ,1),1 );
                    
                    %no cal fer el for!!
                    for i = 1:size(tegcQ,1)
                        distancia(:,i)=abs(sum(repmat(tegcQ(i,:),[d1 1]) - CBgcQ,2));
                    end
                    distancia =  distancia / size(tegcQ,2);
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                case 18 % Signatures de T2 agrupades per fases
                    % Inicialitzacions
                    tegcQ=obj.test.getT2Contribution();
                    tegcQ = tegcQ > repmat((obj.test.getT2ContributionLimitMean + obj.test.getT2ContributionLimitStd * 3),[size(tegcQ,1) 1] ); %pasem a binari
                    CBgcQ=obj.Case_Base.getT2Contribution();
                    CBgcQ= CBgcQ> repmat((obj.Case_Base.getT2ContributionLimitMean + obj.Case_Base.getT2ContributionLimitStd * 3),[size(CBgcQ,1) 1] ); %pasem a binari
                    dfases=repmat(obj.test.getDATA.getStagesDurations(),1,obj.test.getVariableCount);
                    if isempty(dfases)
                       error('There is no value in Fases. You can use this distance method without Fases value.') 
                    end
                    index = 1;
                    cdfases=cumsum(dfases);
                    tegcQ=double(tegcQ);
                    CBgcQ = double(CBgcQ);
                    for i=1:length(dfases)
                       tegcQ(:,i) = sum(tegcQ(:,index:cdfases(i)),2);
                       CBgcQ(:,i) = sum(CBgcQ(:,index:cdfases(i)),2);
                       index = index + dfases(i);
                    end
                    tegcQ(:,(length(dfases)+1):end) = [];
                    CBgcQ(:,(length(dfases)+1):end) = [];
                    % Inicialitzem el valor de k
                    k = min(obj.vk);
                    % Inicialitzem el vector on es guardaran les distncies
                    d1=size(CBgcQ,1);
                    distancia = zeros(d1,size(tegcQ,1));
                    
                   % Clcul de les distncies

                    tegcQ = tegcQ > repmat((dfases *obj.param),size(tegcQ,1),1 );
                    CBgcQ = CBgcQ > repmat((dfases *obj.param),size(CBgcQ,1),1 );
                    
                    % Clcul de les distncies
                    for i = 1:size(tegcQ,1)
                        distancia(:,i)=abs(sum(repmat(tegcQ(i,:),[d1 1]) - CBgcQ,2));
                    end
                    distancia =  distancia / max(cdfases);
                    % Ordenaci dels casos segons la distncia
                    [ordenats_k1,indexs_k1] = sort(distancia,1);
                    % Selecci dels vens ms propers
                    obj.veins.ordenats_k1 = ordenats_k1(1:k,:);
                    obj.veins.indexs_k1 = indexs_k1(1:k,:);
                    obj.veins.ordenats_k2 = obj.veins.ordenats_k1;
                    obj.veins.indexs_k2 = obj.veins.indexs_k1;
                    
                %case xx
                %si s'afegeixen noves distancies recordar a modificar drop4
                %El drop4 optimitzat necesita saber quins metodes son
                %distancies d'un nivell i quines de 2 si no donar error!
                
            end
        end   
        function CBR_reuse(obj)
            %calculs del metode reuse
            
            if(obj.vk(2)==1)
                %Si nomes bolen un cas no calen calculs!
                 obj.classepredit= obj.Case_Base.getClass(obj.veins.indexs_k2(1,:));
            else
                %obj.vk(2) > 1
                vks = min(min(obj.vk),size(obj.veins.indexs_k2,1));
                switch (obj.method)
                    case 1 % voting ponderat per distancia i frequencia de la classe
                        
                        classes=obj.Case_Base.getClass(); %les classes de tots els lots
                        n_classes=max(classes); %el numero de classes (el calculem aixi perque pot haver-hi classes buides mentres fem IB3 ...)
                        
                        classes_i = classes(obj.veins.indexs_k2(1:vks,:));%la classe de l'element
                        
                        %Calculem el que 'val' el bot de cada element
                        pes =  1./ obj.veins.ordenats_k2(1:vks,:);
                        if size(classes_i,1)==1
                            classes_i=classes_i';
                        end
                        if size(pes,1)==1
                            pes=pes';
                        end
                        freq=hist(classes(obj.veins.indexs_k2(1:vks,:)),1:n_classes); %Contem la frequencia en que ens apareixen les classes per cada observaci
                        if size(freq,1)==1
                            freq=freq';
                        end
                        %per tal d'afaborir als grups fem que el vot de l'element valgui mes per la classe predominant
                        if obj.test.getBatchCount == 1
                            pes=pes.*freq(classes_i); %modif ic 22/09/11
                        else
                            pes=pes.* (freq(classes_i + repmat((((1:obj.test.getBatchCount)-1).*n_classes),size(classes_i,1),1)));
                            %pes=pes.* (freq(classes_i + repmat((((1:obj.test.getBatchCount)-1).*n_classes),min([n_classes length(classes)]),1))); %multipliquem per la frequencia de la classe de l'element
                        end
                        %posem els pesos maxims a 0
                        maxi=zeros(1,obj.test.getBatchCount);
                        
                        %inicialitzem les classes a 0
                        obj.classepredit=zeros(1,obj.test.getBatchCount);
                        
                        for i=1:n_classes
                            if size(classes_i,1)>1
                                index=classes_i(1:vks,:)==i;%un punter als que son de clase indicada
                            else
                                index=(classes_i(1:vks)==i)';%quan es d'una sola dimencio s'ha de posar be
                            end
                            maxpes = pes .* index; %pasem de punters a tenir els pesos dels elements
                            
                            maxpes(isnan(maxpes)) = 0;% Per evitar problemes de Inf + NaN = NaN i ens interessa l'inf!!!
                            
                            pesos=sum(maxpes); % sumem els pesos dels elements de classe i
                            obj.classepredit(pesos > maxi)  = i; %si aquest pes 'pesos' es mes gran que l'anterior assignem la classe a l'element
                            maxi(pesos > maxi) =  pesos(pesos > maxi); %actualitzem el vector de pesos maxims
                        end
                        
                    case 2 %Votting simple
                        
                        classes=obj.Case_Base.getClass(); %les classes de tots els lots
                        n_classes=max(classes); %el numero de classes (el calculem aixi perque pot haver-hi classes buides mentres fem IB3 ...)
                        if n_classes == 1
                            obj.classepredit=ones(1,obj.test.getBatchCount);
                        else
                            vks = max(1,min(min(hist(classes,1:n_classes)),vks));
                            if isempty(vks) %no hi ha res a la C_B
                                obj.classepredit=-ones(1,obj.test.getBatchCount);
                            elseif(vks == 1)
                                obj.classepredit=classes(obj.veins.indexs_k2(1:vks,:)); %Contem la frequencia en que ens apareixen les classes per cada observaci
                            else
                                freq=hist(classes(obj.veins.indexs_k2(1:vks,:)),1:n_classes); %Contem la frequencia en que ens apareixen les classes per cada observaci
                                [~,obj.classepredit] = max(freq);%la posici on es troba el maxim es la classe de l'element!
                            end
                        end
                        
                    case 3 %Votting ponderat per distancia
                        
                        classes=obj.Case_Base.getClass(); %les classes de tots els lots
                        n_classes=max(classes); %el numero de classes (el calculem aixi perque pot haver-hi classes buides mentres fem IB3 ...)
                        
                        classes_i = classes(obj.veins.indexs_k2(1:vks,:));%la classe de l'element
                        
                        %Calculem el que 'val' el bot de cada element
                        pes =  1./ obj.veins.ordenats_k2(1:vks,:);
                        
                        %posem els pesos maxims a 0
                        maxi=zeros(1,obj.test.getBatchCount);
                        
                        %inicialitzem les classes a 0
                        obj.classepredit=zeros(1,obj.test.getBatchCount);
                        
                        for i=1:n_classes
                            if size(classes_i,1)>1
                                index=classes_i(1:vks,:)==i;%un punter als que son de clase indicada
                            else
                                index=(classes_i(1:vks)==i)';%quan es d'una sola dimencio s'ha de posar be
                            end
                            maxpes = pes .* index; %pasem de punters a tenir els pesos dels elements
                            
                            maxpes(isnan(maxpes)) = 0;% Per evitar problemes de Inf + NaN = NaN i ens interessa l'inf!!!
                            
                            pesos=sum(maxpes); % sumem els pesos dels elements de classe i
                            obj.classepredit(pesos > maxi)  = i; %si aquest pes 'pesos' es mes gran que l'anterior assignem la classe a l'element
                            maxi(pesos > maxi) =  pesos(pesos > maxi); %actualitzem el vector de pesos maxims
                        end
                        
                    otherwise
                        error('CBR/reuse: Choose a valid method!');
                        
                end
            end
            
        end
        function CBR_retain(obj,fallen)
            %Calculs del metode retain
            
            if( sum(fallen)~=0)
                
                %afegim un cas a la BD si s'ha predit malament
                
                if(size(fallen,1)~=1) && (size(fallen,2)~=1)
                    error('CBR/Retain: the class vector sise is not valid. A vector is needed');
                end
                
                [dades class noms]=obj.test.getBatch(fallen);%els agafem
                
                obj.Case_Base.addBatch(dades,class,noms);%els afegim a la C_B projectantlos ...
            end
        end
        function [matri fallen] = CBR_revise(obj)
            %Calculs de la funcio revise
            
            if isempty(obj.test.getClass()) 
                %hem de demanar la classe a l'usuari
                c=input('Give the new class vector: exemple [ 1 1 1 2 ]\n');
                obj.test.setClass(c);
            end
            
            %matriu de confusio i calcular els estadistics associats
            %si coneixem la classe del testset es automatic.
            matri = compute_extended_confusion_matrix(obj.classepredit,obj.test.getClass(), length(unique(obj.Case_Base.getClass())));
            fallen=(obj.classepredit~=obj.test.getClass());
        end
        
        function Calcula_IB_UdG(obj, nelem, descarta)
            %Calculs de IBUDG
            
            if descarta > nelem
                nelem=descarta;
            end
            
            %nelem=nelem+descarta;
            
            %Metode per netejar la base de casos
            segur=obj.vk;
            obj.vk=[obj.Case_Base.getBatchCount() obj.Case_Base.getBatchCount()];
            obj.retrieve(obj.Case_Base.getDATA());
            obj.vk=segur;
            classes=obj.Case_Base.getClass();
            resu=classes(obj.veins.indexs_k1);
            %tenim una matriu amb dim(1) classe dels veins ordenats per
            %distancia i dim(2) tots els casos
            
            %el primer es ell mateix aixo ens va Be per saver kin agafar
            enemics=(resu~=repmat(resu(1,:),size(resu,1),1));%logics (els que no son de la meva classe)
            [~,IX]=sort(enemics,'descend');
            x2=1:obj.Case_Base.getBatchCount();
            x2=repmat(x2,nelem,1);
            x1=IX(1:nelem,:);
            %index=unique(obj.veins.indexs_k2((x1*obj.Case_Base.getBatchCount())+x2 - 1));
            
            %posem l'element de cada classe k "menys enemics te"  (centroide aproximat)
            bons=ones(size(unique(classes)))*nelem;
            indexcentres=zeros(size(unique(classes)));
            sumaenemics=sum(enemics(2:nelem,:),1);
            for i=1 : obj.Case_Base.getBatchCount()
                if(bons(classes(i))>sumaenemics(i))
                    bons(classes(i))=sumaenemics(i);
                    indexcentres(classes(i))=i;
                end
            end
            temp=unique(obj.veins.indexs_k2(sub2ind(size(obj.veins.indexs_k2),x1,x2)));
            temp2=unique(obj.veins.indexs_k2(sub2ind(size(obj.veins.indexs_k2),x1(1:descarta,:),x2(1:descarta,:))));
            temp=setdiff(temp,temp2);
            if size(temp,1)~=1
                temp=temp';
            end
            
            indexcentres=setdiff(indexcentres,indexcentres(unique(classes(temp))));
            index = unique([temp indexcentres]);
            %a enemics buscar la posicio del primer 1 per cada columna
            
            
            
            obj.Case_Base.deleteBatch(setdiff(1:obj.Case_Base.getBatchCount(),index));
            %treuresoroll(obj,0.7,nelem,[obj.Case_Base.getBatchCount() obj.Case_Base.getBatchCount()],dc);
            
        end
        function calcula_DROP4(obj)
            %Calculs DROP4
            
            C_B_vella=obj.Case_Base.getDATA();%copia auxiliar de la C_B inicial
            cla=C_B_vella.getClass();%classes inicials
            %C_B_Vella l'utilitzarem de conjunt de test.
            
            %primer de tot un filtre per treure el soroll. A diferencia del
            %proposat als papers aqui no el fem iteratiu calculem de cop
            %tots els enemics i treiem els que tenen el 70% dels seus veins
            %de classes diferents a la seva. Aix fa que el nostre filtre
            %de soroll no depengui de l'ordre dels elements. Tambe es mes
            %rapid de calcular en matlab que la versi iterativa.
            
            treuresoroll(obj,0.7,min(obj.vk));
            

            %Per tal d'evitar haver de calcular totes les distancies a cada
            %iteraci el que farem es calcular un sol cop totes les
            %distancies i anar mantenint l'estructura dels veins perque
            %quedi com si haguessim fet el retreave pertinent.
            segur=obj.vk;%faig una copia del VK actual per no perdrel en fer un retreave de tots contra tots.
            obj.vk=[C_B_vella.getBatchCount() C_B_vella.getBatchCount()];%maxim possible aixi calculara totes les distancies
            obj.retrieve(C_B_vella);%tem el retreave
            obj.vk=segur;%tornem a deixar vk com estava
            
            
            %busquem el nombre de classificacio inicial
            classpredit = reuse(obj);%prediccio inicial
            encertats=sum(classpredit == cla);%classificaci inicial (no deixarem que vaxi)
            
            %Per cada lot hem de treurel de la base de casos i omprobar que
            %sense ell no vaixi el % de classificaci de la base de casos.
            for i=1:obj.Case_Base.getBatchCount()%per cada lot
                %treiem el lot
                [dades , clas , noms]=obj.Case_Base.getBatch(1);%(treurem sempre per davant i afegirem per derrera aixi evitem tenir indexs)
                obj.Case_Base.deleteBatch(1);%treiem un lot
                %busquem la classificaci de cada un dels elements de la C_B_vella
                
                %Ara tocaria fer:
                %   retrieve(obj,C_B_vella);
                %   reuse(obj);
                %Per aixo es extremadament inaficient en matlab per tant ens tocara fer el retreave a ma. 
                
                %per si hi ha menys elements a la C_B dels que bolem agafar segons el retreave
                vkseg = obj.vk;
                obj.vk(1) = min(obj.vk(1),obj.Case_Base.getBatchCount);
                obj.vk(2) = min(obj.vk(2),obj.Case_Base.getBatchCount);
                
                if (obj.dc <4 || obj.dc >6)%primer de tot controlem si es una distancia d'un nivell o de dos ya que el procediment canviar.
                    %Aqui es on farem el ''retreave'' de les distancies
                    %d'un sol nivell.
                    
                    x=obj.veins.indexs_k2(:);%el passem els index a 1 dimenci
                    ins=x==1;%busquem els 1
                    
                    %calculem els indexs veins sense l'element (treiem els 1 i restem 1 als altres)
                    temporal=(reshape(x(not(ins))',size(obj.veins.indexs_k2,1)-1,size(obj.veins.indexs_k2,2)))-1; %i els hi restem 1
                    
                    %calculem la matriu d'index veins per si l'element s'afegeix posteriorment
                    x(ins)=obj.Case_Base.getBatchCount()+2; %substitueixo els 1 per num max+2
                    x=x-1;%refem els indexs de tots els altres elements restan 1 (els 1 passen a ser max i els 2->1 3->2....) aixo es per si acabem afegint l'element per darrera com quedara la matriu.
                    segur_i_k2=reshape(x,size(obj.veins.indexs_k2,1),size(obj.veins.indexs_k2,2));%posem be de nou la matriu per si l'element s'afegeix
                    %les distancies no canviaran si s'efegeix l'element
                    segur_o_k2=obj.veins.ordenats_k2;%faix una copia dels ordenats_k2
                    
                    %ja podem modificar index_k2 sense perill
                    obj.veins.indexs_k2=temporal;
                    
                    %calculem ordenats_k2 sense l'element que hem tret
                    x=obj.veins.ordenats_k2(:);%agafem els ordenats en format 1 dimenci
                    obj.veins.ordenats_k2=reshape(x(not(ins))',size(obj.veins.ordenats_k2,1)-1,size(obj.veins.ordenats_k2,2)); %treiem els que eren (index 1)

                    reuse(obj); %Fem un reuse per calcular la classe de cada element
                    
                else
                    
                    %Aqui mantindrem fixes els K1 i K2 (amb tots els elements) 
                    %%%%%%%%%%%%K1%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    x=obj.veins.indexs_k1(:);%el passem els index a 1 dimenci
                    ins=x==1;%busquem els 1
                    
                    %calculem els indexs veins sense l'element (treiem els 1 i restem 1 als altres)
                    temporal=(reshape(x(not(ins))',size(obj.veins.indexs_k1,1)-1,size(obj.veins.indexs_k1,2)))-1; %i els hi restem 1 
                    
                    %calculem la matriu d'index veins per si l'element s'afegeix posteriorment
                    x(ins)=obj.Case_Base.getBatchCount()+2; %substitueixo els 1 per num max+2
                    x=x-1;%refem els indexs de tots els altres elements restan 1 (els 1 passen a ser max i els 2->1 3->2....) aixo es per si acabem afegint l'element per darrera com quedara la matriu.
                    segur_i_k1=reshape(x,size(obj.veins.indexs_k1,1),size(obj.veins.indexs_k1,2));%posem be de nou la matriu per si l'element s'afegeix
                    %les distancies no canviaran si s'efegeix l'element
                    segur_o_k1=obj.veins.ordenats_k1;%faix una copia dels ordenats_k1
                    
                    %ja podem modificar index_k1 sense perill
                    obj.veins.indexs_k1=temporal;
                    
                    %calculem ordenats_k1 sense l'element que hem tret
                    x=obj.veins.ordenats_k1(:);%agafem els ordenats en format 1 dimenci
                    obj.veins.ordenats_k1=reshape(x(not(ins))',size(obj.veins.ordenats_k1,1)-1,size(obj.veins.ordenats_k1,2)); %treiem els que eren (index 1)

                    
                    %%%%%%%%%%%%K2%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    x=obj.veins.indexs_k2(:);%el passem els index a 1 dimenci
                    ins=x==1;%busquem els 1
                    
                    %calculem els indexs veins sense l'element (treiem els 1 i restem 1 als altres)
                    temporal=(reshape(x(not(ins))',size(obj.veins.indexs_k2,1)-1,size(obj.veins.indexs_k2,2)))-1; %i els hi restem 1 
                    
                    %calculem la matriu d'index veins per si l'element s'afegeix posteriorment
                    x(ins)=obj.Case_Base.getBatchCount()+2; %substitueixo els 1 per num max+2
                    x=x-1;%refem els indexs de tots els altres elements restan 1 (els 1 passen a ser max i els 2->1 3->2....) aixo es per si acabem afegint l'element per darrera com quedara la matriu.
                    segur_i_k2=reshape(x,size(obj.veins.indexs_k2,1),size(obj.veins.indexs_k2,2));%posem be de nou la matriu per si l'element s'afegeix
                    %les distancies no canviaran si s'efegeix l'element
                    segur_o_k2=obj.veins.ordenats_k2;%faix una copia dels ordenats_k2
                    
                    %ja podem modificar index_k2 sense perill
                    obj.veins.indexs_k2=temporal;
                    
                    %calculem ordenats_k2 sense l'element que hem tret
                    x=obj.veins.ordenats_k2(:);%agafem els ordenats en format 1 dimenci
                    obj.veins.ordenats_k2=reshape(x(not(ins))',size(obj.veins.ordenats_k2,1)-1,size(obj.veins.ordenats_k2,2)); %treiem els que eren (index 1)

                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    %De K2 ens quedem nomes amb els que estiguin entre els
                    %vk(1) de K1 sempre i quan tinguem prous elements
                    ind=obj.veins.indexs_k1(1:obj.vk(1),:); %agafem els indexs
                    
                    %inicialitzem el tamany

                     orde = zeros(obj.vk(2),length(obj.veins.ordenats_k1));
                     inde = zeros(obj.vk(2),length(obj.veins.ordenats_k1));
                     
                     %per cada fila de ind
                    for buc=1:size(ind,1)
                        %agafem els que toquen de K2
                        orde(buc,:)=(obj.veins.ordenats_k1(obj.veins.indexs_k2==repmat(ind(buc,:),size(obj.veins.indexs_k2,1),1)))';
                        inde(buc,:)=(obj.veins.indexs_k1(obj.veins.indexs_k2==repmat(ind(buc,:),size(obj.veins.indexs_k2,1),1)))';
                    end
                    
                    %fem una copia de K2 per no perdrel's
                    s1 = obj.veins.indexs_k2;
                    s2 = obj.veins.ordenats_k2;
                    %posem els  K2 tallats a VK(2)
                    obj.veins.indexs_k2 = inde;
                    obj.veins.ordenats_k2 = orde;
                    
                    %fem el reuse
                    reuse(obj);                  
                    %reestablim els indexs
                    obj.veins.indexs_k2 = s1;
                    obj.veins.ordenats_k2 = s2;
                end

                %comparem amb la classe real dels elements
                
                %ja podem tornar a posar be VK
                obj.vk = vkseg;
                
                if length(unique(obj.Case_Base.getClass)) ~= length(unique(C_B_vella.getClass)) || sum(obj.classepredit == cla) < encertats %hem empitjorat la clasificacio en treure l'element
                    obj.Case_Base.addBatch(dades,clas,noms);%el tornem a afegir
                    if (obj.dc <4 || obj.dc >6) %segons el tipus de distancia 
                        %distancie 1 nivell
                         obj.veins.indexs_k2=segur_i_k2;%posem be els veins
                         obj.veins.ordenats_k2=segur_o_k2;%posem be les distancies als veins
                    else
                        %distancies 2 nivells
                        %per K1
                        obj.veins.indexs_k1=segur_i_k1;%posem be els veins
                        obj.veins.ordenats_k1=segur_o_k1;%posem be les distancies als veins
                        %per K2
                        obj.veins.indexs_k2=segur_i_k2;%posem be els veins
                        obj.veins.ordenats_k2=segur_o_k2;%posem be les distancies als veins
                    end
                    
                    
                    
                else %ha millorat
                    %com que millora recalculem i no permetrem que vaixi
                    %d'aqui!
                    encertats=sum(obj.classepredit == cla);
                end
            end
        end
        function Calcula_IB3(obj)
            %Calculs IB3
            %  The  IB3 algorithm  (CD  =  Concept  Description).
            %   CD  =  []
            %   For  each x  in  TrainingSet do
            %      For  each y  in  CD  do
            %          Sim[y] = Similarity(x, y)
            %      End
            %
            %      if  Exists{y  in  CD |   acceptable(y)} then
            %          ymax = some  acceptable  y  in  CD with maximal Sim[y]
            %      else
            %          i = randomly-selected value in CD
            %          ymax = some y in CD that is the i-th most similar instance to x
            %      end
            %
            %      if  class(x) = class(ymax) then
            %          classification  <- correct
            %      else
            %          classification  -  incorrect
            %          CD  <-  CD  U   {x}
            %      end
            %
            %      For  each y  in  CD  do
            %         if  Sim[y]  >   Sim[ymax] then
            %           Update  y's  classification  record
            %
            %           if  y's  record  is  significantly  poor then
            %               CD - C -  {y}
            %           end
            %         end
            %      end
            %   end
            
            %els elements del CD no han d'estar ordenats per classe
            
            Old_CD = obj.Case_Base.copy();%el set de test sera una copia de la nostra base de casos (Concept description)
            obj.Case_Base.deleteBatch(1:obj.Case_Base.getBatchCount());%treiem tots els lots de la nostra base de casos(CD=[])
            dades=obj.Case_Base.getDATA.copy();%sera l'element k comprovem
            
            taula=struct('encerts',0,'errors',0,'classe',0,'index',0);
            
            for x=1 : Old_CD.getBatchCount()%For each instance x in TrainingSet
                [dad cla noms]=Old_CD.getBatch(x);%recuperem el lot x
                if x~=1
                    %For each y in CD do
                    %   Sim[y] = Similarity(x,y)
                    %End
                    dades.addBatch(dad,cla,noms);%l'afegim a l'objecte dades
                    copy=obj.vk;
                    retrieve(obj,dades);%Busquem els seus veins
                    Sim = reuse(obj);%Trovem la seva classificaci segons la Case_Base actual
                    obj.vk=copy;
                    dades.deleteBatch(1);%eliminem el lot
                    
                    %if Exists{y in CD | acceptable(y)} then
                    %    ymax = some acceptable y in CD with maximal Sim[y]
                    %else
                    %    i = randomly-selected value in CD
                    %    ymax = some y in CD that is the i-th most similar instance to x
                    %end
                    
                    acceptables=(taula.encerts/(taula.encerts+taula.errors))>0.9;
                    candidat = obj.veins.indexs_k2(acceptables);
                    
                    if(not(isempty(candidat)))
                        Sim = taula.classe(candidat(1));
                        
                    else
                        %si no hi ha cap acceptable
                    end
                    
                    %if class(x) = class(ymax) then
                    %   classification <- correct
                    %else
                    %   classification <- incorrect
                    %   CD = CD U {x}
                    %end
                    if Sim ~= cla
                        %L'hem classificat be no cal fer res altrament l'afegim
                        obj.Case_Base.addBatch(dad,cla,noms);
                        taula.encerts=[taula.encerts 1];
                        taula.errors=[taula.errors 0];
                        taula.classe=[taula.classe cla];
                        taula.index=[taula.index x];
                        
                        taula.errors(obj.veins.indexs_k2(1))=taula.errors(obj.veins.indexs_k2(1))+1;
                    else
                        taula.encerts(obj.veins.indexs_k2(1))=taula.encerts(obj.veins.indexs_k2(1))+1;
                    end
                    
                    %For each y in CD do
                    %   if Sim[y] > Sim[ymax] then
                    %       Update y's classification record
                    %
                    %       if y's record is significantly poor then
                    %           CD = CD - {y}
                    %       end
                    %   end
                    %end
                    
                    %classes=obj.Case_Base.getClass();
                    %resu=classes(obj.veins.indexs_k2);
                    %enemics=(resu~=repmat(resu(1,:),size(resu,1),1));%logics (els que no son de la meva classe)
                    %sumaenemics=sum(enemics(1:size(enemics,1),:),1)./size(enemics,1);
                    enemics= taula.encerts./(taula.encerts + taula.errors);
                    
                    enemicsclase=zeros(max(unique(taula.classe)),1);%Inicialitzem!
                    for z=unique(taula.classe)
                        enemicsclase(z)=mean(enemics(taula.classe==z));
                    end
                    
                    
                    indexs=enemics < (enemicsclase(taula.classe) * 0.7)';%<- % que els fa despreciables despreciables
                    obj.Case_Base.deleteBatch(indexs);
                    
                    in=logical(1:length(taula.encerts));%x es un vector de 1 logics de longitud = al numero de lots
                    in(indexs)=0;%posem a 0 els que bolen eliminar
                    
                    taula.encerts=taula.encerts(in);
                    taula.errors=taula.errors(in);
                    taula.classe=taula.classe(in);
                    taula.index=taula.index(in);
                    
                else
                    obj.Case_Base.addBatch(dad,cla,noms);%el primer s'afegir sempre
                    taula.encerts=1;
                    taula.errors=0;
                    taula.classe=cla;
                    taula.index=1;
                end
            end
        end
        function Calcula_IB2(obj)
            %Calculs IB2
            %
            % IB2 [Aha, 1991, 1992]
            %The  IB2 algorithm  (CD  =  ConceptDescription).
            %CD = []
            % for  each x  in Training  Set  do
            %   for  each  y  in  CD  do
            %       Sim[y] = Similarity(x, y)
            %   end
            %   ymax =  Max(Sim[]) <-
            %   if class(x)  =  class(ymax)
            %     classification  - correct
            %   else
            %     classification  - incorrect
            %       CD = CD U {x}
            %   end
            %end
            
            %els elements del CD no han d'estar ordenats per classe
            
            Old_CD = obj.Case_Base.copy();%el set de test sera una copia de la nostra base de casos (Concept description)
            obj.Case_Base.deleteBatch(1:obj.Case_Base.getBatchCount());%treiem tots els lots de la nostra base de casos(CD=[])
            dades=obj.Case_Base.getDATA.copy();%sera l'element k comprovem
            for x=1 : Old_CD.getBatchCount()%For each instance t in T (T -> Old_CD)
                [dad cla noms]=Old_CD.getBatch(x);%recuperem el lot x
                if x~=1
                    dades.addBatch(dad,cla,noms);%l'afegim a l'objecte dades
                    copia=obj.vk;
                    retrieve(obj,dades);%Busquem els seus veins
                    Sim = reuse(obj);%Trovem la seva classificaci segons la Case_Base actual
                    obj.vk=copia;
                    dades.deleteBatch(1);%eliminem el lot
                    if Sim ~= cla
                        %L'hem classificat be no cal fer res altrament l'afegim
                        obj.Case_Base.addBatch(dad,cla,noms);
                    end
                else
                    obj.Case_Base.addBatch(dad,cla,noms);%el primer s'afegir sempre
                end
            end
        end
        
        function ordena(obj)
            %ordena la base de casos per la distancia al enemic mes proper
            %de cada element (els que tenen enemics molt aprop son els
            %lementables)
            
            %index=1:obj.Case_Base.getBatchCount;
            if obj.Case_Base.getBatchCount > 1 && length(unique(obj.Case_Base.getClass())) > 1%no es pot ordenar 1 sol lot!!
                %calculem les distancies de tots contra tots
                segur=obj.vk;
                obj.vk=[obj.Case_Base.getDATA().getBatchCount() obj.Case_Base.getDATA().getBatchCount()];
                obj.retrieve(obj.Case_Base.getDATA());
                obj.vk=segur;%tornem a deixar vk com estava
                
                classes=obj.Case_Base.getClass();
                resu=classes(obj.veins.indexs_k2);
                enemics=(resu~=repmat(resu(1,:),size(resu,1),1));%logics (els que no son de la meva classe)
                enemics= repmat((1:size(enemics,1)),size(enemics,2),1)'.*enemics;
                enemics(enemics==0)=inf;
                enemics=sort(enemics);
                enemics=enemics(1,:);
                enemics=obj.veins.ordenats_k2(sub2ind(size(obj.veins.ordenats_k2),enemics,1:size(obj.veins.ordenats_k2,2)));
                [~,index]=sort(enemics);
                obj.Case_Base.getDATA().shuffle(fliplr(index));
            end

        end
        function treuresoroll(obj,i,nelem)
            %i es el % de soroll que bolem eliminar i nelem es numero
            %d'elements que fem servir per fer el retrieve
            
            copy=obj.vk;%Quan hi ha un sol element retrieve canvia vk
            obj.retrieve(obj.Case_Base.getDATA());
            obj.vk=copy;
            classes=obj.Case_Base.getClass();
            resu=classes(obj.veins.indexs_k2);
            enemics=(resu~=repmat(resu(1,:),size(resu,1),1));%logics (els que no son de la meva classe)
            sumaenemics=sum(enemics(2:nelem,:),1)./nelem;
            indexs=sumaenemics>=i;
            obj.Case_Base.deleteBatch(indexs);
        end
        
    end
    
end