function [mcbr te] = Cross4( in, divs, vk,dc,method,net,W,est,FM,Metode,limits,conts,classes,parami,~)
% Funci per fer un cross validation. Aquesta funci trevalla sobre
% objectes de tipus DATA i utilitza PCA i CBR de la llibreria exitPCA. Un
% cop realitzat el procediment es mostraran els resultats mitjanant les
% funcins Plot_Confusion_Matrix (Per mostrar les matrius de confusi) i
% plot_Bars (Per mostrar el % de classificaci de cada partici).
%
% Parametres d'entrada:
%
%       in - Un objecte DATA. Aquest objecte s el que ha de contenir les
%            dades sobre les quals aplicarem el cross validation. s molt
%            important que aquest objecte dades tingui inicialitzats tots
%            els camps, especialment la matriu de dades i les corresponents
%            classes. Les fases noms seran necessaries si s'escullen
%            distancies que les utilitzen. Els noms dels lots, si en tenim,
%            s'utilitzeran per identificar els lots que fallen a les
%            matrius de confusi.
%
%       divs - Double que indica el nombre de divisions que es bolen
%              utilitzar en el cross validation. Seguirem el proces descrit
%              pel k-fold Cross-Validation, i divs ens indica la k que
%              utilitzarem.
%
%
%       vk - (Configuraci CBR) Veins que es vol agafar a cada nivell.
%       dc - (Configuraci CBR)
%       method - (Configuraci CBR)
%       net - (Configuraci CBR)
%
%       W - (configuraci PCA) Double que indica el mtode de desdoblament
%           que utilitzarem (0= BW 1 = VW). Cross validation utilitza CBR i
%           aquest no s compatible amb Variable Wise!!
%
%       est - (configuraci PCA) Double que indica el mtode
%             d'Estandarditzaci que utilitzadem.
%             Els valors possibles sn:
%                   1 - Auto Scaling 
%                   2 - Group Scaling 
%                   3 - Continous Scaling 
%                   4 - Block Scaling 
%
%
%       FM - (configuraci PCA) Double per indicar si sha de treure la
%            forma mitja (x =True) o no (x = False) durant el procs
%            destandarditzaci.  
%
%       Metode - (configuraci PCA) Double que indica el procediment a usar
%                per determinar el nombre de components principals.
%                Els possibles valors sn:
%                       0 - Manual
%                       1 - Kaiser
%                       2 - Manual/Visual pel colze
%                       3 - Auto/Visual pel colze(proves)
%                       4 - Manual/Visual pel % d`explicacio de les dades
%                       5 - VRE
%
%
%       limits - (configuraci PCA) Double per indicar el metode que
%                utilitzarem per calcular els  limits.
%                Els possibles valors son:
%                       1=analitic
%                       0= 3 sigma
%
%       conts - (configuraci PCA) Double per indicar el metode de calcul
%               de les contribucins. 
%               Els possibles valors sn:
%                       0 = descartar negatius
%                       1 = negatius i positius
%
%
%
%       classes - Llista de doubles que indiquen les classes que bolem
%                 utilitxar per fer el model.
%                 Exemple:
%                   Tenim un problema amb 4 classes 1,2,3 i 4. I volem que
%                   els models es crein nmes utilitzant les classes 1 i3
%                   per tant haurem d'indicar a [1 3], si bolem que
%                   s'utilitzi nomes la classe 2 podem indicar-ho  [2] o
%                   simplement 2. Si volem totes les classes podem
%                   indicar-ho  [1 2 3 4] o b utilitzant :
%
%       parami - Indica els parametres extres per a les distancies de CBR
%                que ho necessitin.
%
%       ~ - No utilitzat, per mantenir compatibilitat amb versions
%           anteriors on aquest parametre indicava si es mostrarien les
%           matrius de confusi i barres amb % de classificaci o no.
%
%Parametres de sortida:
%
%   mcbr - Cell aray que conte els k cbr que s'han muntat durant el proces
%          de k-fold cross validation.
%
%   te - List de Objectes DATA que contenen els Casos que s'han utilitzat
%        per testejar els k diferents objectes CBR (retornats a mcbr)
%        creats durant el procs de k-fold cross validation.
%

%Aquesta funci pot trigar vastant a executar-se per tant crearem una barra
%de progres perque l'usuari tingui constancia del temps que falta.
progressbar(0);

%Calculem el nombre de lots que tenim per fer les particions de les dades
nlots=in.getBatchCount();

%desordenem tots els casos aleatoriament (si no ho fem sempre tindrem els mateixos casos als models!)
in.shuffle;%comentar per no desordenar aleatoriament

% No ens interessa que puguin quedar particions amb numeros de classe
% descompensats per tant ordenarem 'in' per que quedin les classes ordenades!!
a=in.getClass();
[~,a]=sort(a);
in.shuffle(a);
%ara les particions tindran la mateixa proporcio de numero de lots de cada classe que l'objecte dades k ens passen!
%ATENCIO! 'in' esta ordenat per classes -> les particions tambe -> DROP4 i
%els altres metodes sencitius a l'ordre

%construim un index que ens indicara a quina divisi pertany cada lot
indexs = repmat(1:divs, 1 , floor(nlots/length(1:divs)));
indexs = [indexs 1:mod(nlots,length(1:divs))];

%conseguim el numero maxim de classes
maxclasses = max(in.getClass);
treuoutliers=[];
Numero =[];
for i = divs:-1:1 %per cada divisi
    progressbar((divs-i+1)/(divs+1));
    %construim la base de casos
    C_B = in.copy();%fem una copia
    C_B.deleteBatch(indexs == i); %treiem els que farem servir de dades
    
    %construim el Model amb tots els de la classe indicades
    model=C_B.copy();
    for t=1:length(classes)
        if(t==1)
            %el primer cop hem de recuperar les classes
            inde =model.getClass ~= classes(t);
        else
            %la resta nomes cal fer una and
            inde = and(inde,  (model.getClass ~= classes(t)));
        end
    end
    model.deleteBatch(inde);%treiem tots els lots que no formen part del model
    model = PCA(model);%creo el model
    
    %configurem el PCA
    model.setUnfoldingMethod(W); %CBR no Variable Wise!!
    model.setNormalizationMethod(est);
    model.setTrajectoryRemoval(FM);
    model.setPCSelectionMethod(Metode);
    model.setLimitMethod(limits);
    model.setContributionMethod(conts);
    
    %fem que nomes demani 1 sol cop el numero de components principals a
    %utilitzar assumirem que es sempre el mateix!!
    if (Metode == 0) %metode manual!
        if isempty(Numero)
            Numero = model.getNum();
        else
            model.setNum(Numero);
        end
    end
    
    %construim les dades de test
    test=in.copy();
    test.deleteBatch(indexs ~= i);
    te(i)=test.copy();
    
    %no ens interesa tenir la C_B ordenada per classes!! -> Drop4 IB.. Fallarien
    C_B.shuffle();
    
    outs = model.getOutLiers();
    
    while(sum(outs) > 0)
        if isempty(treuoutliers)
            treuoutliers = questdlg('There are outliers do you want to remove them?','Attention!','YES','NO','YES');
        end
        switch treuoutliers
            case ''
                %no han contestat fem que contestin! 
                outs = 1;
                treuoutliers =[];
            case 'YES'
                %treiem els outliers
                model.deleteBatch(outs);
                outs=model.getOutLiers();
            case 'NO'
                %sortim sense treure outliers
                treuoutliers ='NO';
                break
                
        end
        
    end
    %construim el CBR pertinent
    MY_CBR = CBR(model,C_B,'NO');
    
    switch net
        case 1
            MY_CBR.IB2(vk,dc,method,parami);
        case 2
            MY_CBR.IB3(vk,dc,method,parami);
        case 3
            MY_CBR.DROP4(vk,dc,method,parami);
        case 4
            MY_CBR.IBUdG(min(vk),0,vk,dc,method,parami);%valor per defecte de veins i descartats
        otherwise
            %S'ha escollit no netejar!
    end
    
    MY_CBR.retrieve(test ,vk,dc,parami);
    
    classpredit=MY_CBR.reuse(method);
    [M{i},fallen]=MY_CBR.revise();
    
    %MY_CBR.retain(fallen);
    
    %Guardem les dades les dades pels plots i sortides
    mcbr{i} = MY_CBR;
    f{i}= fallen;
    classp{i} = classpredit;
    classreal{i} = test.getClass;
end
Plot_Confusion_Matrix(classp,classreal,maxclasses,te,M)
plot_Bars(f);
%Resultats promig
progressbar(1);
end