classdef (ConstructOnLoad = true) DATA < handle % Amb <handle estem heredant de handle i totes lesinstancies de DATA es passaran per referencia
    %La classe DATA es l'encarregada de muntar tota l'estructura de dades
    %necessaria per ferr PCA i posteriorment, si es bol, CBA.
    %Dins aquest Objecte, podem trobar, des de les dades originals de
    %l'objecte guardades en 3D fins a metodes per passar-les a 2D
    %utilitzant metodes de bach wide o variable wide.
    %
    %Les dades mes importants emmagatzemades en aquest objecte podem dir
    %que sn:
    %
    %       nom -   Eel nom del proes emmagatzemat
    %
    %       t_inici 
    %       t_fi   -|
    %       t_inc  _|- Aquests 3 parametres han de contenir els timmings
    %               del proces. Sent temps d'inici,fi i increment de temps
    %               entre mostres.
    %               
    %       variables- Aqui s on es guarda el nom de les diferents
    %               variables, per facilitar el seu maneig aquest camp sera
    %               un vector de cell que contenen text amb el nom de les
    %               variables. Aquest camp es necessari i a ms es
    %               important que el seu tamany sigui coerent amb el de les
    %               dades.
    %
    %       fases - Aqui s on es guarda el nom de les diferents fases del
    %               proces. Aquest camp tambe sera un vector de cell.
    %
    %       durafases-Aqui s'ha d'indicar la duraci de les diferents fases
    %               del proces.
    %
    %       lots - Aqui es guardara el nombre de lots que te el proces, si
    %               s'entren les dades ja sigui BW , VB o 3D s'ha de tenir
    %               en compte que aquest parametre s'actualitzara sol.
    %               Tambe es pot fixar manualment si fos necessarli.
    %
    %       data - Aqui s guarden les dades del proces en format 3D
    %
    %       class - Es on es guarda la classe de cada batch en format
    %               numeric 1 ... n
    %
    %       nomclass - Aqui es guarda l'equivalencia, si cal, de la classe
    %               numerica amb el seu text corresponent.
    %
    
%-------------------------------------------------------------------------%
%% ----------------------------- VARIABLES ------------------------------- 
%-------------------------------------------------------------------------%
    properties (Access = private) %Per defecte son privats posar (Access = public),(SetObservable = true),(SetObservable = false),(Access = private) segons les nostres necesitats
        % Propietats privades del nostre objecte
        nom; %nom del proces - nomes s'hi pot posar texte - no importa si no esta inicialitzat
        variables; % sera un vector de cell
        fases;%nom de la fase
        durafases;%[100 20 50 30] 4 fases i 100 20 50 30 mostres cada una
        lots;%1 si no es una Matriu 3D
        reflot;%sera el nom de cada lot
        data;%dades
        class;%numero de classe 1...N
    end
    
%-------------------------------------------------------------------------%
%% ----------------- DEFINICIO DE METODES DE LA CLASSE --------------------
%-------------------------------------------------------------------------%
    methods%Els metodes son publics per defecte
        
%-------------------------------------------------------------------------%
%% -------------------------- CONSTRUCTORS --------------------------------
%-------------------------------------------------------------------------%
        %constructor per defecte
        function obj =DATA()
            %El constructor per defecte: Aquest es el metode constructor
            %de l'objecte DATA. Per defecte es fixen  tots els valors a
            %null esperant que l'usuari entri les dades que realment hi han
            %d'anar, podriem dir que nomes reservem l'espai per l'objecte.
            %Les dades s'inicialitzen a -1 per poder identificar facilment 
            %si hi ha hagut algun errors inicialitzant-les. 
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Retorna un objecte DATA inicialitzat per defecte
            %
            
            %DADES
            obj.nom=[];
            obj.fases=[];
            obj.durafases=[];
            obj.lots=0;%1 si no es una Matriu 3D <- posem a 0 per evitar problemes si ajunten un obj buit amb un de ple
            obj.reflot={};
            obj.data=[];%dades
            obj.class=[];
        end
        
        %funci per fer copies de la nostra estructura
        function copy = copy(obj)
            % Aquest constructor far una copia del nostre objecte i la 
            %retorna per referencia. No es diu com la classe ja que el
            %Matlab no permet el sobrecarregament de metodes d'un objecte.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   copy - Es un objecte DATA, copia de l'actual.
            %
            copy = eval(class(obj));
            meta = eval(['?',class(obj)]);
            for p = 1: size(meta.Properties,1)
                pname = meta.Properties{p}.Name;
                try
                    eval(['copy.',pname,' = obj.',pname,';']);
                catch exeption
                    %hi ha agut algun error, en mostrem el numero i el
                    %que implica.
                    fprintf('\nERROR: ');
                    fprintf(exeption);
                    fprintf(['\nDades/Copy: Could not copy ',pname,'.\n']);
                end
            end
        end

        
%-------------------------------------------------------------------------%
%% ------------------ GETTERS, SETTERS & ALTRES ---------------------------
%-------------------------------------------------------------------------%

        %% ->nom
        function setName(obj,nom)
            %fixa el nom, ha de ser de tipus char
            %
            %Parametres d'entrada:
            %
            %   nom - s el nou nom de l'objecte
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            %mirem si te el format correcte
            z=whos('nom');
            if(strcmp( z.class , 'char'))
                obj.nom = nom;%actualitzem l'objecte
            else
                error('DATA/setName: nom must be char type')
            end
        end
        function obji = getName(obj)
            %Retorna el nom
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obji - retorna el que l'usuari hagi entrat com a nom
            %
            
            obji = obj.nom;
        end
        
        %% ->Dimensions
        function obji = getBatchCount(obj)
            %Retorna el numero de lots de les nostres dades
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obji - es un double, correspon al nombre de lots del nostre
            %         objecte
            %
            
            obji = obj.lots;%tornem els lots
        end
        function num = getVariableCount(obj)
            %Retorna el numero de variables de les nostres dades
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   num - es un double, correspon al nombre de variables del
            %         nostre objecte
            %
            
            if(isempty(obj.data))%si no tenim dades tornem numero de variables 0
                num=0;
            else
                num = size(obj.data,2);%el calculem
            end
        end
        function num = getSampleCount(obj)
            %Retorna el numero de instants de temps de les nostres dades
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   num - es un double, correspon al nombre de instants de
            %         temps del nostre objecte
            %
            if(isempty(obj.data)) %si no hi ha dades tornem 0
                num=0;
            else
                num = size(obj.data,1);%el calculem
            end
        end
        
        %% ->variables
        function setVariableLabel(obj,variables)
            %Fixara els noms de les variables. Aquests son els noms de les
            %diferents variables, se n'ha d'entra 1 per variable a poder
            %ser en el mateix ordre que els teim a les dades si no l'usuari
            %es podria confondre
            %
            %Parametres d'entrada:
            %
            %   variables - s el nou vector de noms de les variables.
            %               Aquest vector ha de ser una cellstring
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            %primer comprobem que ens han passat
            z=whos('variables');
            if(strcmp( z.class , 'cell'))
                if(z.size(2) > 1 && z.size(1)==1)
                    %si es un cell en format columna ya esta be
                    obj.variables = variables;
                else
                    if(z.size(1) > 1 && z.size(2)==1)
                        %si es un cel en format fila cal fer la transposta
                        obj.variables = variables';
                    else
                        %error('DATA/setVariableLabel: El camp variables ha de ser un vector')
                        obj.variables = variables;%si hi ha nomes una variable!!
                    end
                end
            else
                error('DATA/setVariableLabel: variables must be cell type')
            end
        end
        function obj = getVariableLabel(obj, i)
            %Retorna els noms de les variables.
            %
            %Parametres d'entrada:
            %
            %   i - (opcional) per indicar quines classes bolem
            %
            %Parametres de sortida:
            %
            %   obj - es el vector de noms de les variables (format columna)
            %
            
            if nargin > 1 %si indiquen i bolen nomes un subconjunt dels labels
                try
                    obj = obj.variables(i);%fem la indexaci que demanen
                catch err
                    %si hi ha un error en la indexaxi ho indiquem
                    error(['DATA\class:' err.message]);
                end
            else
                obj = obj.variables;%retornem tots els noms
            end
            
            
        end
        
        %% ->Referencialot
        function setBatchLabel(obj, in)
            %Fixara els noms dels lots. Aquests son els noms dels diferents
            %lots de les nodtres dades, se n'ha d'entra 1 per lot en el
            %mateix ordre en que els teim a les dades
            %
            %Parametres d'entrada:
            %
            %   variables - s el nou vector de noms dels lots.
            %               Aquest vector ha de ser un cellstring
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            %per evitar errors no podrem assignar noms als lots si no
            %existeixen lots
            if isempty(obj.data)%comprovem si hi ha dades
                error('DATA/setBatchLabel: dades is empty, set dades first.');
            end
            
            %comprovem el format de les dades entrades per l'usuari
            z=whos('in');
            if(not(strcmp( z.class , 'cell')))%ha de ser un cell
                error('DATA/setBatchLabel: param type is not valid, waiting for cell.');
            end
            if(size(in,1)~=1) && (size(in,2)~=1)%ha de ser un vector (no una matriu)
                error('DATA/setBatchLabel: class size is not valid, waiting for a vector');
            end
            
            if length(in)~=obj.getBatchCount() %ha de coincidir amb el numero de lots existents
                error('DATA/setBatchLabel: number of batch and classes must be the same');
            end
            
            
            %guardem en format fila
            if size(in,2)<size(in,1)
                obj.reflot=in';%guardem els noms
            else
                obj.reflot=in;%guardem els noms
            end
        end
        function out=getBatchLabel(obj,i)
            %retorna el nom dels lots. Aquests son els noms dels diferents
            %lots de les nodtres dades. Retornarem tots o nomes els que
            %s'ens indiqui.
            %
            %Parametres d'entrada:
            %
            %   i - (opcional)es l'index dels lots que volem recuperar-ne
            %       el nom, si no s'indica retornarem tots els noms
            %
            %Parametres de sortida:
            %
            %   out - s un cell string amb els noms indicats, sempre sera
            %         en format fila.
            %
            
            if nargin > 1
                %cas que hi hagi parametre indicant quins noms volem
                try
                    out=obj.reflot(i);
                catch err
                    error('DATA\getBatchLabel: this index is not valid');
                end
            else
                %tornem tot
                out=obj.reflot;
            end
        end
        
        %% ->data
        %per carregar dades en format 3D
        function setRawData(obj,data)
            %Carrega la matriu de dades, aquest metode carregra les dades
            %directament a l'atribut dades del nostre objecte cal vigilar
            %molt be el format que tenen les dades si no pot portar
            %problemes.
            %Format correcte:
            % dim(1)=Temps (Observacions)
            % dim(2)=Variables
            % dim(3)=Batch
            %          _______
            %         |______ |
            %        |______ ||
            %       |______ |||
            %   |  |       ||||
            % Temps|       |||  /
            %   |  |       || Batch
            %      |_______|  /
            %      - Vars -
            %
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %   data - Es una matriu de doubles amb el format descrit
            %          anteriorment. Aquesta matriu sera les noves dades de
            %          l'objecte actual.
            %
            %Parametres de sortida:
            %
            %   *Aquest metode no te parametres de sortida,es un metode
            %   que modificara el nostre objecte.
            
            %Primer de tot comprovem que les dades siguin doubles
            z=whos('data');
            if(not(strcmp( z.class , 'double')))%dades tipus correcte?
                error('DATA/setRawData: data must be double type')
            end
            
            %nomes ens queda asignar les dades i actualitzar lots.
            obj.data=data;%asignem les noves dades
            obj.lots=size(data,3);%asignem el nou nombre de lots
        end
        %per carregar dades en Bach Wide
        function setRawDataBW(obj,data,vars,temps,lots)
            %Carrega la matriu de dades, ho fara per referencia per
            %evitar duplicats. S'espera una matriu desdoblada en Bach wide,
            %pertant ha de ser una matriu 2D.i el format esperat s:
            % dim(1)=Batch
            % dim(2)=Variables*Temps(Observacions)
            %
            %       _______ _______ _______ _______
            %   |  |  V1   |  V2   |  V3   |  V4   |
            % Batch|       |       |       |       | ....
            %   |  |       |       |       |       |
            %      |_______|_______|_______|_______|
            %               - Vars - * -Temps -
            %
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %   data - Ha de ser una matriu 2D de doubles, son les dades
            %           que bolem posar al nostre objecte.
            %
            %   vars - Es el nombre de variables del proces tambe com a
            %           double.
            %   temps - Es el nombre de mostres al llarg del temps del
            %           nostre proces.
            %   lots - numero de lots que ens han entrat
            %
            %Parametres de sortida:
            %
            %   *Aquest metode no te parametres de sortida,es un metode
            %   que modificara el nostre objecte.
            
            %primer de tot comprobem el tipus de dades
            z=whos('data');
            if(strcmp( z.class , 'double'))
                %guardem les dades que ens han passat, les transformem a 3D
                %les guardem a obj.data i actualitzem lots.
                aux=zeros(temps,vars,lots);
                for i=1:lots
                    aux(:,:,i)=reshape(data(i,:)',[temps vars]);
                end
                obj.data=aux;
                obj.lots=lots;
            else
                error('DATA/setRawDataBW: data must be double type')
            end
        end
        %per carregar dades en Variable Wide
        function setRawDataVW(obj,data,vars,temps,lots)
            %Carrega la matriu de dades,  ho fara per referencia per evitar
            %duplicats. S'espera una matriu 2D en format Variable Wide. Les
            %dades les esparem amb el seguent format:
            % dim(1)=Batch*Temps(Observacions)
            % dim(2)=Variables
            %
            %       _______
            %   |  |       |
            %   |  |       |
            %   |  |       |
            %   |  |_______|
            %   |  |       |
            %   |  |       |
            %   |  |       |
            % Bath |_______|
            %   *  |       |
            % Temps|       |
            %   |  |       |
            %   |  |_______|
            %   |  |       |
            %   |  |       |
            %   |  |       |
            %   |  |_______|
            %       -Vars-
            %
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %   data - Ha de ser una matriu 2D de doubles, son les dades
            %           que bolem posar al nostre objecte.
            %
            %   vars - Es el nombre de variables del proces tambe com a
            %           double.
            %   temps - Es el nombre de mostres al llarg del temps del
            %           nostre proces.
            %   lots - numero de lots que ens han entrat
            %
            %Parametres de sortida:
            %
            %   *Aquest metode no te parametres de sortida,es un metode
            %   que modificara el nostre objecte.
            
            %primer de tot comprobem el tipus de dades
            z=whos('data');
            if(strcmp( z.class , 'double'))
                %guardem les dades que ens han passat, les transformem a 3D
                %les guardem a obj.data i actualitzem lots.
                obj.data=reshape(data',[vars temps lots]);
                permute(obj.data,[2 1 3]);
                obj.lots=lots;
            else
                error('DATA/setRawDataVW: data must be double type')
            end
        end
        %Retorna les dades transformades a 2D
        function obji = getRawData(obj,metode)
            %Retorna les dades transformades a 2D utilitzant bach wide o
            %variable wide segons el metode indicat.
            %
            % Si metode = 0 o 'BW' Les dades es retornaran en format Bach wide
            % Si metode = 1 o 'VW' Les dades es retornaran en format Variable wide
            %
            %Parametres d'entrada:
            %
            %   metode - pot ser 1 , 0 , 'VW' o 'BW' i ens indica el format de converci
            %           a utilitzar
            %
            %Parametres de sortida:
            %
            %   obji - aquest metode retorna una matriu de doubles
            %         corresponent a l'unfolding de la matriu 3D on
            %         s'enmagatzemen les dades internament.
            
            %per usuaris humans permetem pas de parametres com 'VW' i 'BW'
            if ischar(metode)
                if strcmpi(metode,'VW')
                    metode = 1;
                end
                if strcmpi(metode,'BW')
                    metode = 0;
                end
            end
            
            if(metode~=1 && metode~=0)%ha de ser 1 o 0 si no error
                error('DATA/getRawData: metode must be 1 , 0 , ''VW'' or ''BW''')
            end
            %nomes cal fer la crida al metode privat que ho fa tot
            obji = to2D(obj,metode);
        end
        %Retorna les dades en 3D
        function obji = getRawData3D(obj)
            %Aquest metode retorna les dades internes de l'objecte, ser
            %una matriu 3D a menys que nomes tinguem un batch. Les dades
            %les retornarem com una matriu 3D amb les dades organitzades de
            %la seguent manera:
            %
            % dim(1)=Temps (Observacions)
            % dim(2)=Variables
            % dim(3)=Batch
            %          _______
            %         |______ |
            %        |______ ||
            %       |______ |||
            %   |  |       ||||
            % Temps|       |||  /
            %   |  |       || Batch
            %      |_______|  /
            %      - Vars -
            %
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te parametres d'entrada
            %
            %Parametres de sortida:
            %
            %   obji - aquest metode retorna una matriu de doubles
            %         corresponent a la matriu 3D on s'enmagatzemen les
            %         dades internament.
            
            obji = obj.data;%retornem la matriu tal com la tenim en memoria
        end
        
        %% ->fases
        function setStages(obj,data)
            %Funci per entrar les fases del proces.
            %
            %Parametres d'entrada:
            %
            %   data - seran les noves fases del proces, ha de ser un
            %           vector de cell.
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            %comprovem que el format dels parametres sigui el correcte
            z=whos('data');
            if(strcmp( z.class , 'cell'))
                obj.fases=data;%actualitzem l'objecte
            else
                disp('DATA/setStages: data must be cell type')
            end
        end
        function obji = getStages(obj)
            %Retorna les fases del proces.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obji - es un vector de cells, corresponens a les fases
            %
            
            obji=obj.fases;
        end
        
        %% ->durafases
        function setStagesDurations(obj,data)
            %Funci per fixar la duraci de les fases.
            %
            %Parametres d'entrada:
            %
            %   data - seran les noves durccions de les fases del proces,
            %           han de ser un vector de doubles de la mateixa
            %           longitud que el nombre de fases del proces.
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            %Comprobem que el parametre sigui l'esperat
            z=whos('data');
            if(strcmp( z.class , 'double'))
                if size(data,2)<size(data,1)
                    obj.durafases=data';%actualitzem l'objecte
                else
                    obj.durafases=data;%actualitzem l'objecte
                end
            else
                disp('DATA/setStagesDurations: data must be a vector')
            end
        end
        function obji = getStagesDurations(obj)
            %retorna la duracio actual de les fases.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obji - es un vector de doubles, corresponens a les
            %          duracions de les fases
            %
            obji=obj.durafases;
        end
        
        %% ->classes
        function setClass(obj,data)
            %Funci per fixar les classes de cada batch. Internament les
            %classes seran doubles, per l'usuari se li permet entrar cell
            %amb text sobre els quals es fara la conversi.
            %
            %Parametres d'entrada:
            %
            %   data - seran les noves classes del nostre proces actual,
            %           han de ser un vector de doubles de la mateixa
            %           longitud que el nombre de fases del proces. O tambe
            %           pot ser un vector de cells amb text.
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            if isempty(obj.data)
                error('DATA/setClass:before setting up the class first set the dades');
            end
            
            %Primer comprovem si les dades son correctes
            z=whos('data');
            if(not(strcmp( z.class , 'double')))
                error('DATA/setClass: invalid type, waiting for double.');
            end
            if(size(data,1)~=1) && (size(data,2)~=1)
                error('DATA/setClass: invalid class size, waiting for vector');
            end
            if length(data)~=obj.getBatchCount()
                error('DATA/setClass: batch count number and class count number must be de same');
            end
            
            %posem en format fila
            if size(data,2)<size(data,1)
                obj.class=data';%guardem les classes
            else
                obj.class=data;%guardem les classes
            end
        end
        function obji = getClass(obj, i)
            %retorna la classe de cada batch o del\s batch idexats per i
            %
            %Parametres d'entrada:
            %
            %   i - (opcional) per indicar quines classes bolem
            %
            %Parametres de sortida:
            %
            %   obji - es un vector de doubles en format fila, corresponens
            %          a les classes de cada batch
            %
            
            obji=obj.class;
            if nargin > 1
                try
                    %si es un string o cellstring hem de recuperar les
                    %posicions reals dels lots a que es trfereixen
                    z=whos('i');
                    if (strcmp( z.class , 'char')  && ~strcmp(i,':'))|| iscellstr(i)
                        %recuperem els noms dels lots
                        auxnoms = obj.getBatchLabel;
                        
                        if isempty(auxnoms)
                            %si es buit bol dir que aquest objecte no se li han
                            %assignat noms de lots -> retornem un error
                            error('DATA/getClass: You are indexing using batch names and this object has no batch names!')
                        end
                        
                        if iscellstr(i)
                            %si es un cell string cal fer un bucle i anar
                            %recuperant les posicions dels lots 1 a 1
                            index = [];
                            for j=1:length(i)
                                aux = find(strcmp(auxnoms,i{j}));
                                if isempty(aux)
                                    %si no existeix el lot que ens demanen mostrem
                                    %warning
                                    warning('DATA/getClass: You gave an inexistent batch name!')
                                else
                                    %ens apuntem el lot que ens demanen
                                    index = [index aux];
                                end
                            end
                            i=index;%posem a i els index dels lots que hem anat trobant
                        else
                            
                            i = find(strcmp(auxnoms,i));
                            
                            %si no hem trobat el lot mostrem un warning!
                            if isempty(i)
                                warning('DATA/getClass: You gave an inexistent batch name!')
                            end
                        end
                    end
                    
                    obji=obji(i);
                catch err
                    error('DATA\getClass:Lindex s incorrecte');
                end
            end
        end
        
        %% ->Afegir/Treure Casos        
        %Ajunta 2 objectes dades
        function obji = merge(obj,dad)
            %Aquesta funci ajuntara 2 objectes dades i retornara un 3r
            %objecte format per la unio dels 2, si cridem x.merge(DATA) x
            %sera l'objecte principal, d'ell s'agafara el nom, temps,
            %duracio de fases... de DATA nomes utilitzarem les dades i les
            %classes.
            %                                                      _______
            % dim(1)=Temps                                        |______ |
            % dim(2)=Variables                                   |______ ||
            % dim(3)=Batch                                      |______ |||
            %         _______              _______             |______ ||||
            %        |______ |            |______ |           |______ ||||
            %       |______ ||           |______ ||          |______ ||||
            %      |______ |||   +      |______ |||     =   |______ ||||  /
            %  |  |       ||||      |  |       ||||     |  |       ||||  /
            %Temps|       |||  /  Temps|       |||  / Temps|       |||Batch
            %  |  |       ||Batch   |  |       ||Batch  |  |       ||  /
            %     |_______|  /         |_______|  /        |_______|  /
            %     - Vars -             - Vars -             - Vars -
            %
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %   dad - Ha de ser un objecte DATA, sera l'objecte secondari
            %           el primari es l'actual.
            %
            %Parametres de sortida:
            %
            %   obji - Es el nou objecte resultat de ajuntar l'actual amb
            %           dad.
            %
            
            %primer de tot comprobem si ens han pasat els parametres correctes
            z=whos('dad');
            if(strcmp( z.class , 'DATA'))
                %creem el nou objecte
                obji=DATA();
                %agafem les dades de l'actual i les copiem al nou objecte dades
                %que aki anomenem obji.
                obji.nom=obj.nom;
                %Per poder-se unir han de tenir les mateixes variables (Han de
                %ser diferents captures d'un mateix proces)
                obji.variables=obj.variables; %son el mateix proces -> tenen les mateixes variables
                obji.fases=obj.fases;%son el mateix proces -> tenen les mateixes fases
                obji.durafases=obj.durafases;%son el mateix proces -> tenen les mateixes duracions de fase
                %El que fem es ajuntar les 2 matrius de dades, per fer-ho les
                %ajuntem per la dimensi 3(Batch) per tant el nombre de lots
                %caldra actualitzar-lo.

                %no ens preocupem si hi ha dades repetides, simplement ajuntem les matrius
                obji.data=cat(3,obj.data, dad.data);%dades
                obji.class=[obj.class, dad.class];%unim les classes
                obji.lots=size(obji.data,3);%actualitzem els lots
                
                obji.reflot=[obj.reflot, dad.reflot];

           else
                disp('DATA/merge: dad must be an object of DATA class')
            end
        end
        %Afegir 1 o varis lots
        function addBatch(obj,dad, clase,ref)
            %Aquesta funci afegeix un o diversos Batch a les nostres
            %dades. Les mides del lot han de ser concordants amb les de la
            %resta de l'estructura que tenim.
            %                                                      _______
            % dim(1)=Temps                                        |______ |
            % dim(2)=Variables                                   |______ ||
            % dim(3)=Batch                                      |______ |||
            %         _______              _______             |______ ||||
            %        |______ |            |______ |           |______ ||||
            %       |______ ||           |______ ||          |______ ||||
            %      |______ |||   +      |______ |||     =   |______ ||||  /
            %  |  |       ||||      |  |       ||||     |  |       ||||  /
            %Temps|       |||  /  Temps|       |||    Temps|       |||Batch
            %  |  |       ||Batch   |  |       ||       |  |       ||  /
            %     |_______|  /         |_______|           |_______|  /
            %     - Vars -             - Vars -             - Vars -
            %
            %      Actual                 dad                 Resultat
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %    dad - matriu de doubles 2D o 3D
            %
            %    clase -  es un vector de doubles
            %
            %    ref - (opcional) es la referencia o nom del lot
            %
            %Parametres de sortida:
            %
            %   No en te
            %

            %mirem si les dades son correctes
            z=whos('dad');
            if(not(strcmp( z.class , 'double')))
                error('DATA/afegeix:dad has an invalid class, waiting double.');
            end
            % les mides son correctes?
            if ~isempty(obj.data)
            if(not(z.size(1)==size(obj.data,1) && z.size(2)==size(obj.data,2)))%ha de ser un vector columna
                error('DATA/afegeix: dad batch size is not the same than actual batch size')% format incorrecte
            end 
            end
            
            %comprovem que la classe del segon operant sigui correcte
            z=whos('clase');
            if(not(strcmp( z.class , 'double')))
                error('DATA/afegeix: class type not valid, waiting for double.');
            end
            
            if(z.size(1)==1 && z.size(2)>=1)%vector fila
                obj.class=[obj.class, clase];%afegim el nou element/s a derrera
            elseif(z.size(1)>=1 && z.size(2)==1)%vector columna
                obj.class=[obj.class, clase'];%afegim el nou element/s a derrera fent transpost internament tenim un vector fila
            else
                error('DATA/afegeix:classe size is not valid, waiting for vector.');
            end
            %afegim el lot a les dades
            obj.data=cat(3,obj.data, dad);%els afegim darrere per la 3a dimenci
            obj.lots=size(obj.data,3);%actualitzem el numero de lots
            if nargin == 4
               obj.reflot=[obj.reflot ref];
            end
        end
        %treure 1 o varis lot
        function deleteBatch(obj,i)
            %Treu els lots indexats per ''i". ''i" pot ser un double, o un
            %logical en format vector de 1 a n elements on n es el numero
            %de lots. O qualsebol format valid en indexacions al Matlab.
            %
            %
            % dim(1)=Temps
            % dim(2)=Variables
            % dim(3)=Batch
            %         _______
            %        |______ |                      _______
            %       |______ ||                     |______ |
            %      |______ |||                    |______ ||
            %  |  |       ||||     - [1] =    |  |       |||
            %Temps|       |||  /            Temps|       |||
            %  |  |       ||Batch             |  |       ||
            %     |_______|  /                   |_______|
            %     - Vars -                        - Vars -
            %
            %      Actual          index          Resultat
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %    i - els lots que es bol treure
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            
            %Comprovem que el tipus de dades sigui el correcte
            z=whos('i');
            % es double ?
            if(not(strcmp( z.class , 'double')) && not(strcmp( z.class , 'logical')) && not(strcmp( z.class , 'char')) && ~iscellstr(i))
                error('DATA/treu: waiting for double or logical')
            end
            % es vector columna o 1*1?
            if(length(z.size())>2 && (z.size(1)==1 || z.size(2)==1))%ha de ser un vector
                error('DATA/treu: waiting for vector')% format incorrecte
            end
            
            %si es un string o cellstring hem de recuperar les posicions reals dels lots a que es trfereixen
            if (strcmp( z.class , 'char')  && ~strcmp(i,':'))|| iscellstr(i)  
                %recuperem els noms dels lots
                auxnoms = obj.getBatchLabel;
                
                if isempty(auxnoms)
                    %si es buit bol dir que aquest objecte no se li han
                    %assignat noms de lots -> retornem un error
                    error('DATA/treu: You are indexing using batch names and this object has no batch names!')
                end
                
                if iscellstr(i)
                    %si es un cell string cal fer un bucle i anar
                    %recuperant les posicions dels lots 1 a 1
                    index = [];
                    for j=1:length(i)
                        aux = find(strcmp(auxnoms,i{j}));
                        if isempty(aux)
                            %si no existeix el lot que ens demanen mostrem
                            %warning
                            warning('DATA/treu: You gave an inexistent batch name!')
                        else
                            %ens apuntem el lot que ens demanen
                            index = [index aux];
                        end
                    end
                    i=index;%posem a i els index dels lots que hem anat trobant
                else
                    
                    i = find(strcmp(auxnoms,i));
                    
                    %si no hem trobat el lot mostrem un warning!
                    if isempty(i)
                        warning('DATA/treu: You gave an inexistent batch name!')
                    end
                end
            end
            
            %format correcte
            
            x=logical(1:obj.lots);%x es un vector de 1 logics de longitud = al numero de lots
            x(i)=0;%posem a 0 els que bolen eliminar
            if(length(x)<=obj.lots)%comprovem el tamany del vector que ens han passat / o la posicio que volien eliminar
                obj.data=obj.data(:,:,x);%treiem els que cal de les dades
                obj.class=obj.class(x);%treiem els que cal de les classes
                if not(isempty(obj.reflot))
                    obj.reflot=obj.reflot(x);
                end
                obj.lots=size(obj.data,3);%Actualitzem els lots
            else
                error('DATA/treu: batch index out of bounds')
            end
        end
        %retorna el lot/lots indicats
        function [data clas noms]=getBatch(obj, i)
            %retorna el lot o lots indicats per el parametre d'entrada 'i',
            %aquest por ser qualsebol tipus d'indexaci valida en matlab.
            %
            %              _______
            %             |______ |
            %            |______ ||
            %           |______ |||
            %          |______ |||| - 8
            %         |______ |||| - 7
            %        |______ |||| - 6
            %       |______ |||| - 5   /
            %   |  |       |||| - 4   /
            % Temps|       ||| - 3 Batch
            %   |  |       || - 2   /
            %      |_______| - 1   /
            %      - Vars -
            %
            %
            %Parametres d'entrada:
            %
            %    i - index del lot/s que es vol recuperar
            %
            %Parametres de sortida:
            %
            %    data - matriu de doubles 2D son les dades del lot
            %
            %    clas -  es un double i correspon a la classe del lot
            %
            %    noms - es el nom del batch
            %
            
            %Comprovem que el tipus de dades sigui el correcte
            z=whos('i');
            % es double ?
            if(not(strcmp( z.class , 'double')) && not(strcmp( z.class , 'logical')) && not(strcmp( z.class , 'char')) && ~iscellstr(i))
                error('DATA/treu: Waiting for double, logical, lot identificator(char)')
            end
            % es vector columna o 1*1?
            if(length(z.size())>2 && (z.size(1)==1 || z.size(2)==1))%ha de ser un vector
                error('DATA/treu: Waiting for vector')% format incorrecte
            end
            
            %si es un string o cellstring hem de recuperar les posicions reals dels lots a que es trfereixen
            if (strcmp( z.class , 'char')  && ~strcmp(i,':'))|| iscellstr(i)  
                %recuperem els noms dels lots
                auxnoms = obj.getBatchLabel;
                
                if isempty(auxnoms)
                    %si es buit bol dir que aquest objecte no se li han
                    %assignat noms de lots -> retornem un error
                    error('DATA/treu: You are indexing using batch names and this object has no batch names!')
                end
                
                if iscellstr(i)
                    %si es un cell string cal fer un bucle i anar
                    %recuperant les posicions dels lots 1 a 1
                    index = [];
                    for j=1:length(i)
                        aux = find(strcmp(auxnoms,i{j}));
                        if isempty(aux)
                            %si no existeix el lot que ens demanen mostrem
                            %warning
                            warning('DATA/treu: You gave an inexistent batch name!')
                        else
                            %ens apuntem el lot que ens demanen
                            index = [index aux];
                        end
                    end
                    i=index;%posem a i els index dels lots que hem anat trobant
                else
                    
                    i = find(strcmp(auxnoms,i));
                    
                    %si no hem trobat el lot mostrem un warning!
                    if isempty(i)
                        warning('DATA/treu: You gave an inexistent batch name!')
                    end
                end
            end
            
            %Recuperem els lots que ens demanen!
            data=obj.data(:,:,i);
            
            %les classes
            if isempty(obj.class)
                clas =[];
            else
                clas=obj.class(i);
            end
            
            
            %i les referencies
            if isempty(obj.reflot)
                noms={};
            else
                noms=obj.reflot(i);
            end
            
        end
        
        %% -> Barrejar els lots
        function index=shuffle(obj,index)
            %Funci per barrejar els lots de la base de casos aleatoriament
            %o be amb els indexs que ens passa l'usuari
            %
            %Parametres d'entrada:
            %
            %    index - vector que indica per cada lot la nova posici
            %            desitjada si no existeix s'assigna aleatoriament
            %
            %Parametres de sortida:
            %
            %    index - vector de doubles qu indica la posici antiga dels lots
            %
            
           
            if nargin < 2%comprovem si ens passen index
                 %creem l'index
                index=randperm(obj.getBatchCount());
            else
                %controlem que l'index que ens passen sigui correcte
                if(size(index)~=obj.getBatchCount())%coincideix mida
                    error('DATA\shuffle: Invalid Index');
                end
                if sum(sort(index) == 1:obj.getBatchCount())~=obj.getBatchCount() %hi son tots
                    error('DATA\shuffle:Invalid Index');
                end
            end
            
            %fem la ordenaci corresponent amb l'index
            obj.data=obj.data(:,:,index);%a les dades
            
            %a les classes
            if isempty(obj.class)%comprovem si la classe existeix
                warning('DATA\shuffle:class is empty');
            else
                obj.class=obj.class(index);
            end
            
            %i a les referencies
            if isempty(obj.reflot)
                warning('DATA\shuffle:lot reference is empty');
            else
                obj.reflot=obj.reflot(index);
            end
        end
        %% ->Emplenar amb dades sintetiques
        function randomFill(obj,nvars,ntemps,nlots,nclass,nfases)
            %Random Data filler, this method fills all your DATA object atributes with
            %randomly generated data. For each generated variable a polinomial mean is
            %generated, in each batch we introduce a totaly random noise and a small
            %variation in the first coeficient of the polynomial equation.
            %All 
            
            %Eliminem totes les dades actuals
            obj.nom=[];
            obj.fases=[];
            obj.durafases=[];
            obj.lots=0;
            obj.reflot={};
            obj.data=[];
            obj.class=[];
            
            obj.setName('Objecte Aleatori!!');
            
            %generem les Dades
            warning('off','all');
            
            %Creem una mitjana per les dades
            
            %creem 100 puns a l'atzar els quals inter polarem
            %Magnitud del perfil
            magni = randi([-1000 1000],1);
            punts = rand([1,ntemps*3])*magni;
            
            for i = nvars:-1:1
                
                %interpolem amb polinomis de diferents graus aquests punts
                P = polyfit(1:ntemps*3,punts,randi([3 20],1));
                for j = nlots:-1:1
                    P(end)= P(end)+(rand*2)-1;% modifiquem el treme de x per introduir variacions als lots
                    %P(end-1)= P(end-1)+(rand * 2 -1)/100;% modifiquem el treme de x^2 per introduir variacions als lots
                    %P(end-3)= P(end-3)+(rand * 2 -1)/10000;% modifiquem el treme de x^3 per introduir variacions als lots
                    mitj(:,i,j) = polyval(P,1:ntemps);
                    %plot(1:ntemps,mitj(:,i,j));
                end
                
                %Canviem alguns dels punts
                punts(randi([1 ntemps*3],[1 ntemps*3]))= rand([1,ntemps*3])*magni;% 50 dels 200 puns que interpolem seran nous
                
            end
            
            mitj = mitj .* 100;%fem les variables mes grans
            
            
            %Afegim un soroll aleatori
            aux =  mitj + abs(rand([ntemps nvars nlots])*10)+0.00001;
            
            obj.setRawData(aux);
            warning('on','all');
            
            %creem els noms de les variables
            aux={};
            for i=nvars:-1:1
                aux{i} = ['Variable_' num2str(i)];
            end
            obj.setVariableLabel(aux);
            
            %creem els noms dels lots
            aux={};
            for i=nlots:-1:1
                aux{i} = ['lot_' num2str(i)];
            end
            obj.setBatchLabel(aux);
            
            
            %creem els noms de les fases
            aux={};
            for i=nfases:-1:1
                aux{i} = ['Fase_' num2str(i)];
            end
            obj.setStages(aux);
            
            %creem les duracions de les fases
            aux=[];
            for i=nfases:-1:2
                aux(i) =  randi(floor((ntemps-1)/nfases + 2)) ;
            end
            aux(1) = ntemps - sum(aux);
            obj.setStagesDurations(aux);
            
            %les classes
            obj.setClass(randi(nclass, [1 nlots]));
        end
            
        %% -> guardar/carregar dades
        function save(obj,fitxer,format)
            %Mtode que llegeix un fitxer per inicialitzar un objecte DATA.
            %El parmetre fitxer inclou el nom i extensio del fitxer (i si
            %cal , la ruta del directori on es troba la informacio). Per
            %una altra banda el parametre format" indica el format del
            %fitxer que s'ha de llegir. Els formats valids son:
            %
            %
            %"mat": guardar la informacio en format per defecte de Matlab,
            %aquest format obliga a usar la extensio de fitxer ".mat".
            %
            %"csv": guardar la informacio en un fitxer on els valors se
            %separen per comes De format propi.
            %
            %Parametres d'entrada:
            %
            %   fitxer - String amb el nom (i ruta del directori,si cal)
            %            amb la informacio a llegir.
            %
            %   format - Format del fitxer a llegir. Els formats valids
            %            son: \mat" i \csv".
            %
            
            switch upper(format)
                case 'MAT'
                    %cridem el save de matlab
                    save(fitxer,'obj');
                case 'CSV'
                    %creem el fitxer amb el format propi
                    fid=fopen(fitxer, 'w+');
                    fclose(fid);
                    aux=[{'Nom'} {obj.nom}];%sempre sera un string no hi ha problema
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[{'Variables'} {length(obj.variables)} ];% sempre seran strings nomes necesitem saver quantes seran
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    cell2csv(fitxer,obj.variables,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[{'Fases'} {length(obj.fases)} ];% sempre seran numeros nomes necesitem saver quantes seran
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    cell2csv(fitxer,obj.fases',',');
                    
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[{'Duracio de Fases'} {length(obj.durafases)}];% sempre seran numeros nomes necesitem saver quantes seran
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[];
                    for i = 1:length(obj.durafases)
                        aux= [aux {obj.durafases(i)}];
                    end
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[{'ref. Lots'} {length(obj.reflot)} ];
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    
                    cell2csv(fitxer,obj.reflot,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[ {'class'} {length(obj.class)}];% sempre seran numeros nomes necesitem saver quantes seran
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[];
                    for i = 1:length(obj.class)
                        aux= [aux {obj.class(i)}];
                    end
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    aux=[{'data'} {size(obj.data,1)} {size(obj.data,2)} {size(obj.data,3)}];% sempre seran numeros
                    cell2csv(fitxer,aux,',');
                    fid=fopen(fitxer, 'a+');
                    fprintf(fid, '\r\n');
                    fclose(fid);
                    dlmwrite(fitxer,obj.getRawData(1),'-append','newline','pc','delimiter',',')  % 0=BW 1=VW
            end
        end
        function load(obj,fitxer,format)
            %Metode que llegeix un fitxer per inicialitzar un objecte DATA.
            %El parametre fitxer" inclou el nom i extensio del fitxer(i si
            %cal , la ruta del directori on es troba la informacio). Per una
            %altra banda el parametre "format" indica el format del fitxer
            %que s'ha de llegir. Els formats valids son:
            %
            %
            %"mat": guardar la informacio en format per defecte de Matlab,
            %aquest format obliga a usar la extensio de fitxer ".mat".
            %
            %"csv": guardar la informacio en un fitxer on els valors se
            %separen per comes De format propi.
            %
            %Parametres d'entrada:
            %
            %   fitxer - String amb el nom (i ruta del directori,si cal)
            %            amb la informacio a llegir.
            %
            %   format - Format del fitxer a llegir. Els formats valids
            %            son: \mat" i \csv".
            %
            
            switch upper(format)
                case 'CSV'
                    fid=fopen(fitxer, 'r');
                    if fid <= 0
                        error('DATA\Load: error while opening file');
                    end
                    
                    %nom
                    s=fgetl(fid);
                    a=textscan(s,'%s','delimiter',',');
                    if not(strcmpi(a{1}(1),'NOM'))
                        error('DATA\Load: NOM marker is not there');
                    end
                    
                    if length(a{1})>1
                        obj.setName(a{1}{2});
                    else
                        obj.nom=[];
                    end
                    
                    
                    %variables
                    s=fgetl(fid);
                    a=textscan(s,'%s','delimiter',',');
                    if not(strcmpi(a{1}(1),'VARIABLES'))
                        error('DATA\Load: VARIABLES marker is not there');
                    end
                    
                    if length(a{1})>1
                        num=str2double(a{1}(2));
                    else
                        error('DATA\Load: after VARIABLES marker you must give a number');
                    end
                    
                    s=fgetl(fid);
                    if num >0
                        a=textscan(s,'%s','delimiter',',');
                        if length(a{1})~= num
                            error('DATA\Load: VARIABLES given number and found number must match');
                        end
                        obj.setVariableLabel(a{1});
                    else
                        obj.variables=[];
                    end
                    
                    
                    %fases
                    s=fgetl(fid);
                    a=textscan(s,'%s','delimiter',',');
                    if not(strcmpi(a{1}(1),'FASES'))
                        error('DATA\Load: FASES marker not found');
                    end
                    
                    if length(a{1})>1
                        num=str2double(a{1}(2));
                    else
                        error('DATA\Load: after FASES marker you must give a number');
                    end
                    
                    s=fgetl(fid);
                    if num >0
                        a=textscan(s,'%s','delimiter',',');
                        if length(a{1})~= num
                            error('DATA\Load: FASES given number and found number must match');
                        end
                        obj.setStages(a{1});
                    else
                        obj.fases=[];
                    end
                    
                    
                    %Duracio de fases
                    s=fgetl(fid);
                    a=textscan(s,'%s','delimiter',',');
                    if not(strcmpi(a{1}(1),'Duracio de fases'))
                        error('DATA\Load: Duracio de fases marker not found');
                    end
                    
                    if length(a{1})>1
                        num=str2double(a{1}(2));
                    else
                        error('DATA\Load:after Duracio de fases marker you must give a number');
                    end
                    
                    s=fgetl(fid);
                    if num >0
                        a=textscan(s,'%s','delimiter',',');
                        if length(a{1})~= num
                            error('DATA\Load: Duracio de fases given number and found number must match');
                        end
                        obj.setStagesDurations(str2double(a{1}));
                    else
                        obj.durafases=[];
                    end
                    
                    
                    % ref. lots
                    s=fgetl(fid);
                    a=textscan(s,'%s','delimiter',',');
                    if not(strcmpi(a{1}(1),'ref. lots'))
                        error('DATA\Load:ref. batch marker not found');
                    end
                    
                    if length(a{1})>1
                        num=str2double(a{1}(2));
                    else
                        error('DATA\Load:after ref. batch marker you must give a number');
                    end
                    
                    s=fgetl(fid);
                    if num >0
                        a=textscan(s,'%s','delimiter',',');
                        if length(a{1})~= num
                            error('DATA\Load: ref. batch given number and found number must match');
                        end
                        nomlots=a{1};%Setnomlot s'ha de fer despres d'entrar les dades!! nomes ens guardem el valors de moment
                    else
                        obj.reflot=[];
                        nomlots=-1;
                    end
                    
                    %classes
                    s=fgetl(fid);
                    a=textscan(s,'%s','delimiter',',');
                    if not(strcmpi(a{1}(1),'class'))
                        error('DATA\Load:class marker not found');
                    end
                    
                    if length(a{1})>1
                        num=str2double(a{1}(2));
                    else
                        error('DATA\Load:after duracions de fases marker you must give a number');
                    end
                    
                    s=fgetl(fid);
                    if num >0
                        a=textscan(s,'%s','delimiter',',');
                        if length(a{1})~= num
                            error('DATA\Load: duracions de fases given number and found number must match');
                        end
                        classes=str2double(a{1});
                    else
                        obj.durafases=[];
                        classes=-1;
                    end
                    
                    %DATA
                    s=fgetl(fid);
                    a=textscan(s,'%s','delimiter',',');
                    if not(strcmpi(a{1}(1),'data'))
                        error('DATA\Load:data marker not found');
                    end
                    
                    if length(a{1})==4
                        tmps=str2double(a{1}(2));
                        var=str2double(a{1}(3));
                        lot=str2double(a{1}(4));
                    else
                        error('DATA\Load:data marker format is data,time,variables number,batch number,');
                    end
                    
                    if(tmps == 0 && var==0)
                        obj.data=[];
                    else
                        M=dlmread(fitxer,',',12,0);
                        Z=reshape(M',var,tmps,lot);
                        obj.data=permute(Z,[2 1 3]);
                        obj.lots=size(obj.data,3);
                        
                        if iscell(nomlots)
                            obj.setBatchLabel(nomlots);
                        end
                        if classes ~= -1
                            obj.setClass(classes);
                        end
                    end
                    case 'MAT'
                        %cridem el load de matlab
                         load(fitxer);
            end
        end
        
%-------------------------------------------------------------------------%
%% -------------------- Visualitzaci de l'objecte -----------------------
%-------------------------------------------------------------------------%    
        %sobreescriptura event de dobleclick sobre un objecte DATA
        function openvar(nom,~)
            %Aquesta es una funci que sobreescriu l'event per defecte de
            %doble click sobre un objecte de tipus DATA en el workspace.
            %sobreescrivim la funci de visualitzaci per defecte,
            %el parametre ignorat s l'objecte actual el nom ens el passen
            %en format text. Aqui invocarem una pantalla especifica per
            %visualitzar l'objecte.
            %
            %!!ATENCIO!! -> Aquest metode no hauria de ser invocat per
            %l'usuari directament a menys que sapiga molt b que esta fent!
            
            x=whos('nom');
            if strcmp(x.class,'DATA')
                %no puc saber el nom de la variable -> no fem res!
            else
                if(evalin('base',['size(' nom ');']) == [1 1])
                    Menu_Editar_Dades2(nom)
                else
                    %arrays, no podem obrir-los amb la UI!!
                end
            end
        end
        %visualitzaci de l'objecte per consola i menus desplegables de Matlab
        function disp(obj,~)
            %funci que sobreescriu la visualitzaci de l'objecte per
            %consola i en alguns menus desplegables de matlab.
            %Mostrarem informaci util per l'usuari (tamany de les dades)
            %en comptes de la informaci per defecte.
            %
            % Aquesta funci s'invoca automaticament per Matlab quan cal
            % l'usuari no necesita invocarla en cap moment.
            
            %sobre escric la visualitzaci de l'objecte
            if nargin ~=2
                disp(['nom = ' obj.nom])
            end
            disp(['Num_temps = ' num2str(obj.getSampleCount())])
            disp(['Num_vars = ' num2str(obj.getVariableCount())])
            disp(['Num_batch = ' num2str(obj.getBatchCount())])
        end
    end
    
%-------------------------------------------------------------------------%
%% --------------------------- Privades -----------------------------------
%-------------------------------------------------------------------------%
    
    methods(Access = private)
        %converteix la matriu a 2D
        function x = to2D(obj,metode)
            %Metode privat de la classe DATA que converteix la matriu de
            %dades interna a 2D, si metode es 1 fara un unfolding per
            %variable wide si es 0 el fara per bach wide.
            %
            %Parametres d'entrada:
            %
            %   metode - pot ser 1 o 0, 1= VW i 0= BW
            %
            %Parametres de sortida:
            %
            %   x -  son les dades resultants de l'unfolding.
            %
            
            %recuperem el tamany de les dades
            z=size(obj.data);
            d=length(z);
            if(d==3)
                %dades de 3D
                if(metode == 0)
                    %metode escollit es BW
                    x=reshape(obj.data,z(2)*z(1),z(3))';
                else
                    %metode escollit es VW
                    c=permute(obj.data,[1 3 2]);
                    x=reshape(c, z(1)*z(3),z(2));
                end
            else
                if(d==2)
                    %dades son 2D -> nomes tenim 1 lot
                    x=obj.data;
                    if (metode == 0) %si s'ha de forar conversi i BW
                        
                        x=reshape(obj.data,z(2)*z(1),1)';
                    end
                else
                    disp('DATA/to2D: Matrix shape error');
                    x=[];
                end
            end
        end
    end
end