classdef (ConstructOnLoad = true) PCA < handle
    %La classe PCA sera l'encarregada de mantenir en memoria les dades
    %necessaries per fer els calculs sobre un model PCA. Aquesta classe
    %tambe permet la projecci de dades sobre el model PCA emmagatzemat,
    %creant aixi els PCA no models.
    %
    %Les atributs d'aquest objecte sn ptivats i sn:
    %
    %    DATA - Objecte tipus DATA on hi ha el conjunt de dades inicials
    %           utilitzades per crear el model PCA. Aquest camp
    %           s'inicialitza amb el constructor de la clase i es va
    %           actualitzant segons les operacions fetes.
    %           Aquest atribut pot ser recuperat en tot moment amb getDATA.
    %           
    %
    %    estandar - Es l'atribut intern o indiquem si el proces esta
    %               estandaritzat o no ho esta. Aquest atribut no s
    %               accesible des de l'exterior de la classe per cap
    %               matode.
    %
    %    model - aquesta variable ens indica si les dades emmagatzemades a
    %            l'objecte formen un model o provenen d'una projecci soble
    %            un model extern. Podem consultar-ho amb el metode ismodel.
    %    
    %    Z - Matriu bidimencional de doubles, on s'hi emmagatzemaran les
    %        dades Estandaritzades i s'inicialitzar a null.
    %
    %    P - Matriu bidimencional de doubles, on guardem la matriu de
    %        Components principals, El seu valor per defecte s null.
    %
    %    Num - Es el numero de components principals que s'utilitzaran per 
    %          fer el model.Es pot consultar amb getNum.
    %
    %    Num_save - Numero de components principals que s'utilitzaran per
    %               si s'entra manualment i poder evitar que es demani
    %               diversos cops a l'usuari.
    %
    %    T - Matriu de doubles on guardem la projecci de Z sobre el model
    %        PCA. Ha de tenir tantes files com Z i tantes columnes com P.
    %        El seu valor per defecte s null.  
    %
    %    Landa - Vector que cont els valors propis associats a les
    %            components principals retingudes. Ha de tenir tants
    %            elements com columnes t P. El seu valor per defecte s
    %            null.   
    %
    %    Landai - Vector que cont els valors propis associats a totes les
    %             components principals. T el nombre delements igual al
    %             rang de Z.  
    %
    %    sta - Vector de doubles que conte la desviaci de cadascuna de les
    %          variables mesurades del procs. T tants elements com
    %          columnes t Z.
    %
    %    mea - Vector de doubles que conte la mitjana de cadascuna de les
    %          variables mesurades del procs. T tants elements com
    %          columnes t Z.  
    %
    %    E - Matriu de doubles que cont la matriu residual de projectar Z
    %        sobre el model PCA. 
    %
    %    T2 - Vector de doubles que conte els valors de lndex T^2 de cada
    %         observaci en Z. Te tants elements com files t Z. 
    %
    %    LimT2 - Double que indica el lmit de T^2. qualsevol observaci
    %            amb un valor de T^2 per sobre daquest lmit es considera
    %            defectuosa per culpa de T^2.
    %
    %    LimQ - Double que indica el lmit de Q. qualsevol observaci amb
    %           un valor de Q per sobre daquest lmit es considera
    %           defectuosa per culpa de Q.
    %
    %    metode -Double que identifica el mtode de selecci del nombre de
    %            components principals que sutilitzar. Els possibles
    %            valors sn:  
    %             0=Manual
    %             1=kaiser
    %             2=colze
    %             3=colze+punt recomenat
    %             4=% de varincia explicada
    %             5=VRE
    %
    %    metodee - Mtode destandarditzaci que sutilitzar.
    %              Els valors vlids sn:
    %              1=AS - Auto scaling (opci per defecte)
    %              2=GS - Grup scaling
    %              3=CS - Continous scaling
    %              4=BS - Block scaling
    %
    %    treuFM - Boole que indica si volem treure la forma mitja de les
    %             dades (treuFM = true, valor per defecte), o no (treuFM =
    %             false). El fet de treure la forma mitja afecta si estem
    %             treballant en VW o en BW i saplica el Continuous
    %             scaling.
    %
    %    conv - Boole que fixa el mtode de desdoblar la matriu
    %           tridimensional de dades. conv = false desdobla en BW (opci
    %           per defecte), mentre que conv = 1 desdobla en VW.  
    %           0= BW
    %           1= VW
    %
    %    structura - Estructura de dades on es guarda tota la informaci
    %                relacionada amb el mtode del VRE. 
    %
    %    analitic - Boolea que indica el metode que es vol fer servir en
    %               calcular els limits de les contribucins ( 0=3_sigma i
    %               1=analitic)
    %
    %    ContMetode - Boolea que indica el metode que es vol fer servir en
    %                 calcular les contribucins (0 = descartar negatius i
    %                 1 = negatius i positius)
    %
    %    CLimQstd - limit de Q - Desviaci tipica - Vector de doubles que
    %              cont la desviaci estndard de les contribucions de Q.
    %              Sutilitza per al clcul del lmit daquestes
    %              contribucions.   
    %
    %    CLimQmea - limit de Q - Mitjana - Vector de doubles per
    %               emmagatzemar la mitjana de les contribucions de Q.
    %               Sutilitza per al clcul del lmit daquestes
    %               contribucions.
    %
    %    CLimT2std - limit de T2 - Desviaci tipica - Vector de doubles que
    %                cont la desviaci estndard de les contribucions de T
    %                . sutilitza per al clcul del lmit daquestes
    %                contribucions.
    %    
    %    CLimT2mea - limit de T2 - Mitjana - Vector de doubles que cont la
    %                mitjana de les contribucions de T . Sutilitza per al
    %                clcul del lmit daquestes contribucions.
    %
%-------------------------------------------------------------------------%
%% -----------------------------VARIABLES---------------------------------
%-------------------------------------------------------------------------%
    
    properties (Access = private) %(Access = public) %Els atributs de la classe son privats, per fer proves o debugar canviar a (Access = public)
        % Propietats privades del nostre objecte, aqui nomes les definirem,
        % tot i aixo sels podria asignar un valor per defecte. En el nostre
        % cas el valor per defecte l'assignarem al constructor.
        DATA;%Obgecte tipus DATA on hi haura les nostres dades inicials
        estandar;%vadiable que ens indica si el proces esta estandaritzat o no
        model;% ens indica si les dades emmagatzemades a la c
        
        Z;%dades estandaritzades
        P;%Components principals
        Num;%numero de components principals que s'utilitzaran per fer el model
        Num_save;%numero de components principals que s'utilitzaran per fer el model (si s'entra manualment el guardem aqui)
        T;%matriu de T
        Landa;%matriu de landas
        Landai;%les landas inicials
        sta;%desviaci estandard
        mea;%mitjana
        E;%Error
        T2;%T^2
        LimT2;%Limit de T^2
        LimQ;%Limit de Q
        metode;%el metode de seleccio de numero de components que s'utilitzara  0=Manual 1=kaiser 2=colze 3=colze+punt recomenat 4=% de landa explic
        metodee;%es el metode d'estandaritzaci que s'utilitzara 1=autoscaling 2=GS 3=CS 4=BS 5=CBR
        treuFM;%Treure la forma mitja 1=si 0 = no nomes afecta si estem trevallan en VW
        conv;%0= BW 1 = VW
        analitic;%0=3_sigma; 1=analitic
        structura;%estructura de dades on guardarem tot lo necesari per el CBR
        %Contribucions als limits de Q i T2
        CLimQstd;%Q - Desviaci tipica
        CLimQmea;%Q - Mitjana
        CLimT2std;%T2 - Desviaci tipica
        CLimT2mea;%T2 - Mitjana
        
        ContMetode;%metode de calcular les contribucins (0 = descartar negatius i 1 = negatius i positius)
    end

%-------------------------------------------------------------------------%
%% ------------------DEFINICIO DE METODES DE LA CLASSE--------------------
%-------------------------------------------------------------------------%

    methods
        
        %Els metodes descrits a continuaci son publics i per tant
        %accesibles des de fora. Cal controlar el que s'ens passa per
        %parametre per evitar conflictes amb els tipus i/o formats de les
        %dades.
        
%-------------------------------------------------------------------------%
%%------------------------------CONSTRUCTORS------------------------------  
%-------------------------------------------------------------------------%
        
        %constructor per defecte, En el nostre cas al constructor se li
        %haura de passar un obgecte de tipus DATA, si no no es permetra
        %crear l'objecte. 
        function obj = PCA(data)
            %Es el constructor per defecte de l'objecte. Demana un objecte
            %de tipus DATA per crear un objecte PCA, un PCA sense dades no
            %te cap sentit.
            %Carrega l'objecte DATA i Inicialitza tots els atributs de
            %l'objecte a buits. Fins que siguin necessaris.
            %
            %Parametres d'entrada:
            %
            %   data - un objecte de tipus DATA per poder crear el PCA 
            %
            %Parametres de sortida:
            %
            %   obj - Retorna un objecte PCA inicialitzat per defecte
            %
            
            z=whos('data');
            if(strcmp( z.class , 'DATA'))
                obj.DATA = data.copy();
                obj.estandar=0;%fixem a no estandaritzades
                obj.model=1;%es un model
                obj.metode=1;%per defecte utilitzarem el metode de Kaiser
                obj.metodee=1;%per defecte Auto Scaling
                obj.treuFM=1;%per defecte treurem sempre la Forma mitja
                obj.conv=0;%per defecte bach wide
                obj.analitic=0;%per defecte 3 sigma
                obj.ContMetode=0;
                
                obj.structura=[];
                
                %Borrem tot el que haviem calculat calculat
                obj.Z=[];
                obj.P=[];
                obj.T=[];
                obj.Landa=[];
                obj.Landai=[];
                obj.sta=[];
                obj.mea=[];
                obj.E=[];
                obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                obj.Num=[];
                obj.Num_save=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
                
            else
                disp('data must be a DATA object')
            end
        end
        %aquesta funci ens retorna una copia del nostre objecte.
        function copy = copy(obj)
            %Constructor de copia
            %Crea una copia del nostre objecte i el retorna per referencia
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada, obj es
            %   l'objecte actual i forma part de la OO de matlab.
            %
            %Parametres de sortida:
            %
            %   obj - Retorna un objecte PCA inicialitzat com a copia de
            %         l'actual
            %
            
            %Copiem les dades i creem un nou PCA per defecte
            copy=PCA(obj.DATA.copy);
            
            %El configurem com l'obvecte actual
            copy.estandar=obj.estandar;
            copy.model=obj.model;
            copy.metode=obj.metode;
            copy.metodee=obj.metodee;
            copy.treuFM=obj.treuFM;
            copy.conv=obj.conv;
            copy.analitic=obj.analitic;
            copy.ContMetode=obj.ContMetode;
            
            %tambe l'estructura per si s'utilitzava VRE
            copy.structura=obj.structura;
            
            
            %Copiem tot el que haviem calculat previament
            copy.Z=obj.Z;
            copy.P=obj.P;
            copy.T=obj.T;
            copy.Landa=obj.Landa;
            copy.Landai=obj.Landai;
            copy.sta=obj.sta;
            copy.mea=obj.mea;
            copy.E=obj.E;
            copy.T2=obj.T2;
            copy.LimT2=obj.LimT2;
            copy.LimQ=obj.LimQ;
            copy.Num=obj.Num;
            copy.CLimQstd=obj.CLimQstd;
            copy.CLimQmea= obj.CLimQmea;
            copy.CLimT2std=obj.CLimT2std;
            copy.CLimT2mea=obj.CLimT2mea;
            copy.Num_save=obj.Num_save;
            
        end
        
     
        
%-------------------------------------------------------------------------%
%% ------------------------- GETTERS AND SETTERS--------------------------                            
%-------------------------------------------------------------------------%
        
        %Per trevallar amb les dades
        function obj = getRawData(obj)
            %Retorna les dades transformades a 2D utilitzant bach wide o
            %variable wide segons el metode indicat.
            %
            % Si metode = 0 o 'BW' Les dades es retornaran en format Bach wide
            % Si metode = 1 o 'VW' Les dades es retornaran en format Variable wide
            %
            %Parametres d'entrada:
            %
            %   metode - pot ser 1 , 0 , 'VW' o 'BW' i ens indica el format de converci
            %           a utilitzar
            %
            %Parametres de sortida:
            %
            %   obji - aquest metode retorna una matriu de doubles
            %         corresponent a l'unfolding de la matriu 3D on
            %         s'enmagatzemen les dades internament.

            obj = obj.DATA.getRawData(obj.conv);%cridem el metode adecuat del nostre objecte dades
        end
        function dades =getDATA(obj)
            %retorna una copia de l'objecte DATA de l'objecte actual
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Retorna una copia de l'objecte que tenim assignat al
            %         nostre PCA com a dades
            %
            dades=obj.DATA.copy();%fem una copia i la retornem
        end
        %classes
        function clas=getClass(obj, i)
            %Retorna la classe de les dades
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   clas - Retorna un vector amb la classe de totes les dades
            %
             if nargin > 1
                clas=obj.DATA.getClass(i);
            else
                clas=obj.DATA.getClass();
             end
        end
        function setClass(obj,class)
            %Metode per donar classe a les dades d'un PCA
            %Parametres d'entrada:
            %
            %   class - La nova classe de les nostres dades
            %
            %Parametres de sortida:
            %
            %   No en te
            %
           obj.DATA.setClass(class); %passem la classe a l'objecte DATA perque ho guardi
        end
        %lots
        function deleteBatch(obj,i)
            %Treu els lots indexats per ''i". ''i" pot ser un double, o un
            %logical en format vector de 1 a n elements on n es el numero
            %de lots. O qualsebol format valid en indexacions al Matlab.
            %
            %
            % dim(1)=Temps
            % dim(2)=Variables
            % dim(3)=Batch
            %         _______
            %        |______ |                      _______
            %       |______ ||                     |______ |
            %      |______ |||                    |______ ||
            %  |  |       ||||     - [1] =    |  |       |||
            %Temps|       |||  /            Temps|       |||
            %  |  |       ||Batch             |  |       ||
            %     |_______|  /                   |_______|
            %     - Vars -                        - Vars -
            %
            %      Actual          index          Resultat
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %    i - els lots que es bol treure
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            
            %treiem un lot de dades PCA s'ha de modificar
            if(obj.isModel)
                %es un model
                obj.estandar=0;%fixem a no estandaritzades
                %Borrem tot el que haviem calculat calculat
                obj.structura=[];
                obj.Z=[];
                obj.P=[];
                obj.T=[];
                obj.Landa=[];
                obj.Landai=[];
                obj.sta=[];
                obj.mea=[];
                obj.E=[];
                obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                obj.Num=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
            else
                %no es model
                %aux=obj.DATA.getRawData3D();
                %lots=size(aux,3);
                
                lots=obj.DATA.getBatchCount();
                %vars=size(aux,2);%no necesitem
                %temps=size(aux,1);
                
                temps = obj.DATA.getSampleCount();
                %obj.structura=[];
                if(obj.conv==0)
                    %bach wise
                    x=logical(1:lots);%x es un vector de 1 logics de longitud = al numero de lots
                    x(i)=0;%posem a 0 els que bolen eliminar

                    if(not(isempty(obj.Z)))
                        obj.Z=obj.Z(x,:);%lots*(vars*temps)
                    end
                    if(not(isempty(obj.T)))
                        obj.T=obj.T(x,:);%lots*(vars*temps)
                    end
                    if(not(isempty(obj.E)))
                        obj.E=obj.E(x,:);
                    end
                    if(not(isempty(obj.T2)))
                        obj.T2=obj.T2(x);
                    end
                    
                else
                    %variable wise
                    x=logical(1:lots*temps);%x es un vector de 1 logics de longitud = al numero de lots
                    x(temps*(i-1)+(1:temps))=0;%posem a 0 els que bolen eliminar
                   
                    if(not(isempty(obj.Z)))
                        obj.Z=obj.Z(x,:);%(lots*temps)*vars
                    end
                    if(not(isempty(obj.T)))
                        obj.T=obj.T(x,:);%(lots*temps)*vars
                    end
                    if(not(isempty(obj.E)))
                        obj.E=obj.E(x,:);
                    end
                    if(not(isempty(obj.T2)))
                        obj.T2=obj.T2(x);
                    end
                end
            end
            obj.DATA.deleteBatch(i);%treiem el lot de les dades
        end
        function addBatch(obj,dad, clase, nom)
            %Aquesta funci afegeix un o diversos Batch a les nostres
            %dades. Les mides del lot han de ser concordants amb les de la
            %resta de l'estructura que tenim.
            %                                                      _______
            % dim(1)=Temps                                        |______ |
            % dim(2)=Variables                                   |______ ||
            % dim(3)=Batch                                      |______ |||
            %         _______              _______             |______ ||||
            %        |______ |            |______ |           |______ ||||
            %       |______ ||           |______ ||          |______ ||||
            %      |______ |||   +      |______ |||     =   |______ ||||  /
            %  |  |       ||||      |  |       ||||     |  |       ||||  /
            %Temps|       |||  /  Temps|       |||    Temps|       |||Batch
            %  |  |       ||Batch   |  |       ||       |  |       ||  /
            %     |_______|  /         |_______|           |_______|  /
            %     - Vars -             - Vars -             - Vars -
            %
            %      Actual                 dad                 Resultat
            % **recordar que el matlab les dimensions sn 1-ordenades
            % 2-absices 3- profunditat**
            %
            %Parametres d'entrada:
            %
            %    dad - matriu de doubles 2D o 3D
            %
            %    clase -  es un vector de doubles
            %
            %    ref - (opcional) es la referencia o nom del lot
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            
            %afegim un nou lot de dades, PCA s'ha de modificar
            if(obj.isModel)
                %es un model
                obj.estandar=0;%fixem a no estandaritzades
                %Borrem tot el que haviem calculat calculat
                obj.structura=[];
                obj.Z=[];
                obj.P=[];
                obj.T=[];
                obj.Landa=[];
                obj.Landai=[];
                obj.sta=[];
                obj.mea=[];
                obj.E=[];
                obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                obj.Num=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
            else
                %no es model
                aux=obj.DATA.copy();%muntem un nou objecte dades
                aux.setRawData(dad);%li entrem les dades
                aux.setClass(clase);
                aux2=obj.project(aux);%el projectem al nostre model
                %afegim les dades del nou lot a les actuals
                if(not(isempty(aux2.Z)))
                    obj.Z=[obj.Z; aux2.getZ()];
                else
                    error('no dades?');
                end
                if(not(isempty(aux2.T)))
                    obj.T=[obj.T; aux2.getT()];
                else
                    obj.T=[];
                end
                if(not(isempty(aux2.E)))
                    obj.E=[obj.E; aux2.getE()];
                else
                    obj.E=[];
                end
                if(not(isempty(aux2.T2)))
                    obj.T2=[obj.T2; aux2.getT2()];
                else
                    obj.T2=[];
                end
            end
            if nargin == 4
                obj.DATA.addBatch(dad,clase,nom);
            else
                obj.DATA.addBatch(dad,clase);
            end
        end
        function [data clas nom]=getBatch(obj, i)
            %retorna el lot o lots indicats per el parametre d'entrada 'i',
            %aquest por ser qualsebol tipus d'indexaci valida en matlab.
            %
            %              _______
            %             |______ |
            %            |______ ||
            %           |______ |||
            %          |______ |||| - 8
            %         |______ |||| - 7
            %        |______ |||| - 6
            %       |______ |||| - 5   /
            %   |  |       |||| - 4   /
            % Temps|       ||| - 3 Batch
            %   |  |       || - 2   /
            %      |_______| - 1   /
            %      - Vars -
            %
            %
            %Parametres d'entrada:
            %
            %    i - index del lot/s que es vol recuperar
            %
            %Parametres de sortida:
            %
            %    data - matriu de doubles 2D son les dades del lot
            %
            %    clas -  es un double i correspon a la classe del lot
            %
            %    noms - es el nom del batch
            %
            
            %no afectara per res al PCA fem la crida a dades i prou
            [data clas nom]=obj.DATA.getBatch(i);
        end
        %dimencions
        function obji = getBatchCount(obj)
            %Retorna el numero de lots de les nostres dades
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obji - es un double, correspon al nombre de lots del nostre
            %         objecte
            %
            
            obji = obj.DATA.getBatchCount();
        end
        function num = getVariableCount(obj)
            %Retorna el numero de variables de les nostres dades
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   num - es un double, correspon al nombre de variables del
            %         nostre objecte
            %
            num = obj.DATA.getVariableCount();
        end
        function num = getSampleCount(obj)
            %Retorna el numero de instants de temps de les nostres dades
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   num - es un double, correspon al nombre de instants de
            %         temps del nostre objecte
            %
            num = obj.DATA.getSampleCount();
        end
        %noms de lots
        function setBatchLabel(obj, in)
            %Fixara els noms dels lots. Aquests son els noms dels diferents
            %lots de les nodtres dades, se n'ha d'entra 1 per lot en el
            %mateix ordre en que els teim a les dades
            %
            %Parametres d'entrada:
            %
            %   variables - s el nou vector de noms dels lots.
            %               Aquest vector ha de ser un cellstring
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            obj.DATA.setBatchLabel(in);
        end
        function out=getBatchLabel(obj,i)
            %retorna el nom dels lots. Aquests son els noms dels diferents
            %lots de les nodtres dades. Retornarem tots o nomes els que
            %s'ens indiqui.
            %
            %Parametres d'entrada:
            %
            %   i - (opcional)es l'index dels lots que volem recuperar-ne
            %       el nom, si no s'indica retornarem tots els noms
            %
            %Parametres de sortida:
            %
            %   out - s un cell string amb els noms indicats, sempre sera
            %         en format fila.
            %
            if nargin > 1
                out=obj.DATA.getBatchLabel(i);
            else
                out=obj.DATA.getBatchLabel();
            end
        end
        %Z
        function obj = getZ(obj)
            %Mtode que retorna la matriu de dades estandaritzades i
            %desdoblades en BW o VW (en funci de com shagi creat el model
            %PCA).  
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Retorna una matriu bidimencional
            %
            
            if(isempty(obj.Z) && obj.getBatchCount>0)%si no estan calculades les calculem
                estandaritza(obj);
            end
            obj = obj.Z;%retornem les dades estandaritzades
        end
        %P;
        function obj = getP(obj)
            %Mtode que retorna la matriu de dades estandaritzades i
            %desdoblades en BW o VW (en funci de com shagi creat el model
            %PCA).
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - La matriu de components principals.
            %
            
            if(isempty(obj.P) && obj.getBatchCount>0)%si no estan calculades les calculem
                obj.pca();%calculem T,P,...
            end
            obj = obj.P;%retornem les components principals
        end
        %T;
        function obj = getT(obj)
            %Mtode que retorna la matriu que cont la projecci de les
            %dades en el subespai de les components principals.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Retorna La matriu de T
            %
            if(isempty(obj.T) && obj.getBatchCount>0)%si no estan calculades les calculem
                obj.pca();%calculem T,P...
            end
            obj = obj.T;%retornem T
        end
        %Landa;
        function obj = getLambda(obj)
            %Mtode que retorna el vector de valors propis associats a les
            %components principals retingudes.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb els valors propis associats a
            %         les components principals retingudes
            %
            
            if(isempty(obj.Landa) && obj.getBatchCount>0)%si no estan calculades les calculem
                obj.pca();
            end
            obj = obj.Landa;
        end
        function obj = getLambdaAll(obj)
            %Mtode que retorna el vector de valors propis associats a les
            %totes components principals (retingudes o no)
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb els valors propis associats a
            %         les components principals
            %
            
            if(isempty(obj.Landai) && obj.getBatchCount>0)%si no estan calculades les calculem
                obj.pca();
            end
            obj = obj.Landai;
        end      
        %sta;
        function obj = getStd(obj)
            %Mtode que retorna el vector de desviacions estndards per
            %estandaritzar les dades.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb les desviacions estndards per
            %         estandaritzar les dades. 
            %
            if(isempty(obj.sta) && obj.getBatchCount>0)%si no estan calculades les calculem
                estandaritza(obj);
            end
            obj = obj.sta;
        end
        %mea;
        function obj = getMean(obj)
            %Mtode que retorna la mitjana de les dades que sutilitza per
            %estandaritzar les dades.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb les mitjanes per estandaritzar
            %         les dades. 
            %
            if(isempty(obj.mea) && obj.getBatchCount>0)%si no estan calculades les calculem
                estandaritza(obj);
            end
            obj = obj.mea;
        end
        %E;
        function obj = getE(obj)
            %Mtode que retorna la matriu residual.
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Matriu 2D de doubles amb la matriu residual de la
            %         projecci. 
            %
            if(isempty(obj.E) && obj.getBatchCount>0)%si no estan calculades les calculem
                calculaE(obj);
            end
            obj = obj.E;
        end
        %getT2;
        function obj = getT2(obj)
            %Mtode que retorna el vector de valors de lndex T^2 per a
            %les dades projectades. 
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles que cont els valors de T^2 de les
            %         dades projectades.
            %
            if(isempty(obj.T2) && obj.getBatchCount>0)%si no estan calculades les calculem
                %calculaLimT2(obj);
                if(isempty(obj.T) && obj.getBatchCount>0)
                    obj.pca();
                end
                obj.T2=((obj.T).^2);
                obj.T2=((obj.T2)./repmat(obj.Landa',size(obj.T2,1),1))*ones(size(obj.T2,2),1);
            end
            obj = obj.T2;
        end
        %LimT2
        function obj = getT2Limit(obj)
            %Mtode que retorna el lmit de de T^2. Qualsevol observaci
            %amb un valor de T^2 superior a aquest valor es considera
            %defectuosa per T^2.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Double amb el lmit de T^2.
            %
            if(isempty(obj.LimT2)&& obj.getBatchCount>0)%si no estan calculades les calculem
                calculaLimT2(obj);
            end
            obj = obj.LimT2();
        end
        %LimQ;
        function obj = getQLimit(obj)
            %Mtode que retorna el lmit de de Q. Qualsevol observaci amb
            %un valor de Q superior a aquest valor es considera defectuosa
            %per Q.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Retorna el limit per Q
            %
            if(isempty(obj.LimQ)&& obj.getBatchCount>0)%si no estan calculades les calculem
                calculaLimQ(obj);
            end
            obj = obj.LimQ;
        end
        %Num;
        function obj = getNum(obj)
            %Mtode que retorna el nombre de components principals retingudes.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Double amb el nombre de components principals
            %         retingudes 
            %
            if(isempty(obj.Num))
                %si no s'ha inicialitzat el fixarem manualment
                %disp('Encara no s`ha definit el nombre de components principals a utilitzar!');
                %N=input('Entra el numero de components a utilitzar:');
                N = str2double(inputdlg('Number of Principal componens to use:'));
                z=whos('N');
                while(~strcmp(z.class,'double'))
                    %N=input('Entra el numero de components a utilitzar:');
                    N = str2double(inputdlg('Number of Principal componens to use:'));
                    z=whos('N');
                end
                %el guarden
                obj.Num=N;
                obj.Num_save=N;
            end
            obj = obj.Num;
        end
        %fixem el numero de components principals
        function setNum(obj, n)
            %Mtode que fixa el nombre de components principals a retenir
            %de manera manual. 
            %
            %Parametres d'entrada:
            %
            %   n - double que indica el nombre de components principals a
            %       retenir
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            
            %comprobem que ens passin un double
            z=whos('n');
            if(not(strcmp(z.class,'double')))
                error('PCA/setNum: Waiting for double, no changes have been made.');
            end
            if(obj.model~=1)
                error('PCA/setNum: Object must be model, no changes have been made.');
            end
            %El metode de selecci de components principals el canviem a
            %manual
            obj.metode = 0;
            obj.Num_save=n;%guardem el numero a la copia de seguretat
            if(n ~= obj.Num) %si es el mateix no fem res!
                if(n < obj.Num)%si es mes petit no cal fer tans calculs!
                    obj.Num = n;
                    %nomes cal tallar
                    obj.P = obj.P(:, 1:(obj.Num));
                    obj.T = obj.T(:, 1:(obj.Num));
                    obj.Landa = obj.Landa(1:(obj.Num));
                    
                    obj.E=[];
                    obj.T2=[];
                    obj.LimT2=[];
                    obj.LimQ=[];
                    obj.CLimQstd=[];
                    obj.CLimQmea=[];
                    obj.CLimT2std=[];
                    obj.CLimT2mea=[];
                    
                    obj.structura=[];
                    
                else
                    obj.Num = n;

                    %Borrem tot el que haviem calculat calculat
                    obj.P=[];
                    obj.T=[];
                    obj.Landa=[];
                    obj.Landai=[];
                    obj.E=[];
                    obj.T2=[];
                    obj.LimT2=[];
                    obj.LimQ=[];
                    obj.CLimQstd=[];
                    obj.CLimQmea=[];
                    obj.CLimT2std=[];
                    obj.CLimT2mea=[];
                    
                    obj.structura=[];
                    
                    obj.pca();
                end
            end
        end
        %isModel
        function bool =isModel(obj)
            %Mtode que indica si la instncia de PCA s un model (True) o
            %s una projecci (False).
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Boolea que indica si la instncia s un model PCA
            %         (True) o s una projecci (False). 
            %
            bool=obj.model;
        end
        %Selecci components principals
        function metode =getPCSelectionMethod(obj)
            %Mtode que retorna el procediment usat per determinar el
            %nombre de components principals. 
            %Els possibles valors sn:
            %   0 - Manual
            %   1 - Kaiser
            %   2 - Manual/Visual pel colze
            %   3 - Auto/Visual pel colze(proves)
            %   4 - Manual/Visual pel % d`explicacio de les dades
            %   5 - VRE
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   obj - Double que indica el mtode amb qu sha determinat
            %         el nombre de components principals. 
            %
            metode=obj.metode;
            
        end
        function setPCSelectionMethod(obj,x)
            %Mtode que fixa el procediment a usar per determinar el
            %nombre de components principals. 
            %Els possibles valors sn:
            %   0 - Manual
            %   1 - Kaiser
            %   2 - Manual/Visual pel colze
            %   3 - Auto/Visual pel colze(proves)
            %   4 - Manual/Visual pel % d`explicacio de les dades
            %   5 - VRE
            %
            %Parametres d'entrada:
            %
            %   x - Double que indica el mtode amb qu sha determinat
            %       el nombre de components principals.
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            %comprovem el parametre d'entrada
            z=whos('x');
            if(not(strcmp(z.class,'double')))%tipus ha de ser double
                error('PCA/setPCSelectionMethod: waiting for double, no changes have been made.');
            end
            if(not(0<=x && x<6))%comprovem el rang
                error('PCA/setPCSelectionMethod: No valid selection method.');
            end
            if(not(obj.isModel()))%si no es model no es pot canviar
                error('PCA/setPCSelectionMethod: No changes can be made in non model objects!');
            end
            
            %Si es el mateix k l'actual no cal fer res altrament borrar tot
            if(obj.metode~=x)
                obj.structura=[];
                
                %Borrem tot el que haviem calculat calculat
                obj.Z=[];
                obj.P=[];
                obj.T=[];
                obj.Landa=[];
                obj.Landai=[];
                obj.sta=[];
                obj.mea=[];
                obj.E=[];
                obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                obj.Num=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
                 obj.estandar=0;%fixem a no estandaritzades
            end
            %fixem el nou metode
            obj.metode=x;
            
            %missatge de VRE nomes funciona en VW! 
            if(x==5 && obj.conv==0 )
                warndlg('PCA/setPCSelectionMethod: VRE works only with Variable Wise.','ATTENTION!');
            end
        end
        %Estandarditzaci
        function metode =getNormalizationMethod(obj)
            %Mtode per saber el procs destandarditzaci utilitzat. 
            %Els valors possibles sn:
            %
            %1 - Auto Scaling 
            %2 - Group Scaling 
            %3 - Continous Scaling 
            %4 - Block Scaling 
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   metode - es un double equivalent al metode utilitzat
            %
            metode=obj.metodee;

        end
        function setNormalizationMethod(obj,x)
            %Mtode per fixar el procs destandarditzaci. 
            %Els valors vlids sn:
            %
            %1 - Auto Scaling 
            %2 - Group Scaling 
            %3 - Continous Scaling 
            %4 - Block Scaling 
            %
            %Parametres d'entrada:
            %
            %   x - ha de ser de tipus double, sera el nou metode
            %   d'estandaritzaci. Els valors possibles son:
            %       1 - Auto Scaling
            %       2 - Group Scaling
            %       3 - Continous Scaling
            %       4 - Block Skaling
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            z=whos('x');
            if(not(strcmp(z.class,'double')))
                error('PCA/setNormalizationMethod: Waiting for double, no changes have been made.');
            end
            if(not(1<=x && x<=4))
                error('PCA/setNormalizationMethod: No valid estandaritzation method.');
            end
             if(not(obj.isModel()))
                error('PCA/setNormalizationMethod: No changes can be made in non model objects!');
            end
            if(obj.metodee~=x)
                obj.structura=[];
                
                %Borrem tot el que haviem calculat calculat
                obj.Z=[];
                obj.P=[];
                obj.T=[];
                obj.Landa=[];
                obj.Landai=[];
                obj.sta=[];
                obj.mea=[];
                obj.E=[];
                obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                obj.Num=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
                 obj.estandar=0;%fixem a no estandaritzades
            end
            
            obj.metodee=x;
            
        end
        %treure la forma mitja
        function setTrajectoryRemoval(obj,x)
            %Mtode per indicar si sha de treure la forma mitja (x =True)
            %o no (x = False) durant el procs destandarditzaci. 
            %
            %Parametres d'entrada:
            %
            %   x - ha de ser de tipus double, s'espera un 1 o un 0, si es
            %   1 forarem treure la forma mitja de les dades.
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            z=whos('x');
            if(not(strcmp(z.class,'double')))
                error('PCA/setTrajectoryRemoval: Waiting for double, no changes have been made.');
            end
            if(not(0<=x && x<=1))
                error('PCA/setTrajectoryRemoval: Only is allowed 1 or 0');
            end
            if(not(obj.isModel()))
                error('PCA/setTrajectoryRemoval: No changes can be made in non model objects!');
            end
            if(obj.treuFM~=x)
                obj.structura=[];
                
                %Borrem tot el que haviem calculat calculat
                obj.Z=[];
                obj.P=[];
                obj.T=[];
                obj.Landa=[];
                obj.Landai=[];
                obj.sta=[];
                obj.mea=[];
                obj.E=[];
                obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                obj.Num=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
                 obj.estandar=0;%fixem a no estandaritzades
            end
            
            obj.treuFM=x;
            
        end
        function val =getTrajectoryRemoval(obj)
            %Mtode per indicar si sha tret la forma mitja (val =True) o
            %no (val = False) durant el procs destandarditzaci.
            %
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   val - es de tipus double, 1 o 0, si es 1 indica que s'ha
            %         tret la forma mitja de les dades altament no s'haura
            %         tret 
            %
            val = obj.treuFM;
        end
        %setUnfoldingMethod
        function setUnfoldingMethod(obj,x)
            %Mtode per fixar el tipus de desdoblament.
            %1= VW i 0=BW
            %Parametres d'entrada:
            %
            %   x - ha de ser de tipus double, s'espera un 1 o un 0, si es
            %   1 apartir d'ara trevallarem en VW si es un 0 trevallarem en
            %   BW
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            z=whos('x');
            if(not(strcmp(z.class,'double')))
                error('PCA/setUnfoldingMethod: Waiting for double, no changes have been made.');
            end
            if(not(0<=x && x<=1))
                error('PCA/setUnfoldingMethod: Only is allowed 1 or 0');
            end
            if(not(obj.isModel()))
                error('PCA/setUnfoldingMethod: No changes can be made in non model objects!');
            end
            if(obj.conv~=x)
                obj.structura=[];
                
                %Borrem tot el que haviem calculat calculat
                obj.Z=[];
                obj.P=[];
                obj.T=[];
                obj.Landa=[];
                obj.Landai=[];
                obj.sta=[];
                obj.mea=[];
                obj.E=[];
                obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                obj.Num=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
                 obj.estandar=0;%fixem a no estandaritzades
            end
            
            obj.conv= x;
        end
        function obj = getUnfoldingMethod(obj)
            %Metode que retorna el metode d'unfolding que s'ha utilitzat
            %1= VW i 0=BW
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   obj - Retorna el metode d'unfolding utilitzat 1= VW i 0=BW.
            %
            obj = obj.conv;
        end
        %Metode calcul Limits
        function setLimitMethod(obj,x)
            %Metode per fixar el metode que utilitzarem per calcular els
            %limits 1=analitic  i 0= 3 sigma(per defecte)
            %
            %Parametres d'entrada:
            %
            %   x - ha de ser de tipus double, s'espera un 1 o un 0, si es
            %   1 apartir d'ara calcularem els limits analiticament, si es
            %   0 es calcularan amb 3 sigma.
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            z=whos('x');
            if(not(strcmp(z.class,'double')))
                error('PCA/setLimitMethod: Waiting for double, no changes have been made.');
            end
            if(not(0<=x && x<=1))
                error('PCA/setLimitMethod: Only is allowed 1 or 0');
            end
            if(not(obj.isModel()))
                error('PCA/setUnfoldingMethod: No changes can be made in non model objects!');
            end
            if(obj.analitic~=x)
                %obj.structura=[];
                
                %Borrem tot el que haviem calculat calculat
                %obj.Z=[];
                %obj.P=[];
                %obj.T=[];
                %obj.Landa=[];
                %obj.Landai=[];
                %obj.sta=[];
                %obj.mea=[];
                %obj.E=[];
                %obj.T2=[];
                obj.LimT2=[];
                obj.LimQ=[];
                %obj.Num=[];
                obj.CLimQstd=[];
                obj.CLimQmea=[];
                obj.CLimT2std=[];
                obj.CLimT2mea=[];
            end
            obj.analitic= x;
        end
        function x = getLimitMethod(obj)
            %Metode per fixar el metode que utilitzarem per calcular els
            %limits 1=analitic  i 0= 3 sigma(per defecte)
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   x - Es un 1 o un 0, si es ens indica el metode que s'esta
            %   utilitzant per calcular els limits 1 = analitic 0= 3 sigma
            %
            x = obj.analitic;
        end
        %fixa el metode de calcular contribucins        
        function setContributionMethod(obj,x)
            %metode per fixar el metode de calcul de les contribucins, en
            %cas d'indicar 0 = descartar negatius i 1 = negatius i positius
            %
            %Parametres d'entrada:
            %
            %   x - ha de ser de tipus double, s'espera un 1 o un 0
            %       0 = descartar negatius 
            %       1 = negatius i positius 
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            if x ~= 1 && x ~= 0
                error('PCA/setContributionMethod: Waiting for double, no changes have been made.');
            end
            if obj.model ~= 1
                error('PCA/setContributionMethod: No changes can be made in non model objects!');
            end
            
            obj.ContMetode=x;
            
            %m'han canviat el metode de calcular les contribucins les
            %borrem perke poden no ser correctes
            obj.CLimQstd=[];
            obj.CLimQmea=[];
            obj.CLimT2std=[];
            obj.CLimT2mea=[];
        end
        function x = getContMetode(obj)
            %Metode per saver el metode de calcul de les contribucins, en
            %cas de retornat 0 = descartar negatius i 1 = negatius i positius
            %
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   x - Es de tipus double, s'espera un 1 o un 0
            %       0 = descartar negatius 
            %       1 = negatius i positius 
            %
            x = obj.ContMetode;
        end
        %getQ
        function obj = getQ(obj)
            %Mtode que retorna el vector amb els valors de Q de les dades
            %projectades. 
            %
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   obj - Vector de dades amb els valors de Q per a cadascuna
            %         de les observacions.
            %
            
            if(isempty(obj.E) && obj.getBatchCount>0)%si no estan calculades les calculem
                calculaE(obj);
            end
            %Retorna les dades.
            obj = ((obj.E).^2)*ones(size(obj.E,2),1);
        end
        %get contribucio de Q
        function sort = getQContribution(obj)
            %Mtode que retorna les contribucions de cada observaci i
            %variable de lndex Q.
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   sort - Matriu de doubles amb les contribucions de Q per
            %          cada observaci i variable. 
            %

            %Retorna les dades.
            sort = calculaContQ(obj);
        end 
        %get contribucio de T^2
        function sort = getT2Contribution(obj)
            %Mtode que retorna les contribucions de cada observaci i
            %variable de lndex T^2.
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   sort - Matriu de doubles amb les contribucions de T^2 per
            %   cada observaci i variable.
            %
            sort = calculaContT2(obj);
        end
        %get contribucio de limit Q mea
        function obj = getQContributionLimitMean(obj)
            %Mtode que retorna la mitjana de les contribucions de Q per
            %cada variable que sutilitza per al clcul del lmit de les
            %contribucions de Q.
            %
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb la mitjana de les contribucions
            %         de Q. 
            %
            if(isempty(obj.CLimQmea))%si no estan calculades les calculem
                calculaContQ(obj);
            end

                obj=obj.CLimQmea;
        end
        %get contribucio de limit Q std
        function obj = getQContributionLimitStd(obj)
            %Mtode que retorna la desviaci estndard de les contribucions
            %de Q per cada variable que sutilitza per al clcul del lmit
            %de les contribucions de Q.
            %
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb les desviacions estndards de
            %         les contribucions de Q. 
            %
            if(isempty(obj.CLimQstd))%si no estan calculades les calculem
                calculaContQ(obj);
            end

                obj=obj.CLimQstd;
        end
        %get contribucio de limit T^2 mea
        function obj = getT2ContributionLimitMean(obj)
            %Mtode que retorna la mitjana de les contribucions de T^2 per
            %cada variable que sutilitza per al clcul del lmit de les
            %contribucions de T^2.
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb la mitjana de les contribucions
            %         de T^2
            %
            if(isempty(obj.CLimT2mea))%si no estan calculades les calculem
                calculaContT2(obj);
            end

                obj=obj.CLimT2mea;
        end
        %get contribucio de limit T^2 std
        function obj = getT2ContributionLimitStd(obj)
            %Mtode que retorna la desviaci estndard de les contribucions
            %de T^2 per cada variable que sutilitza per al clcul del
            %lmit de les contribucions de T^2.
            %
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   obj - Vector de doubles amb les desviacions estndards de
            %         les contribucions de T^2.
            %
            if(isempty(obj.CLimT2std))%si no estan calculades les calculem
                calculaContT2(obj);
            end

                obj=obj.CLimT2std;
        end
        %crea fitxer pel weka
        function weka(obj,nomfitx)
            %Mtode que crea un fitxer ,i lanomenara i posicionar segons
            %lstring que contingui nomfitx,on es posen les dades
            %necessries per entrar la base de casos al weka.
            %
            %Parametres d'entrada:
            %
            %   nomfitx - String amb el nom del fitxer (i opcionalment la
            %             ruta)
            %
            %Parametres de sortida:
            %
            %   Creara un fitxer anomenat 'nomfitx'.arff i hi posara les
            %   dades necesaries per entrar la base de casos al weka.
            %
            aux=obj.foraCQ;
            celles=obj.DATA.getClass();
            aux = [aux'; celles];
             if isempty(strfind(nomfitx,'.arff'))
                nomfitx = strcat(nomfitx,'.arff');
             end
            new_file = fopen(nomfitx,'w');
            
            fprintf(new_file,'@RELATION %s \r\n',obj.DATA.getName());
            
            for x = 1:length(obj.DATA.getVariableLabel)
                for y = 1:length(obj.DATA.getStages)
                    fprintf(new_file,'@ATTRIBUTE %s_%s NUMERIC\r\n',obj.DATA.getVariableLabel{x},obj.DATA.getStages{y});
                end
            end
            
            fprintf(new_file,'@ATTRIBUTE class NUMERIC\r\n');
            %fprintf(new_file,'%%1=bo\n%%0=regular\n%%-1=dolent\n');
            fprintf(new_file,'@DATA\r\n');
            fclose(new_file);
            
            dlmwrite(nomfitx,aux,'-append','newline','pc');
        end
        %reconstruct si s'ha fet CBA
        function [x_rec, f_i, direccio, valid] = reconstruct(obj)
            %Si s'ha utilitzat VRE fa una reconstruccio per recuperar les
            %dades originals. Aquesta funcio es privada.
            if obj.metode==5 && obj.conv==1 %si (CBR I VW){
                %si s'ha utilitzat VRE i estem en VW es pot fer
                if(isempty(obj.P))%si no estan calculades les calculem
                    obj.pca();
                end
            
% | X:          Array or matrix of double values whose observations       |
% |             (rows) have to be reconstructed (if their square          |
% |             prediction error surpasses the statistical limit of the   |
% |             PCA model).                                               |
% | PCA_Struct:	Structure that contains the information required to       |
% |             project new observations to a given principal component   |
% |             analysis (PCA) model. This structure must have, at least: |
% |             -> mx: array containing the mean for each variable        |
% |             (column) to subtract to X.                                |
% |             -> std: array containing the standard deviation of each   |
% |             variables (column) in X. This value is used to rescalate  |
% |             (divide) each variable.                                   |
% |             -> P: matrix containing the weights (loadings) to project |
% |             each observation (row) in X to the PCA model.             |
% |             -> SPE_lim: statistical limit for the SPE. This value is  |
% |             used to determine if an observation (row) in X is faulty  |
% |             (greater than SPE_lim) or not (equal or lower).           |
X=obj.Z;
MPCA_struc= struct('mx',obj.getMean,'std',obj.getStd,'P',obj.getP,'SPE_lim',obj.getQLimit,'lambda',obj.getLambda,'T2_lim',obj.getT2Limit,'R',(X'*X)/(size(X,1)-1));
                [f_i, direccio, x_rec] = VRE_fault_rec(X, MPCA_struc);
                valid=f_i==0;

            else
                %no es pot fer
                disp('RECONSTRUCTION FAILED');
            end
        end
        %Funci per treure outliers del Model.
        function canvis = deleteOutLiers(obj,alt)
            %Metode que elimina els outliers per T^2 i Q del model dins de
            %cada classe. amb el parametre alt podem seleccinar si bolem
            %eliminar-los tots o noms els extrems.
            %
            % +-1.5 * RIQ per trobar tots els outliers (alt {'all' 'ALL' 'All'})
            %
            % +-2.75 * RIQ per trobar nomes els extrems (Opci per defecte)
            %
            %Parametres d'entrada:
            %
            %   alt - (opcional) indicant un dels strings {'all' 'ALL'
            %          'All'} s'eliminaran outliers amb +-1.5 * RIQ
            %          altrament es fara amb +-2.75 * RIQ
            %
            %Parametres de sortida:
            %
            %   canvis - indica si hi ha hagut canvis (si hi ha canvis pot
            %            haver-hi nous outliers) 
            %
            if(obj.isModel)
                %mostrem la distribuci de les dades inicials
                if(obj.conv == 1)
                    error('PCA/deleteOutLiers: can not be used in Variable Wise.');
                end
                
                if nargin == 1
                    ind = getOutLiers(obj);
                else
                    ind = getOutLiers(obj,alt);
                end
                obj.deleteBatch(ind);
                
                %indica el numero de lots que hem tret
                canvis = sum(ind);
                
            else
                %no es un model. No podem modificar les dades dels PCA No
                %model perque depenen del Model al qual s'han projectat.
                 
                error('PCA/deleteOutLiers: The object must be a Model.');
                
            end
        end
        %Funci que comprova si hi ha outliers
        function outs = getOutLiers(obj,alt)
            %Metode que detecta els outliers per T^2 i Q del model dins de
            %cada classe. amb el parametre alt podem seleccinar si bolem
            %detectar-los tots o noms els extrems.
            %
            % +-1.5 * RIQ per trobar tots els outliers (alt {'all' 'ALL' 'All'})
            %
            % +-2.75 * RIQ per trobar nomes els extrems (Opci per defecte)
            %
            %Parametres d'entrada:
            %
            %   alt - (opcional) indicant un dels strings {'all' 'ALL'
            %          'All'} es detectaran outliers amb +-1.5 * RIQ
            %          altrament es fara amb +-2.75 * RIQ
            %
            %Parametres de sortida:
            %
            %   outs - les posicions dels outliers 
            %
            if nargin ==2
               switch alt
                   case {'all' 'ALL' 'All'}
                       %no fer res!   
                   otherwise
                       %ha de fer el mateix que si nomes ens passesin 1
                       %parametre
                       alt = [];
               end
            end
            
            if nargin ==1
                 alt = [];
            end
            
            if(obj.isModel)
                
                if(obj.conv == 1)
                    error('PCA/getOutLiers: can not be used in Variable Wise.');
                end
                cl = obj.getClass;
                for i = unique(cl)
                    index = (cl == i);
                    
                    %Treiem els outliers de manera automatica.
                    Qs=obj.getQ;
                    Ts=obj.getT2();
                    
                    %Treiem outliers per Q
                    %primer de tot calculem els quartils
                    qrts = prctile(Qs(index),[25 75]);
                    
                    %calculem el rang interquartilic
                    IRQ = qrts(2) - qrts(1);
                    
                    if isempty(alt)
                        %afegim el +-2.75 * RIQ per trobar els extrems
                        qrts(1) = qrts(1) - 2.75*IRQ;
                        qrts(2) = qrts(2) + 2.75*IRQ;
                    else
                        %afegim el +-RIQ per trobar els outliers
                        qrts(1) = qrts(1) - 1.5*IRQ;
                        qrts(2) = qrts(2) + 1.5*IRQ;
                    end
                    
                    %Calculem els outliers per Q i els guardem a ind
                    ind1 = or(Qs < qrts(1) , Qs > qrts(2));
                    
                    %tots els que son outliers de classes diferents de la que
                    %toca en realitat no ho son!
                    ind1(~index) = 0;
                    
                    
                    
                    
                    %Treiem outliers per T^2
                    %primer de tot calculem els quartils
                    qrts = prctile(Ts(index),[25 75]);
                    
                    %calculem el rang interquartilic
                    IRQ = qrts(2) - qrts(1);
                    
                    if isempty(alt)
                        %afegim el +-2.75 * RIQ per trobar els extrems
                        qrts(1) = qrts(1) - 2.75*IRQ;
                        qrts(2) = qrts(2) + 2.75*IRQ;
                    else
                        %afegim el +-RIQ per trobar els outliers
                        qrts(1) = qrts(1) - 1.5*IRQ;
                        qrts(2) = qrts(2) + 1.5*IRQ;
                    end
                    
                    %treiem els outliers extrems
                    ind = or(Ts < qrts(1) , Ts > qrts(2));
                    ind(~index) = 0;
                    
                    ant{i}= or(ind,ind1);
                    
                end
                
                outs = [ant{:}];
                outs= sum(outs,2);
                outs = outs >= 1;
                
            else
                %no es un model. No podem modificar les dades dels PCA No
                %model perque depenen del Model al qual s'han projectat.
                
                error('PCA/getOutLiers: The object must be a Model.');
                
            end
        end
 
%-------------------------------------------------------------------------%
%% ------------------------------- ALTRES --------------------------------  
%-------------------------------------------------------------------------%
        
        %PCA
        function [a b c] = pca(obj)
            %Mtode que retorna la informaci relacionada amb el model PCA.
            %Parametres d'entrada:
            %
            %   No en te
            %
            %Parametres de sortida:
            %
            %   a - Matriu de doubles que cont les components principals retingudes
            %   b - Matriu de doubles amb les projeccions de les observacions al subespai de components principals.
            %   c - Vector de doubles amb els valors propis associats a les components principals retingudes
            %
            
            if(~obj.isModel())
                errordlg('PCA\pca: You can''t use this method ober a non model object!','error');
            end
            
            if(isempty(obj.P)|| (isempty(obj.T) && obj.getBatchCount>0) || isempty(obj.Landa))
                %si no s'ha fet encara cap cop pca el fem
                if(obj.estandar~=1)
                    %estandaritzem les dades si cal
                    estandaritza(obj);
                end
                
                %ens quedem amb el numero de components principals que
                %necesitem segons el cas
                
                %comentat per poder assignar manualment el numero de
                %components principals amb el metode setNum i no haver de
                %fer el popup sempre.
%                if(isempty(obj.Num))%no s'ha fixat el numero de components principals que bolem
                    
                    switch obj.metode
                        case 0
                            %si no s'ha definit cap numero de components
                            %principals fem que l'usuari l'entri manualment
                            
                            
%                             
%                             r=obj.Z'*obj.Z/(size(obj.Z,1)-1);
%                             [V, D] = eig(r);
%                             obj.Landa=flipud(diag(D));
%                             obj.P=fliplr(V);
%                             obj.T=obj.Z*obj.P;

                            %if exist('princomp')
                            if license('test','statistics_toolbox')%mes eficient que l'exists 
                                [obj.P obj.T obj.Landa]=princomp(obj.Z);
                            else
                                r=obj.Z'*obj.Z/(size(obj.Z,1)-1);
                                [~,D,obj.P]=svd(r,'econ');
                                obj.Landa=diag(D);
                                obj.T=obj.Z*obj.P;
                            end
                            if(isempty(obj.Num))
                                if isempty(obj.Num_save)
                                    getNum(obj);
                                else
                                    obj.Num = obj.Num_save;
                                end
                            end
                        case 1
                            %toca fer KAiser
%[obj.P obj.T obj.Landa]=princomp(obj.Z);
                            %toca fer GrafScree
                            %if exist('princomp')
                            if license('test','statistics_toolbox')%mes eficient que l'exists 
                                [obj.P obj.T obj.Landa]=princomp(obj.Z);
                            else
                                r=obj.Z'*obj.Z/(size(obj.Z,1)-1);
                                [~,D,obj.P]=svd(r,'econ');
                                obj.Landa=diag(D);
                                obj.T=obj.Z*obj.P;
                            end
                            cDKaiser(obj);
                        case 2
                            %toca fer GrafScree
                            %if exist('princomp')
                            if license('test','statistics_toolbox')%mes eficient que l'exists 
                                [obj.P obj.T obj.Landa]=princomp(obj.Z);
                            else
                                r=obj.Z'*obj.Z/(size(obj.Z,1)-1);
                                [~,D,obj.P]=svd(r,'econ');
                                obj.Landa=diag(D);
                                obj.T=obj.Z*obj.P;
                            end
                            cGrafscree(obj);
                        case 3
                            %colze analitic
                            %if exist('princomp')
                            if license('test','statistics_toolbox')%mes eficient que l'exists 
                                [obj.P obj.T obj.Landa]=princomp(obj.Z);
                            else
                                r=obj.Z'*obj.Z/(size(obj.Z,1)-1);
                                [~,D,obj.P]=svd(r,'econ');
                                obj.Landa=diag(D);
                                obj.T=obj.Z*obj.P;
                            end
                            colzeanalitic(obj);
                        case 4
                            %toca % variana
                            %if exist('princomp')
                            if license('test','statistics_toolbox')%mes eficient que l'exists 
                                [obj.P obj.T obj.Landa]=princomp(obj.Z);
                            else
                                r=obj.Z'*obj.Z/(size(obj.Z,1)-1);
                                [~,D,obj.P]=svd(r,'econ');
                                obj.Landa=diag(D);
                                obj.T=obj.Z*obj.P;
                            end
                            cVarExp(obj);
                        case 5
                            %VRE
                            obj.structura=variance_reconstruction_error(obj.Z,eye(size(obj.Z,2)));
                            obj.Num=obj.structura.l_rec{obj.structura.n_loops};
                            obj.Z=obj.structura.X_rec{obj.structura.n_loops};
                            r=obj.Z'*obj.Z/(size(obj.Z,1)-1);
                            [V, D] = eig(r);
                            obj.Landa=flipud(diag(D));
                            obj.P=fliplr(V);
                            obj.T=obj.Z*obj.P;
                        otherwise
                            error('PCA\pca: Invalid component selection method')
                    end

%                end
                
                %agafem les dades que necesitem i la resta les eliminem
                obj.P = obj.P(:, 1:(obj.Num));
                obj.T = obj.T(:, 1:(obj.Num));
                obj.Landai=obj.Landa;
                obj.Landa = obj.Landa(1:(obj.Num));
                
            end
            if nargout ~= 0
            %retornem les dades demanades
            a=obj.P;
            b=obj.T;
            c=obj.Landa;
            end
        end
        %Projecta - crea un PCA no model a partir d'unes dades
        function nou = project(obj,x)
            %Mtode que projecta un objecte DATA sobre el model PCA. Es
            %retorna un nou objecte PCA amb la informaci corresponent de
            %les dades projectades.
            %
            %Parametres d'entrada:
            %
            %   x - Objecte DATA amb la informaci qu es vol projectar
            %       sobre el model PCA.
            %
            %Parametres de sortida:
            %
            %   nou - Objecte de tipus PCA amb tota la informaci en el
            %         subespai de les components principals de lobjecte.
            %

            %comprobem si les dades son correctes
            z=whos('x');
            if(not(strcmp( z.class , 'DATA')))
                %si ens passen algo que no es del tipus DATA Error
                error('PCA\project:waiting for DATA');
            end
            
            if(isempty(obj.P)||isempty(obj.Landa)||isempty(obj.mea)||(isempty(obj.T) && obj.getBatchCount>0))
                %si no hem calculat les dades del nostre model ho
                %fem ara
                obj.pca();
            end
            
            nou=PCA(x);%creem un nou PCA
            
            %li fixem la desviacio estandard i la mitjana del
            %nostre model
            nou.mea=obj.mea;
            nou.sta=obj.sta;
            nou.conv=obj.conv;
            nou.metode=obj.metode;
            nou.metodee=obj.metodee;
            nou.treuFM=obj.treuFM;
            nou.ContMetode=obj.ContMetode;
            
            %Li fixem els limits
            
            %Fem servir els getxxxx per asegurar que els inicialitzarem
            %amb algun valor
            
            nou.LimT2=obj.getT2Limit;%Limit de T^2
            nou.LimQ=obj.getQLimit;%Limit de Q
            nou.CLimQstd=obj.getQContributionLimitStd;%Q - Desviaci tipica
            nou.CLimQmea=obj.getQContributionLimitMean;%Q - Mitjana
            nou.CLimT2std=obj.getT2ContributionLimitStd;%T2 - Desviaci tipica
            nou.CLimT2mea=obj.getT2ContributionLimitMean;%T2 - Mitjana
            
            %L'estandaritzem
            nou.estandaritz2();
            
            %Fem que no sigui un model
            nou.model=0;%no es un model
            
            nou.structura=obj.structura;
            if(obj.metode==5)
                nou.Z=nou.Z(:,nou.structura.sensors{nou.structura.n_loops});
                nou.T=nou.Z*obj.P;
                nou.P=obj.P;
                nou.Landa=obj.Landa;
                nou.Landai=obj.Landai;
                nou.Num=obj.Num;
            else
                nou.T=nou.Z*obj.P;
                nou.P=obj.P;
                nou.Landa=obj.Landa;
                nou.Landai=obj.Landai;
                nou.Num=obj.Num;
            end
        end

%-------------------------------------------------------------------------%
%% ------------------------ PLOTS & VISUALITZADORS -----------------------   
%-------------------------------------------------------------------------%

        %grafic contribucions al limit de T2
        function plotT2Contribution(obj,mostrar)
            % Metode que fa un plot de les contribucins de T^2. Es pot
            % mostrar 1 batch o un conjunt de batch. 
            % Exemples:
            %  1, [1 2 3 4], 1:5, : ...
            %Parametres d'entrada:
            %
            %   mostrar - el numero de batch o el grup de batch que volem
            %             mostrar en el grafic 
            %
            %Parametres de sortida:
            %
            %   No en te
            %     
            if nargin == 1
                error('PCA\plotT2Contribution: we need another parameter!')
            end
            aux=obj.getT2Contribution();
            aux=aux(mostrar,:)';
            
            bar(aux,'HitTest','off');
         
            %mostrem el limits de contribucio del proces
            obj.plotLimCT2('k');
            
            if(obj.conv == 0)
                
                hold on;
                % Crea las lineas de las divisiones de las fases.
                z=obj.DATA.getStagesDurations();
                a=0;
                i=1;

                %Trobem el maxim i minim que ocuperan els plots
                maxim = max(max(max(aux)),max(obj.getT2ContributionLimitMean+obj.getT2ContributionLimitStd*3));
                minim = min(min(min(aux)),min(-(obj.getT2ContributionLimitMean+obj.getT2ContributionLimitStd*3))) * obj. ContMetode; %el Cont metode 0 = positius  1 = positius i negatius
                
                vars=obj.DATA.getVariableLabel();
                fases=obj.DATA.getStages();
                for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                    a=a+z(i);
                    line([a,a],[maxim+0.2,minim-0.2],'linestyle',':','color',[0.40,0.40,0.40],'HitTest','off');
                    text(a-z(i)/2,minim-0.2,fases(i),'FontSize',8,'LineWidth',0.5);
                    if(i==length(z))
                        i=0;
                        line([a,a],[maxim+0.2,minim-0.2],'linestyle','--','color','k','HitTest','off');
                        text(a-sum(z)/2,maxim+0.2,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                    end
                    i=i+1;
                end
                hold off;
            end
        end
        %grafic contribucions al limit de Q
        function plotQContribution(obj,mostrar)
            % Metode que fa un plot de les contribucins de Q. Es pot
            % mostrar 1 batch o un conjunt de batch. 
            % Exemples:
            %  1, [1 2 3 4], 1:5, : ...
            %Parametres d'entrada:
            %
            %   mostrar - el numero de batch o el grup de batch que volem
            %             mostrar en el grafic 
            %
            %Parametres de sortida:
            %
            %   No en te
            %  
            if nargin == 1
                error('PCA\plotQContribution: We need another parameter!')
            end
            
            aux=obj.getQContribution();
            aux=aux(mostrar,:)';
            bar(aux,'HitTest','off');
           
                        
            %mostrem el limits de contribucio del proces
            obj.plotLimCQ('k');
            if(obj.conv == 0)
                hold on;
                
                % Crea las lineas de las divisiones de las fases.
                z=obj.DATA.getStagesDurations();
                a=0;
                i=1;

                %Trobem el maxim i minim que ocuperan els plots
                maxim = max(max(max(aux)),max(obj.getQContributionLimitMean+obj.getQContributionLimitStd*3));
                minim = min(min(min(aux)),min(-(obj.getQContributionLimitMean+obj.getQContributionLimitStd*3))) * obj. ContMetode; %el Cont metode 0 = positius  1 = positius i negatius
                
                vars=obj.DATA.getVariableLabel();
                fases=obj.DATA.getStages();
                for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                    a=a+z(i);
                    line([a,a],[maxim+0.2,minim-0.2],'linestyle',':','color',[0.40,0.40,0.40],'HitTest','off');
                    text(a-z(i)/2,minim-0.2,fases(i),'FontSize',8,'LineWidth',0.5);
                    if(i==length(z))
                        i=0;
                        line([a,a],[maxim+0.2,minim-0.2],'linestyle','--','color','k','HitTest','off');
                        text(a-sum(z)/2,maxim+0.2,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                    end
                    i=i+1;
                end
                hold off;
            end
        end
        
        function plotT2MeanContribution(obj)
            %Aquesta funci dibuixa les mitjes de les contribucioons de T^2
            %de cada classe
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            cla = obj.getClass();
            num=unique(cla);
            col=hsv(length(num));%jenerem els colors
            in = 1;
            
            conts=obj.getT2Contribution();
            
            for j = num %per cada classe que te elements
                subplot(length(num),1,j)
                hold on;
                title(['Class ' num2str(j)])
                aux =(conts(cla == j,:))';
                mitja = mean( aux , 2);
                estan = std( aux ,0, 2);
                %plot(mitja,'color',col(in,:))
                aux2 = mitja - estan;
                if(obj.getContMetode == 0)
                    aux2(aux2 < 0) = 0;
                end
                area(mitja + estan,'FaceColor',col(in,:), 'BaseValue',min(aux2), 'LineStyle','none')
                area(aux2,'FaceColor','W','BaseValue',min(aux2),'LineStyle','none')
                in=in+1;
                
                
                %linies de fases
                % Crea las lineas de las divisiones de las fases.
                z=obj.DATA.getStagesDurations();
                a=0;
                i=1;
                maxim=max(max(mitja + estan)) +0.2;
                minim = min(min(aux2)) -0.2;
                vars=obj.DATA.getVariableLabel();
                fases=obj.DATA.getStages();
                for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                    a=a+z(i);
                    line([a,a],[maxim,minim],'linestyle',':','color',[0.40,0.40,0.40]);
                    aux =a-z(i)/2;
                    text(aux,minim,fases(i),'FontSize',8,'LineWidth',0.5);
                    if(i==length(z))
                        i=0;
                        line([a,a],[maxim,minim],'linestyle','--','color','k');
                        text(aux - sum(z)*2/3,maxim,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                    end
                    i=i+1;
                end
            end
        end
        
        function plotQMeanContribution(obj)
            %Aquesta funci dibuixa les mitjes de les contribucioons de Q
            %de cada classe
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            cla = obj.getClass();
            num=unique(cla);
            col=hsv(length(num));%jenerem els colors
            in = 1;
            
            conts=obj.getQContribution();
            
            for j = num %per cada classe que te elements
                subplot(length(num),1,j)
                hold on;
                title(['Class ' num2str(j)])
                aux =(conts(cla == j,:))';
                mitja = mean( aux , 2);
                estan = std( aux ,0, 2);
                aux2 = mitja - estan;
                if(obj.getContMetode == 0)
                    aux2(aux2 < 0) = 0;
                end
                area(mitja + estan,'FaceColor',col(in,:), 'BaseValue',min(aux2), 'LineStyle','none')
                area(aux2,'FaceColor','W','BaseValue',min(aux2),'LineStyle','none')
                in=in+1;
                
                %linies de fases
                % Crea las lineas de las divisiones de las fases.
                z=obj.DATA.getStagesDurations();
                a=0;
                i=1;
                maxim=max(max(mitja + estan)) +0.2;
                minim = min(min(aux2)) -0.2;
                vars=obj.DATA.getVariableLabel();
                fases=obj.DATA.getStages();
                for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                    a=a+z(i);
                    line([a,a],[maxim,minim],'linestyle',':','color',[0.40,0.40,0.40]);
                    aux =a-z(i)/2;
                    text(aux,minim,fases(i),'FontSize',8,'LineWidth',0.5);
                    if(i==length(z))
                        i=0;
                        line([a,a],[maxim,minim],'linestyle','--','color','k');
                        text(aux - sum(z)*2/3,maxim,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                    end
                    i=i+1;
                end
            end
        end
        
        function handle = plotT2Q(obj,forma,color,aux)
            %Crea el grafic estndard de control T^2 vs Q.
            %
            %Parametres d'entrada:
            %
            %   forma -(opcional) Per indicar la forma dels punts
            %   color -(opcional) Per forar el color dels punts
            %   aux -(opcional) 'log' posara els eixos en escala
            %                    logaritmica i amb 'no' no borrara els
            %                    datatips existents nomes afegira les noves
            %                    dades
            %
            %Parametres de sortida:
            %
            %   handle - el handle del plot
            %
            
            %recupero les dades i les guardo en local
            T22=obj.getT2;
            Q=obj.getQ;
            classe=obj.getClass; 
            dcm_obj = datacursormode(gcf);
            
            hold on
            %tr indicara si bolem colors o ens els passen
            tr= 1==1;
            if isempty(whos('color')) || isempty(color)
                col=hsv(length(unique(classe)));%jenerem els colors
                tr= 1==0;
            end
            
            if isempty(whos('forma'))
                forma = '.';%pintem punts
            end
            in =1;
            num=unique(classe);
            handle=zeros(1,max(num));
            for i = num%per cada classe
                if tr %color o no?
                    handle(i)=semilogy(T22(classe==i),Q(classe==i),forma,'color',color);
                else
                    handle(i)=semilogy(T22(classe==i),Q(classe==i),forma,'color',col(in,:));
                    in=in+1;
                end
            end
            
            if exist('aux','var') && strcmp(aux,'no')
                userdata =get(gca,'Userdata');
                noms=obj.getBatchLabel();
                
                userdata.noms = {userdata.noms{:}, noms{:}};
                userdata.punts{1} = [userdata.punts{1}; T22];
                userdata.punts{2} = [userdata.punts{2}; Q];
            else
                userdata.punts={T22,Q};
                userdata.noms=obj.getBatchLabel();
            end
            if nargin ~= 4 %si es 4 no volen limits
                line([obj.getT2Limit obj.getT2Limit],[1 max(Q)],'color','k','HitTest','off') %T2
                line([1 max(T22)],[obj.getQLimit obj.getQLimit],'color','k','HitTest','off') %Q
            end
            
            %posem a logaritmic
            if nargin >= 4 && strcmp(aux,'log')
                ax = axis;
                set (gca, 'yscale', 'log')
                set (gca, 'xscale', 'log')
                axis (ax)
            end
            
            %nom dels eixos
            xlabel('T2')
            ylabel('Q')
            
            hold off
            
            set(gca,'userdata',userdata);
            set(dcm_obj,'UpdateFcn',@tipmelot)
        end
               
        function plotScores(varargin)
            %Grafica els scores que se li passen per parametre.
            %depenent de la quantitat d'scores desitjada el gradic
            %resultant variar.
            %
            % 1 score es tealitzara la grafia d'aquest al llarg del temps
            % 2 scores es realitzara la grafica d'un contra l'altre
            % 3 scores es realitzara la grafica 3D 
            %
            % A cada classe se li assignara un color diferent
            
            obj = varargin{1};
            switch length(varargin)
                case 1
                    errordlg('We need parameters!','Error');
                    
                case 2
                    a=varargin{2};
                    ts=obj.getT();
                    dcm_obj = datacursormode(gcf);
                    
                    hold on
                    
                    clas=obj.DATA.getClass;
                    col=hsv(max(clas));%el color dels punts!
                    for i=unique(clas)
                        index=clas ==i;
                        plot(ts(index,a),'.','color',col(i,:));
                    end
                    userdata.punts={ts(:,a)};
                    userdata.noms=obj.getBatchLabel();
                    line([0,length(ts)],[sqrt(obj.Landa(a))*3, sqrt(obj.Landa(a))*3],'color','K','HitTest','off')
                    line([0,length(ts)],[-sqrt(obj.Landa(a))*3, -sqrt(obj.Landa(a))*3],'color','K','HitTest','off')
                    axis equal
                    xlabel('Time')
                    ylabel(['T(' num2str(a) ') (' num2str(sum(obj.Landa(a))*100/sum(obj.Landa)) '%)'])
                    hold off
                    
                    set(gca,'userdata',userdata);
                    set(dcm_obj,'UpdateFcn',@tipme)
                    
                case 3
                    a=varargin{2};
                    b=varargin{3};
                    ts=obj.getT();
                    dcm_obj = datacursormode(gcf);
                    hold on
                    
                    %per optimitzar guardem les lambdes a una variable!
                    lamb = obj.Landai;
                    sumlamb = sum(lamb);
                    
                    [X,Y,Zeta]=ellipsoid(0,0,0,sqrt(lamb(a))*3,sqrt(lamb(b))*3,0);
                    surf(X,Y,Zeta,'EdgeColor','none','facecolor',[0.9 0.9 0.9],'HitTest','off');
                    
                    clas=obj.DATA.getClass;
                    col=hsv(max(clas));%el color dels punts!
                    for i=unique(clas)
                        index=clas ==i;
                        plot(ts(index,a),ts(index,b),'.','color',col(i,:));
                    end
                    
                    userdata.punts={ts(:,a),ts(:,b)};
                    userdata.noms=obj.getBatchLabel();
                    

                    axis equal
                    xlabel(['T(' num2str(a) ') (' num2str((lamb(a))*100/sumlamb) '%)'])
                    ylabel(['T(' num2str(b) ') (' num2str(sum(lamb(b))*100/sumlamb) '%)'])
                    hold off
                    
                    set(gca,'userdata',userdata);
                    set(dcm_obj,'UpdateFcn',@tipme)
                    
                case 4
                    a=varargin{2};
                    b=varargin{3};
                    c=varargin{4};
                    ts=obj.getT();
                    dcm_obj = datacursormode(gcf);
                    set(dcm_obj,'SnapToDataVertex','off');
            
                    
                    clas=obj.DATA.getClass;
                    col=hsv(max(clas));%el color dels punts!
                    for i=unique(clas)
                        index=clas ==i;
                        plot3(ts(index,a),ts(index,b),ts(index,c),'.','color',col(i,:));
                        hold on
                    end
                    
                    %per optimitzar guardem les lambdes a una variable!
                    lamb = obj.Landai;
                    sumlamb = sum(lamb);
                    
                    [X,Y,Zeta]=ellipsoid(0,0,0,sqrt(lamb(a))*3,sqrt(lamb(b))*3,sqrt(lamb(c))*3);
                    surf(X,Y,Zeta,'EdgeColor','none','facecolor',[0.9 0.9 0.9],'HitTest','off');                    
                    
                    
                    userdata.punts={ts(:,a),ts(:,b),ts(:,c)};
                    userdata.noms=obj.getBatchLabel();

                    alpha(0.3);
                    axis equal
                    xlabel(['T(' num2str(a) ') (' num2str((lamb(a))*100/sumlamb) '%)'])
                    ylabel(['T(' num2str(b) ') (' num2str(sum(lamb(b))*100/sumlamb) '%)'])
                    zlabel(['T(' num2str(c) ') (' num2str(sum(lamb(c))*100/sumlamb) '%)'])
                    
                    hold off
                    set(gca,'userdata',userdata);
                    set(dcm_obj,'UpdateFcn',@tipme)
                    
                otherwise
                    errordlg('4D is not allowed!','Error');
                    
            end
            
        
            
        end
        
        function plotVarianceCaptured(obj)
            %mostra el % de variana explicat de cada una de les variables,
            %en diferents colors es representa quines components principals
            %estan explicant quines variables.
            
            mP=obj.getP();%agafem les P
            %comprobem en quin format estan desdoblades les dades
            if(obj.conv == 0)
                %Batch-Wise
                mP= mP.^2;
                aux= reshape(mP,obj.DATA.getSampleCount,obj.Num * obj.DATA.getVariableCount);
                v1=reshape((sum(aux,1)'./ obj.DATA.getSampleCount) * 100,obj.DATA.getVariableCount,obj.Num)';
                bar(v1','stacked');
                if ~isempty(obj.DATA.getVariableLabel)
                    set(gca,'XTickLabel',obj.DATA.getVariableLabel);
                    set(gca,'XTick',1:length(obj.DATA.getVariableLabel));
                end
                inst=cell(1,obj.Num);
                for i=1:obj.Num
                   inst{i} = ['% ' num2str(i)] ;
                end
                legend(inst)
            else
                %Variable-Wise
                %nomes podem mostrar el % de cada variable
                bar(mP.^2 * 100,'stacked');
                inst=cell(1,obj.Num);
                for i=1:obj.Num
                   inst{i} = ['% ' num2str(i)] ;
                end
                legend(inst)
            end
            xlabel('Variable names')
            ylabel('%')
        end
        
        function plotQSignatures(obj)
            %Grafic que mostra les signatures de Q, es mostrara una grafica
            %on es mostrara per cada lot i instant de temps si falla o no
            %falla, les classes es diferenciaran amb colors asignats
            %automaticament.
            
            %preparem les dades
            CBgcQ=obj.getQContribution();
            CBgcQ= CBgcQ> repmat((obj.getQContributionLimitMean + obj.getQContributionLimitStd *3),[size(CBgcQ,1) 1] ); %pasem a binari
            class = obj.getClass;
            mcl = max(class);
            
            hold on
            %els colors de les classes
            colormap('lines');
            color= colormap('lines');
            
            %Colorets per la llegenda
            
            leg = cell(mcl,1);
            for i = 1 : mcl
                plot(0,0,'.','color',color(i,:),'HitTest','off');
                leg{i} = ['Classe' num2str(i)];
            end
            
            %titols i llegendes
            title('Signatures de Q');
            xlabel('Time * Variables');
            ylabel('Batch Number');
            legend(leg);
            
            %Passem a dibuixar totes les dades
            sbq1 = size(CBgcQ,1);%evitem repetir crides
            sbq2 = size(CBgcQ,2);%evitem repetir crides
            set (gca, 'yscale', 'linear')
            set (gca, 'xscale', 'linear')
            set (gca,'xLim',[-2  sbq2+10])
            set (gca,'yLim',[-2  sbq1+10])
            set(gca,'Ytick',1 :floor(sbq1/20)+1: sbq1)
            set(gca,'Xtick',1 :floor(sbq2/20)+1: sbq2)
            set(gca,'Yticklabel',1 :floor(sbq1/20)+1: sbq1)
            set(gca,'Xticklabel',1 :floor(sbq2/20)+1: sbq2)
            aux2 = repmat((1:sbq1)',[1 sbq2]);%per optimitzar ho fem fora el bucle
            aux1 = CBgcQ .* repmat((1 : sbq2), [sbq1 1] );%per optimitzar ho fem fora el bucle
            for i = 1 : mcl
                index = i ==class;
                plot(  aux1 .* repmat(index', [1 sbq2]) , aux2 , '.', 'color', color(i,:),'HitTest','off');
            end
            %eliminem els ounts erronis
            plot(zeros(obj.getBatchCount+1,1),0:obj.getBatchCount,'.W','HitTest','off');
            
            % Crea las lineas de las divisiones de las fases.
            z=obj.DATA.getStagesDurations();
            a=0;
            i=1;
            maxim=max(get(gca,'Ytick')) + 5;
            vars=obj.DATA.getVariableLabel();
            fases=obj.DATA.getStages();
            for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                a=a+z(i);
                line([a,a],[maxim,-0.2],'linestyle',':','color',[0.40,0.40,0.40],'HitTest','off');
                aux =a-z(i)/2;
                text(aux,-0.2,fases(i),'FontSize',8,'LineWidth',0.5);
                if(i==length(z))
                    i=0;
                    line([a,a],[maxim,-0.2],'linestyle','--','color','k','HitTest','off');
                    text(aux - sum(z)*2/3,maxim,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                end
                i=i+1;
            end
            
            
            hold off;
        end
        
        function plotT2Signatures(obj)
            %Grafic que mostra les signatures de T^2, es mostrara una
            %grafica on es mostrara per cada lot i instant de temps si
            %falla o no falla, les classes es diferenciaran amb colors
            %asignats automaticament.
            
            %Recuperem les contribucions de tots el elements
            CBgcQ=obj.getT2Contribution();
            
            %calculem els que es passen pel criteri de 3 sigma
            CBgcQ= CBgcQ> repmat((obj.getT2ContributionLimitMean + obj.getT2ContributionLimitStd * 3 ),[size(CBgcQ,1) 1] ); %pasem a binari
            
            %classes i el nombre maxim de classes per optimitzar
            class = obj.getClass;
            mcl = max(class);
            
            hold on
            %els colors de les classes
            colormap('lines');
            color= colormap('lines');
            
            %Colorets per la llegenda
            leg = cell(mcl,1);
            
            %posem 3 punts per fer la llegenda
            for i = 1 : mcl
                plot(0,0,'.','color',color(i,:),'HitTest','off');
                leg{i} = ['Classe' num2str(i)];
            end

            %titols i llegendes
              title('Signatures de T2');
              xlabel('Time * Variables');
              ylabel('Batch number');
              legend(leg);

            %Passem a dibuixar totes les dades   
            sbq1 = size(CBgcQ,1);%evitem repetir crides
            sbq2 = size(CBgcQ,2);%evitem repetir crides
            set (gca, 'yscale', 'linear')
            set (gca, 'xscale', 'linear')
            set (gca,'xLim',[-2  sbq2+10])
            set (gca,'yLim',[-2  sbq1+10])          
            set(gca,'Ytick',1 :floor(sbq1/20)+1: sbq1)
            set(gca,'Xtick',1 :floor(sbq2/20)+1: sbq2)
            set(gca,'Yticklabel',1 :floor(sbq1/20)+1: sbq1)
            set(gca,'Xticklabel',1 :floor(sbq2/20)+1: sbq2)
            
            aux2 = repmat((1:sbq1)',[1 sbq2]);%per optimitzar ho fem fora el bucle
            aux1 = CBgcQ .* repmat((1 : sbq2), [sbq1 1] );%per optimitzar ho fem fora el bucle
            for i = 1 : mcl
                index = i ==class;
                plot(  aux1 .* repmat(index', [1 sbq2]) , aux2 , '.', 'color', color(i,:),'HitTest','off');
            end
            %eliminem els ounts erronis
            plot(zeros(obj.getBatchCount+1,1),0:obj.getBatchCount,'.W','HitTest','off');
            
            % Crea les linees de divisions de fases.
            z=obj.DATA.getStagesDurations();
            a=0;
            i=1;
            maxim=max(get(gca,'Ytick')) + 5;
            vars=obj.DATA.getVariableLabel();
            fases=obj.DATA.getStages();
            for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                a=a+z(i);
                line([a,a],[maxim,-0.2],'linestyle',':','color',[0.40,0.40,0.40],'HitTest','off');
                aux =a-z(i)/2;
                text(aux,-0.2,fases(i),'FontSize',8,'LineWidth',0.5);
                if(i==length(z))
                    i=0;
                    line([a,a],[maxim,-0.2],'linestyle','--','color','k','HitTest','off');
                    text(aux - sum(z)*2/3,maxim,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                end
                i=i+1;
            end
            hold off;
        end
        
        function plotScreePlot(obj)
            %Grafica de les Lambdas, amb punt seleccionat per a nombre de
            %components principals marcat amb asterisc vermell
            hold on;
            plot(obj.Landai(obj.Landai~=0));
            plot(obj.Num,obj.Landa(obj.Num),'*r');
            legend({'Lambda';'Priincipal components number'});
            hold off;
        end
        
        function plotMean(obj)
            %Crea una grafica on es ostra la mitja de les variables
            
            hold on
            plot(obj.mea,'b')
            legend({'Mean'});
            
            %linies de fases
            % Crea las lineas de las divisiones de las fases.
            z=obj.DATA.getStagesDurations();
            a=0;
            i=1;
            maxim=max(max(obj.mea)) +0.2;
            minim = min(min(obj.mea)) -0.2;
            vars=obj.DATA.getVariableLabel();
            fases=obj.DATA.getStages();
            for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                a=a+z(i);
                line([a,a],[maxim,minim],'linestyle',':','color',[0.40,0.40,0.40]);
                aux =a-z(i)/2;
                text(aux,minim,fases(i),'FontSize',8,'LineWidth',0.5);
                if(i==length(z))
                    i=0;
                    line([a,a],[maxim,minim],'linestyle','--','color','k');
                    text(aux - sum(z)*2/3,maxim,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                end
                i=i+1;
            end            
            hold off;
        end
        
        function plotStd(obj)
            %Crea una grafica on es ostra la desviacio estandard de les
            %variables del procs
            
            hold on
            plot(obj.sta,'b')
            legend({'Standard Desviation'});
            
            %linies de fases
            % Crea las lineas de las divisiones de las fases.
            z=obj.DATA.getStagesDurations();
            a=0;
            i=1;
            maxim = max(max(obj.sta)) +0.2 ;
            minim = min(min(obj.sta)) -0.2;
            vars=obj.DATA.getVariableLabel();
            fases=obj.DATA.getStages();
            for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                a=a+z(i);
                line([a,a],[maxim,minim],'linestyle',':','color',[0.40,0.40,0.40]);
                aux =a-z(i)/2;
                text(aux,minim,fases(i),'FontSize',8,'LineWidth',0.5);
                if(i==length(z))
                    i=0;
                    line([a,a],[maxim,minim],'linestyle','--','color','k');
                    text(aux - sum(z)*2/3,maxim,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                end
                i=i+1;
            end            
            hold off;
        end
        
        function plotP(obj,index)
            %Grafica les components Principals indicades en l'index
            %
            %Parametres d'entrada:
            %
            %   index - Index valid en matlab indicant els batch a graficar 
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            hold on

            plot(obj.P(:,index))
            mis=cell(1,length(index));
            for i = 1:length(index)
               mis{i} = ['Principal Component ' num2str(index(i))] ;
            end
            legend(mis)
            %linies de fases
            % Crea las lineas de las divisiones de las fases.
            z=obj.DATA.getStagesDurations();
            a=0;
            i=1;
            maxim=max(max(obj.P(:,index))) ;
            minim = min(min(obj.P(:,index))) ;
            vars=obj.DATA.getVariableLabel();
            fases=obj.DATA.getStages();
            for x = 1:length(z)*length(obj.DATA.getVariableLabel)
                a=a+z(i);
                line([a,a],[maxim,minim],'linestyle',':','color',[0.40,0.40,0.40]);
                aux =a-z(i)/2;
                text(aux,minim,fases(i),'FontSize',8,'LineWidth',0.5);
                if(i==length(z))
                    i=0;
                    line([a,a],[maxim,minim],'linestyle','--','color','k');
                    text(aux - sum(z)*2/3,maxim,vars(x/length(z)),'FontSize',15,'LineWidth',0.7);
                end
                i=i+1;
            end            
            hold off;
        end
        
        function plotLoadings(varargin)
            %Grafica els loadings indicats.
            %Depenent del nombre de loadings demanat es realitzarant
            % grafics diferents.
            % 1 sol loading s'el representara al llarg del temps
            % 2 o 3 loadings sels representara un contre l'altre en 2D o 3D
            %
            %Cada una de les variables es mostrara d'un color diferent
            %assignat automaticament, dins una mateixa variable es mostrara
            %un degradat de color al llarg del temps.
            
            obj = varargin{1};
            switch length(varargin)
                case 1
                    errordlg('we need parameters!','Error');
                case 2
                    Ps=obj.getP();
                    a = varargin{2};
                    
                    vars = obj.getVariableCount;
                    temps = obj.getSampleCount;
                    col=hsv(vars);%el color dels punts!
                    hold on
                    for i=1:vars
                        index = ((i-1) * temps) +1 : i*temps;
                        plot(Ps(index,a),'-','color',col(i,:));
                    end                                      
                    hold off
                    
                    xlabel('Time')
                    ylabel(['PC ' num2str(a) ' (' num2str(sum(obj.Landa(a))*100/size(Ps,1)) '%)'])
                    
                    noms = obj.DATA.getVariableLabel;
                    if ~isempty(noms)
                        legend(noms)
                    end
                case 3
                    Ps=obj.getP();
                    a = varargin{2};
                    b = varargin{3};
                    
                    vars = obj.getVariableCount;
                    temps = obj.getSampleCount;
                    col=degradatcolors(vars, temps);%el color dels punts!
                    hold on
                    for j=temps:-1:1
                        for i=1:vars
                            ind =(j + (i-1)*temps);
                            plot(Ps(ind,a),Ps(ind,b),'*','color',col(ind,: ));
                        end
                    end
                    
                    minimx = min(min(Ps(:,a)));
                    minimy = min(min(Ps(:,b)));
                    maximx = max(max(Ps(:,a)));
                    maximy = max(max(Ps(:,b)));
                    line([0,0],[(minimy - abs(minimy/10)), (maximy + maximy/10)],'color','K','linestyle','--');
                    line([(minimx - abs(minimx/10)), (maximx + maximx/10)],[0,0],'color','K','linestyle','--');
                    
                    hold off
                    xlabel(['PC ' num2str(a) ' (' num2str(sum(obj.Landa(a))*100/size(Ps,1)) '%)'])
                    ylabel(['PC ' num2str(b) ' (' num2str(sum(obj.Landa(b))*100/size(Ps,1)) '%)'])
                    
                    noms = obj.DATA.getVariableLabel;
                    if ~isempty(noms)
                        legend(noms)
                    end
                    
                case 4
                    Ps=obj.getP();
                    a = varargin{2};
                    b = varargin{3};
                    c = varargin{4};
                                       
                    vars = obj.getVariableCount;
                    temps = obj.getSampleCount;
                    col=degradatcolors(vars, temps);%el color dels punts!
                    for j=temps:-1:1
                        for i=1:vars
                            ind =(j + (i-1)*temps);
                            plot3(Ps(ind,a),Ps(ind,b),Ps(ind,c),'*','color',col(ind,:));
                            hold on
                        end
                    end
                    
                      minimx = min(min(Ps(:,a)));
                    minimy = min(min(Ps(:,b)));
                    minimz = min(min(Ps(:,c)));
                    
                    maximx = max(max(Ps(:,a)));
                    maximy = max(max(Ps(:,b)));
                    maximz = max(max(Ps(:,c)));
                    
                    
                    line([0,0],[(minimy - abs(minimy/10)), (maximy + maximy/10)],[0,0],'color','K','linestyle','--');
                    line([(minimx - abs(minimx/10)), (maximx + maximx/10)],[0,0],[0,0],'color','K','linestyle','--');
                    line([0,0],[0,0],[(minimz - abs(minimz/10)),(maximz + maximz/10 )],'color','K','linestyle','--');
                    
                    hold off
                    
                    xlabel(['PC ' num2str(a) ' (' num2str(obj.Landa(a)*100/size(Ps,1)) '%)'])
                    ylabel(['PC ' num2str(b) ' (' num2str(obj.Landa(b)*100/size(Ps,1)) '%)'])
                    zlabel(['PC ' num2str(c) ' (' num2str(obj.Landa(c)*100/size(Ps,1)) '%)'])
                    
                    noms = obj.DATA.getVariableLabel;
                    if ~isempty(noms)
                        legend(noms)
                    end
                otherwise
                    errordlg('4D is not allowed!','Error');
            end
        end
        
        function plotOutLiers(obj)
            %Funci que fa un boxplot per T i Q on es mostraran els
            %outliers i en el datatip es posen els corresponents numeros de
            %lot i noms del lot.
            %
            %Parametres d'entrada:
            %
            %   Aquest metode no te cap parametre d'entrada
            %
            %Parametres de sortida:
            %
            %   Aquest metode no te cap parametre de sortida
            %
            
            if(obj.conv == 1)
                error('PCA/plotOutLiers: can not be used in Variable Wise.');
            end
            
            %recuperem Q i T^2 i fem el diagrama de caixa
            Qs=obj.getQ;
            Ts=obj.getT2();
            
            %Recuperem tambe les Classes i noms per optimitzar el bucle
            cl=obj.getClass;
            noms = obj.getBatchLabel();
            
            %per cada classe busquem els outlies
            for i = unique(cl)
                %Fem una nova figura i en recuperem el cursor per editar el
                %datatip
                fig=figure;
                dcm_obj = datacursormode(fig);
                
                %creem un axes i el pasem a boxplot perque el faci a la
                %figura on tenim capturat el ratoli
                axe=axes;
                boxplot(axe,[Ts(cl==i) Qs(cl==i)]);%dibuixem el boxplot de la classe
                
                hold on;
                %posem les dades que bolem mostrar al datatip al userdata
                set(gca,'userdata',{Ts, Qs, noms});
                
                %Posem el titoli etiquetes als eixos
                title(['Class ' num2str(i) ]);
                hold off;
                set(gca,'XTickLabel',{'T^2' 'Q'});
                
                %posem la funci de datatip a una de propia (datatiper localitzada al final d'aquest fitxer)
                set(dcm_obj,'UpdateFcn',@datatiper);

            end
            
        end
%-------------------------------------------------------------------------%
%% -------------------- Visualitzaci de l'objecte -----------------------
%-------------------------------------------------------------------------%    

        %que fer quan fan doble click subre un objecte
        function openvar(nom,~)%el parametre comentat es l'objecte actual
            %Aquesta es una funci que sobreescriu l'event per defecte de
            %doble click sobre un objecte de tipus PCA en el workspace.
            %sobreescrivim la funci de visualitzaci per defecte,
            %el parametre ignorat s l'objecte actual el nom ens el passen
            %en format text. Aqui invocarem una pantalla especifica per
            %visualitzar l'objecte.
            %
            %!!ATENCIO!! -> Aquest metode no hauria de ser invocat per
            %l'usuari directament a menys que sapiga molt b que esta fent!
            x=whos('nom');
            if strcmp(x.class,'PCA')
                %no puc saber el nom de la variable -> no fem res!
            else
                if(evalin('base',['size(' nom ');']) == [1 1])
                        Menu_Configura_PCA(nom)
                 end
            end
        end
        
        
        %com es mostra a consola
        function disp(obj)
            %funci que sobreescriu la visualitzaci de l'objecte per
            %consola i en alguns menus desplegables de matlab.
            %Mostrarem informaci util per l'usuari (tamany de les dades)
            %en comptes de la informaci per defecte.
            %
            % Aquesta funci s'invoca automaticament per Matlab quan cal
            % l'usuari no necesita invocarla en cap moment.
           obj.DATA.disp(1)    
           disp(['unique(classes) = ' num2str(unique(obj.getClass))])
        end
        
      function s = saveobj(varargin)
          %Sobreescripture del metode Save de matlab perque funcioni
          %correctament amb objectes de tipus PCA.
          %
          % Varargin contindra l'objecte actual
          % s - l'estructura que guardarem
          
          obj = varargin{1};
          %retorna l'objecte en format Structura
          s.DATA = obj.DATA;
          s.estandar = obj.estandar;
          s.model = obj.model;
          s.Z = obj.Z;
          s.P = obj.P;
          s.Num = obj.Num;
          s.Num_save = obj.Num_save;
          s.T = obj.T;
          s.Landa = obj.Landa;
          s.Landai = obj.Landai;
          s.sta = obj.sta;
          s.mea = obj.mea;
          s.E = obj.E;
          s.T2 = obj.T2;
          s.LimT2 = obj.LimT2;
          s.LimQ = obj.LimQ;
          s.metode = obj.metode;
          s.metodee = obj.metodee;
          s.treuFM = obj.treuFM;
          s.conv = obj.conv;
          s.analitic = obj.analitic;
          s.structura = obj.structura;
          s.CLimQstd = obj.CLimQstd;
          s.CLimQmea = obj.CLimQmea;
          s.CLimT2std = obj.CLimT2std;
          s.CLimT2mea = obj.CLimT2mea;
          s.ContMetode = obj.ContMetode;
      end
    end
    
    methods (Static)
        function obj = loadobj(obj)
          %Sobreescripture del metode load de matlab perque funcioni
          %correctament amb objectes de tipus PCA.
          %
          % obj -entra com a structura i surt com a objecte PCA
            if isstruct(obj)
                % Call default constructor
                newObj = PCA(DATA());
                % Assign property values from struct
                newObj.DATA = obj.DATA;
                newObj.estandar = obj.estandar;
                newObj.model = obj.model;
                newObj.Z = obj.Z;
                newObj.P = obj.P;
                newObj.Num = obj.Num;
                newObj.Num_save = obj.Num_save;
                newObj.T = obj.T;
                newObj.Landa = obj.Landa;
                newObj.Landai = obj.Landai;
                newObj.sta = obj.sta;
                newObj.mea = obj.mea;
                newObj.E = obj.E;
                newObj.T2 = obj.T2;
                newObj.LimT2 = obj.LimT2;
                newObj.LimQ = obj.LimQ;
                newObj.metode = obj.metode;
                newObj.metodee = obj.metodee;
                newObj.treuFM = obj.treuFM;
                newObj.conv = obj.conv;
                newObj.analitic = obj.analitic;
                newObj.structura = obj.structura;
                newObj.CLimQstd = obj.CLimQstd;
                newObj.CLimQmea = obj.CLimQmea;
                newObj.CLimT2std = obj.CLimT2std;
                newObj.CLimT2mea = obj.CLimT2mea;
                newObj.ContMetode = obj.ContMetode;
                obj = newObj;
            end
        end
    end
   
%-------------------------------------------------------------------------%
%% --------------------------- METODES PRIVATS ---------------------------  
%-------------------------------------------------------------------------%
    
    methods(Access = private)
        % Si es necesita que siguin publics per qualsebol rao, es pot 
        %canviar "methods(Access = private)" per "methods(Access = public)" o "methods"
        %aquests metodes no seran accesibles des de fora de l'objecte,
        %aqui posarem tots els metodes que no ens interessa que els
        %usuaris utilitzin en qualsebol moment
        
%-------------------------------------------------------------------------%
%% -------- METODES PER CALCULAR EL NOMBRE DE COMPONENTS PRINCIPALS ------   
%-------------------------------------------------------------------------%

        %metode de Kaiser
        function cDKaiser(obj)
            %Aplica el criteri de Kaiser per fixar el numero de components
            %principals que agafarem. Aquesta funcio es privada
            obj.Num = find(obj.Landa >= 1,1,'last');%l'ultim mes gran que 1
            
            if isempty(obj.Num)
                obj.Num= 1;
            end
        end
        %metode de Graf Scree
        function cGrafscree(obj)
            %Mostrem el grafic i demanem a l'usuari que ens entri el
            %punt on es trova el colze. Aquesta funcio es privada
            figure
            plot(obj.Landa);
            N = str2double(inputdlg('Entra el punt on hi ha el colze:'));
            %N=input('Entra el punt on hi ha el colze:');
            z=whos('N');
            while(~strcmp(z.class,'double'))
                plot(obj.Landa);
                N = str2double(inputdlg('Entra el punt on hi ha el colze:'));
                %N=input('Entra el punt on hi ha el colze:');
                z=whos('N');
            end
            obj.Num=N;
            
        end
        %metode de la variana explicada
        function cVarExp(obj)
            %Calcula el nombre de components necessaris perque la
            %variana explicada sigui el % entrat per l'usuari. Aquesta
            %funcio es privada
            N = str2double(inputdlg('Choose % of variance:'));
            %N=input('Entra el % de variana:');
            z=whos('N');
            while(~strcmp(z.class,'double') || (N>100 || N<0))
                %N=input('Entra el % de variana:');
                N = str2double(inputdlg('Choose % of variance:'));
                z=whos('N');
            end
            
            i = find(cumsum(obj.Landa)*100 / sum(obj.Landa)>N,1);
            
            obj.Num=i;

        end
        %metode grafic amb recomanacio de punt (automatic)!
        function colzeanalitic(obj)
            %necesitem minim 3 variables per aplicar el metode.Busca el
            %punt mes proper a l'origen de cordenades (el colze). Aquesta
            %funcio es l'automatitzaci del metode Graff Scree i es privada
            
            i=2;
            zmax=0;
            
            lan = obj.Landa~=0;
            lan=obj.Landa(lan);
            Land=lan/max(lan);
            
            
            while(i<size(Land,1)-1)
                %calculem la distancia del (0,0) a la recta que formen
                %l'anterior i el seguent a l'actual
                z=dpr(0,0,(i-1)/(size(Land,1)),Land(i-1),(i+1)/(size(Land,1)),Land(i+1));%( px,py , r1x,r1y , r2x,r2y) (primer la x i la y del punt i despres x,y x, y de dos punts de la recta)
                if(zmax<z)
                    zmax=z;%encara no hem arrivat al colze
                    x=i;
                end
                i=i+1;%avancem l'actual
            end
            obj.Num=x;
        end
        
%-------------------------------------------------------------------------%
%% --------------------- METODES PER ESTANDARITZAR ----------------------- 
%-------------------------------------------------------------------------%
        
        %estandaritza
        function estandaritza(obj)
            %estandaritza la mostra. Aquesta funcio es privada
            if(obj.estandar~=1)
                
                switch (obj.metodee)
                    %cas1 = AUto Scaling
                    case 1
                        %tan si es una matriu 2D, 3D passada a 2D es
                        %fara igual
                        %demanem les dades a l'objecte DATA
                        data=getRawData(obj);
                        %alculem la mitjana i la desviaci estandard
                        obj.mea=mean(data,1);
                        obj.sta=std(data,0,1);

                        %Cas 2 = Grup Scaling
                    case 2
                        %agafem el nombre de variables
                        vars=obj.DATA.getVariableCount();
                        if(obj.conv==0)%batch wide
                            %demanem les dades a l'objecte DATA
                            data=getRawData(obj);
                            duration=size(data,2)/vars;%numero de columnes per cada variable
                            for i=1:1:vars
%                                 % Seleccionem les columnes que pertanyen a la variable i
                                 aux = data(:,duration * (i - 1) + 1:duration * i);
%                                 % Calculem la desviaci estndard de les dades
                                 deviation_i = std(aux(:));
%                                 % Emplenem les corresponents desviacions estndard
%                                 obj.sta(duration * (i - 1) + 1:duration * i) = repmat(deviation_i, 1, duration);
                                
                                 %optimitzat
                                 obj.sta(duration * (i - 1) + 1:duration * i) =  deviation_i;
                            end
                        else%variable wise
                            if(obj.treuFM>=1)
                                %pasem a batch wise
                                obj.conv=0;%fixem a metode de conversio BW
                                data=getRawData(obj);
                                obj.conv=1;%deixem com estava
                                %calculem
                                duration=size(data,2)/vars;%numero de columnes per cada variable
                                for i=1:1:vars
                                    % Seleccionem les columnes que pertanyen a la variable i
                                    aux = data(:,duration * (i - 1) + 1:duration * i);
                                    % Calculem la desviaci estndard de les dades
                                    deviation_i = std(aux(:));
                                    % Emplenem les corresponents desviacions estndard
                                    %obj.sta(duration * (i - 1) + 1:duration * i) = repmat(deviation_i, 1, duration);
                                    
                                    obj.sta(duration * (i - 1) + 1:duration * i) = deviation_i;
                                end
                            else
                                %demanem les dades a l'objecte DATA
                                data=getRawData(obj); 
                                %alculem la mitjana i la desviaci estandard
                                %obj.mea=mean(data,1);
                                obj.sta=std(data,0,1);
                            end
                        end
                        %alculem la mitjana
                        obj.mea=mean(data,1);
                        
                        %Cas 3 = Continous Scaling
                    case 3
                        %agafem el nombre de variables
                        vars=obj.DATA.getVariableCount();
                        if(obj.conv==0)%batch wide
                            %demanem les dades a l'objecte DATA
                            data=getRawData(obj);
                            duration=size(data,2)/vars;%numero de columnes per cada variable
                            for i=1:1:vars
                                % Seleccionem les columnes que pertanyen a la variable i
                                aux = data(:,duration * (i - 1) + 1:duration * i);
                                %Calculem la mitjana i la desviacio tipica
                                mean_i = mean(aux(:));
                                deviation_i = std(aux(:));
                                % Emplenem les mitjanes
                               % obj.mea(duration * (i - 1) + 1: duration *i) = repmat(mean_i, 1, duration); %optimitzem el repmat no es necesari
                               obj.mea(duration * (i - 1) + 1: duration * i) = mean_i;
                               
                                % Emplenem les desviacions
                                %obj.sta(duration * (i - 1) + 1:duration *i) = repmat(deviation_i, 1, duration);%optimitzem el repmat no es necesari
                                obj.sta(duration * (i - 1) + 1:duration * i) = deviation_i;
                            end
                        else%variable wise
                            %pasem a batch wise
                            if(obj.treuFM>=1)
                                obj.conv=0;%fixem a metode de conversio BW
                                data=getRawData(obj);
                                obj.conv=1;%deixem com estava
                                %calculem
                                duration=size(data,2)/vars;%numero de columnes per cada variable
                                for i=1:1:vars
                                    % Seleccionem les columnes que pertanyen a la variable i
                                    aux = data(:,duration * (i - 1) + 1:duration * i);
                                    %Calculem la mitjana i la desviacio tipica
                                    mean_i = mean(aux(:));
                                    deviation_i = std(aux(:));
                                    % Emplenem les mitjanes
                                    %obj.mea(duration * (i - 1) + 1: duration * i) = repmat(mean_i, 1, duration);
                                    % Emplenem les desviacions
                                    %obj.sta(duration * (i - 1) + 1:duration * i) = repmat(deviation_i, 1, duration);
                                    
                                    %optimitzacions
                                    obj.mea(duration * (i - 1) + 1: duration * i) = mean_i;
                                    obj.sta(duration * (i - 1) + 1:duration * i) = deviation_i;
                                end
                            else
                                %demanem les dades a l'objecte DATA
                                data=getRawData(obj);
                                %alculem la mitjana i la desviaci estandard
                                obj.mea=mean(data,1);
                                obj.sta=std(data,0,1);
                            end
                        end
                        
                        %Cas 4 = Block Scaling
                    case 4
                        %agafem el nombre de variables
                        vars=obj.DATA.getVariableCount();
                        if(obj.conv==0)%batch-wise
                            %agafem les duracins
                            duration = obj.DATA.getSampleCount();
                            %Demanem els blocks a l'usuari
                            aux = inputdlg(['Enter the block format: ' char(10) ' You must enter a double list ' char(10) ' The list must contain a double for each variable. ' char(10) ' Each number indicates the number of variables/group.']);
                            block = evalin('base',aux{1});%evaluem per si entren expresions
                            
                            if(length(block) ~= obj.getVariableCount)
                                %Error
                                error('PCA\estandaritza: The length of the block is diferent from the number of variables.');
                            end
                            
                            if(min(block) <= 0 || max(block) > obj.getVariableCount)
                                %Error
                                error('PCA\estandaritza: All elements in block, must be grater than 0 and lower or equal than the number of variables.');
                            end
                            
                            for i=2:max(block)
                                if(mod(sum(block==i),i) ~= 0)
                                    %Error
                                    error(['PCA\estandaritza:the number of ' num2str(i) ' must be multiple of ' num2str(i)]);
                                end
                            end
                            %demanem les dades a l'objecte DATA
                            data=getRawData(obj);
                            %alculem la mitjana i la desviaci estandard
                            obj.mea=mean(data,1);
                            obj.sta=std(data,0,1);
                            for i=1:1:length(block)
                                obj.sta(duration * (i - 1) + 1:duration * i) = obj.sta(duration * (i - 1) + 1:duration * i) .* repmat(sqrt(block(i) * duration), 1, duration);
                            end
                        else%variable wide
                            %pasem a batch wide
                            if(obj.treuFM>=1)
                                obj.conv=0;%fixem a metode de conversio BW
                                data=getRawData(obj);
                                obj.conv=1;%deixem com estava
                                %calculem
                                duration=size(data,2)/vars;%numero de columnes per cada variable
                                
                                %Demanem els blocks a l'usuari
                                aux = inputdlg(['Enter the block format: ' char(10) ' You must enter a double list ' char(10) ' The list must contain a double for each variable. ' char(10) ' Each number indicates the number of variables/group.']);
                                block = evalin('base',aux{1});%evaluem per si entren expresions
                                
                                if(length(block) ~= obj.getVariableCount)
                                    %Error
                                    error('PCA\estandaritza: The length of the block is diferent from the number of variables.');
                                end
                                
                                if(min(block) <= 0 || max(block) > obj.getVariableCount)
                                    %Error
                                    error('PCA\estandaritza: All elements in block, must be grater than 0 and lower or equal than the number of variables.');
                                end
                                
                                for i=2:max(block)
                                    if(mod(sum(block==i),i) ~= 0)
                                        %Error
                                        error(['PCA\estandaritza:the number of ' num2str(i) ' must be multiple of ' num2str(i)]);
                                    end
                                end
                                
                                %alculem la mitjana i la desviaci estandard
                                obj.mea=mean(data,1);
                                obj.sta=std(data,0,1);
                                for i=1:1:length(block)
                                    obj.sta(duration * (i - 1) + 1:duration * i) = obj.sta(duration * (i - 1) + 1:duration * i) .* repmat(sqrt(block(i) * duration), 1, duration);
                                end
                            else
                                %demanem les dades a l'objecte DATA
                                data=getRawData(obj);
                                %alculem la mitjana i la desviaci estandard
                                obj.mea=mean(data,1);
                                obj.sta=std(data,0,1);
                                
                                duration=size(data,2)/vars;%numero de columnes per cada variable
                                
                                
                                %Demanem els blocks a l'usuari
                                aux = inputdlg(['Enter the block format: ' char(10) ' You must enter a double list ' char(10) ' The list must contain a double for each variable. ' char(10) ' Each number indicates the number of variables/group.']);
                                block = evalin('base',aux{1});%evaluem per si entren expresions
                                
                                if(length(block) ~= obj.getVariableCount)
                                    %Error
                                    error('PCA\estandaritza: The length of the block is diferent from the number of variables.');
                                end
                                
                                if(min(block) <= 0 || max(block) > obj.getVariableCount)
                                    %Error
                                    error('PCA\estandaritza: All elements in block, must be grater than 0 and lower or equal than the number of variables.');
                                end
                                
                                for i=2:max(block)
                                    if(mod(sum(block==i),i) ~= 0)
                                        %Error
                                        error(['PCA\estandaritza:the number of ' num2str(i) ' must be multiple of ' num2str(i)]);
                                    end
                                end
                                
                                for j=1:length(block)
                                    obj.sta(j)=obj.sta(j)*sqrt(block(j) * duration);
                                end
                            end
                        end
                    otherwise
                        error('PCA\estandaritza:estandaritzation method not valid')
                end
                
                %calculem la matriu de Z
                obj.Z = (data - repmat(obj.mea, size(data, 1), 1)) ./ repmat(obj.sta, size(data, 1), 1);
                %fixem a estandaritzat
                obj.estandar=1;
                
                if( obj.conv==1 && obj.metodee~=1)%les dades eren VW
                    
                    duration=size(data,2)/vars;%numero de columnes per cada variable
                    %convertim Z de BW a VW
                    x=reshape(obj.Z',[duration vars size(data,1)]);
                    x=permute(x,[1 3 2]);
                    obj.Z=reshape(x,size(x,2)*size(x,1),size(x,3));
                end
            end
        end
        %estandaritz2 - Per quan no es un model
        function estandaritz2(obj)
            %estandaritza la mostra amb la mitja i std que tinguem
            %entrada en aquest moment. Aquesta funcio es privada
            if (obj.conv==1 && obj.metodee~=1 && obj.treuFM>=1)
                obj.conv=0;
                data=obj.getRawData();%agafem les dades en BW encara que estiguem trevallant en VW
                obj.conv=1;%Tornem a indicar que estem en
                %VW
            else
                data=obj.getRawData();%Obtenim les dades en BW o VW(en el cas de metodee==1)
            end
            
            %calculem la matriu de Z
            obj.Z = (data - repmat(obj.mea, size(data, 1), 1)) ./ repmat(obj.sta, size(data, 1), 1);
            %fixem a estandaritzat
            obj.estandar=1;
            
            if( obj.conv==1 && obj.metodee~=1 && obj.treuFM>=1)%les dades eren VW
                %convertim Z de BW a VW
                vars=obj.DATA.getVariableCount();
                duration=size(data,2)/vars;
                x=reshape(obj.Z',[duration vars size(data,1)]);
                x=permute(x,[1 3 2]);
                obj.Z=reshape(x,size(x,2)*size(x,1),size(x,3));
                
            end
            
        end
        
%-------------------------------------------------------------------------%
%% ------------------------ METODES PER CALCULAR -------------------------  
%-------------------------------------------------------------------------%
        
        %CalculaE
        function calculaE(obj)
            %calcula la matriu d'Errors. Aquesta funcio es privada
            if(isempty(obj.P)||isempty(obj.Z))&& obj.getBatchCount>0
                %si no s'han calculat mai les dades ho fem ara
                obj.pca();
            end
            %calculem la matriu i la guardem
            obj.E= (obj.Z-obj.Z*obj.P*obj.P');
        end      
        %Metode que escull entre analitic i 3 sigma
        function calculaLimQ(obj)
            %calculem el limit de Q
            if(obj.analitic)
                %analitic = 1 -> analitic
                calculaLimQv2(obj)
            else
                %analitic = 0 -> 3 sigma
                calculaLimQv1(obj)
            end
        end
        %Metode que escull entre analitic i 3 sigma
        function calculaLimT2(obj)
            %Calculem el limit de T^2
            if(isempty(obj.T) && obj.getBatchCount>0)
                %necesitem la matriu de T si no la tenim la calculem
                obj.pca();
            end

            if(obj.analitic)
                %analitic = 1 -> analitic
                calculaLimT2v2(obj)
            else
                %analitic = 0 -> 3 sigma
                calculaLimT2v1(obj)
            end
        end
        %CalculaLimQ per 3 sigma
        function calculaLimQv1(obj)
            %metode per calcular el limit de Q. Aquesta funcio es privada
            if(isempty(obj.E) && obj.getBatchCount>0)
                %si encara no s'ha calculat mai la matriu d'errors la
                %calculem ara
                getE(obj);
            end
            %fem els calculs i guardem el resultat
            Q=((obj.E).^2)*ones(size(obj.E,2),1);
            obj.LimQ=mean(Q)+std(Q)*3;
        end
        %CalculaLimQ analitic
        function calculaLimQv2(obj)
            %metode per calcular el limit de Q. Aquesta funcio es privada
            if(isempty(obj.Landai) && obj.getBatchCount>0)
                %si encara no s'ha calculat mai la matriu d'errors la
                %calculem ara
                obj.pca();
            end
            %fem els calculs i guardem el resultat
            
            alpha=0.95; %grau de confiana
            ca=norminv(alpha, 0, 1);
            ncp=obj.Num;
            D=obj.Landai;
            theta=zeros(3);
            aux = D(ncp+1:length(D));
            for i=1:3
                theta(i)=sum(aux.^i);
            end
            h0=1-2*theta(1)*theta(3)/3/theta(2)^2;
            obj.LimQ=theta(1)*(ca*h0*sqrt(2*theta(2))/theta(1)+1+theta(2)*h0*(h0-1)/theta(1)^2)^(1/h0);
            
        end
        %CalculaLimT2 per 3 sigma
        function calculaLimT2v1(obj)
            %metode per calcular el limit de T^2. Aquesta funcio es privada
            %guardem les dades
            obj.T2=((obj.T).^2);
            obj.T2=((obj.T2)./repmat(obj.Landa',size(obj.T2,1),1))*ones(size(obj.T2,2),1);
            
            obj.LimT2=mean(obj.T2)+ std(obj.T2)*3;
        end
        %CalculaLimT2 analitic
        function calculaLimT2v2(obj)
            %metode per calcular el limit de T2 analiticament. Aquesta
            %funcio es privada
            if(isempty(obj.Num))
                %si no s'han calculat encara les dades ho fem
                obj.pca();
            end
            %guardem les dades
            alpha=0.95;%es el grau de confiana estre 0 i 1
            n=size(obj.Z,1);%es el nombre d'observacions
            a=obj.Num;%es el numero de components principals retingudea al model
            obj.LimT2=a*(n-1)/(n-a)*finv(alpha,a,n-a);
            
        end
        %CalculaContT2
        function x = calculaContT2(obj)
            %metode per calcular les contribucions de T2. Aquesta funcio es privada
            if(isempty(obj.T) && obj.getBatchCount>0)
                obj.pca();
            end
            %Retorna les dades.
            %T2 mirar componen a component si es pasa -> sum de totes les
            %que es passen
            aux = (obj.T * diag(1./sqrt(obj.Landa)) * obj.P') .* obj.Z;
            
            if(obj.ContMetode == 0)
                %no bolen els negatius els posem tots a 0
                x = aux .* (aux > 0) ;
            else
                %bolen positius i negatius!!
                x = aux;
            end
                       
            if(isempty(obj.CLimT2mea)||isempty(obj.CLimT2std))
                
                % Emplenem les mitjanes
                obj.CLimT2mea=mean(x);%(duration * (i - 1) + 1: duration * i) = repmat(mean_i, 1, duration);
                % Emplenem les desviacions
                obj.CLimT2std=std(x);%(duration * (i - 1) + 1:duration * i) = repmat(deviation_i, 1, duration);
                
            end
            
            
        end
        %calculaContQ
        function x = calculaContQ(obj)
            %Calcula les contribucions a Q i el seu limit. Aquesta funcio
            %es privada
            if(isempty(obj.E)&& obj.getBatchCount>0)
                calculaE(obj);
            end
            
            %Retorna les dades.
            if(obj.ContMetode == 0)
                %no bolen els negatius elevem al ^2
                x = (obj.E).^2;
            else
                %no elevem al quadrat hi haura positius i negatius!
                x = obj.E;
            end
            
            if(isempty(obj.CLimQmea)||isempty(obj.CLimQstd))
                % Emplenem les mitjanes
                obj.CLimQmea=mean(x);%Calculem la contribucio a la mitjana
                % Emplenem les desviacions
                obj.CLimQstd=std(x);%Calculem la contribucio a la std
                
            end
            
        end
        
%-------------------------------------------------------------------------%
%% ---------------------- VISUALITZADORS PARCIALS -------------------------  
%-------------------------------------------------------------------------%  
        %mira si un objecte no model es bo per la contribuci al limit Q
        function vect = foraCQ(obj)
            %mira si un objecte no model es bo per la contribuci al limit
            %Q
            %Parametres d'entrada:
            %
            %   confi - Es el nivell de confiana per decidir si un element
            %           esta fora o no
            %
            %Parametres de sortida:
            %
            %   bool - es la classificaci de cada element
            %
           % if obj.model==0%no es model
                x= (repmat(obj.getQContributionLimitMean-obj.getQContributionLimitStd *3,size(obj.getQContribution,1),1) > obj.getQContribution) + (obj.getQContribution > repmat(obj.getQContributionLimitMean+obj.getQContributionLimitStd *3,size(obj.getQContribution,1),1));%Matriu de booleans amb 0 obs. bones i 0 les dolentes
                z=obj.DATA.getStagesDurations();
                resp=zeros(size(x,1),length(z)*length(obj.DATA.getVariableLabel()));
                k=1;
                for i=1:length(obj.DATA.getVariableLabel())%per cada variable
                    for j=1:length(z)
                        resp(:,(i-1)*length(z)+j)= sum(x(:,k:k+z(j)-1),2);
                        k=k+z(j);
                    end
                end
                vect=resp;
           % else%es model
           %     error('PCA/foraCQ: Es un model!');
           % end
        end      
        %grafic de barres amb contribucions al limit de T2
        function plotLimCT2(obj,char)
            %grafic de barres amb contribucions al limit de T2
            %Parametres d'entrada:
            %
            %   char - el numero de batch o el grup de batch que volem
            %           graficar 
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            if(isempty(obj.CLimT2mea)||isempty(obj.CLimT2std))%si no estan calculades les calculem
                calculaContT2(obj);
            end
            hold on;
            plot(obj.CLimT2mea+obj.CLimT2std*3,char,'HitTest','off');
            
            if(obj.ContMetode == 1)
                %hi haura negatius!
                plot(obj.CLimT2mea-obj.CLimT2std*3,char,'HitTest','off'); 
            end
            
            hold off;
        end
        %grafic de barres amb contribucions al limit de Q
        function plotLimCQ(obj,char)
            %grafic de barres amb contribucions al limit de Q
            %Parametres d'entrada:
            %
            %   char - el numero de batch o el grup de batch que volem
            %           graficar 
            %
            %Parametres de sortida:
            %
            %   No en te
            %
            if(isempty(obj.CLimQmea)||isempty(obj.CLimQstd))%si no estan calculades les calculem
                calculaContQ(obj);
            end
            hold on;
            plot(obj.CLimQmea+obj.CLimQstd*3,char,'HitTest','off');
            
            if(obj.ContMetode == 1)
                %hi haura negatius!
                plot(obj.CLimQmea-obj.CLimQstd*3,char,'HitTest','off');
            end
            
            hold off;
        end  

    end
end

%outliers tip
function out = datatiper(varargin)
%funci per posar el datatip en la funci plotOutLiers()
%No invocar!
%
% varargin - l'estructura que ens passa matlab en crear l'event
% out - El misatge que es mostrara segons que hagi seleccionat l'usuari
try    
dades=get(gca,'Userdata');
    pos=varargin{2}.Position;
    a=get(gco,'yData');
    pos(2)=a(pos(2) - 7);
    nomlots = dades{3};
    if pos(1)==1 ||  pos(1)==2
        numlot=find(dades{pos(1)} ==  pos(2), 1,'first');
        if ~isempty(numlot)
            out ={['Batch number:' num2str(numlot)];['Batch name: ' nomlots{numlot}] };
        else
            out='Not a batch!';
        end
    else
        out='Not a batch!';
    end
catch
    out='Not a batch!';
end
end

%Scores tip
function out = tipme(varargin)
%funci per posar el datatip utilitzat al grafic plotScores
%No invocar!
%
% varargin - l'estructura que ens passa matlab en crear l'event
% out - El misatge que es mostrara segons que hagi seleccionat l'usuari
    dades=get(gca,'Userdata');
    pos=varargin{2}.Position;
    switch length(dades.punts)
        case 1
            numlot=find(dades.punts{1} ==  pos(2),1,'first');
        case 2
            numlot=find(and(dades.punts{1} ==  pos(1), dades.punts{2} ==  pos(2)),1,'first');
        case 3
            numlot=find(and(and(dades.punts{1} ==  pos(1), dades.punts{2} ==  pos(2)), dades.punts{3} ==  pos(3)),1,'first');
    end
        
    if ~isempty(numlot)
        out ={['Batch number:' num2str(numlot)];['Batch name: ' dades.noms{numlot}] };
    else
        out='Not a batch!';
    end

end

%plotT2Q tip
function out = tipmelot(varargin)
%funci per posar el datatip plotT2Q
%No invocar!
%
% varargin - l'estructura que ens passa matlab en crear l'event
% out - El misatge que es mostrara segons que hagi seleccionat l'usuari
try
    dades=get(gca,'Userdata');
    pos=varargin{2}.Position;
    switch length(dades.punts)
        case 1
            numlot=find(dades.punts{1} ==  pos(2),1,'first');
        case 2
            numlot=find(and(dades.punts{1} ==  pos(1), dades.punts{2} ==  pos(2)),1,'first');
        case 3
            numlot=find(and(and(dades.punts{1} ==  pos(1), dades.punts{2} ==  pos(2)), dades.punts{3} ==  pos(3)),1,'first');
    end
        
    if ~isempty(numlot)
        out ={['Batch name: ' dades.noms{numlot}] };
    else
        out='Not a batch!';
    end
catch err
    out='Not a batch!';
end
end