function pls = PCA2PLS(obj)
%funci per convertir objectes PCA a estructures de dades compatibles amb
%la PLS Toolbox.
%
%
%Parametres d'entrada
%
%       obj - un objecte PCA
%
%Parametres de sortida
%
%       pls - una estructura de dades compatible amb PLS Toolbox

x=whos('obj');
if ~strcmp(x.class,'PCA') %comprobem l'entrada sigui de la classe adequada
    error('Error/PCA2PLS: obj must be an instance of PCA class!');
end
%reproduim la estructura asica de dades de PLS Toolbox
temps = clock;
datasource = {struct('name','', 'type','data', 'author','', 'date',temps, 'moddate',temps,'size',size(obj.getZ),'include_size',size(obj.getZ), 'description','','uniqueid',num2str(temps))};
loads ={[obj.getT] ; [obj.getP]};
pred = {[]};
tsqs = {[obj.getT2]; [sum(obj.getP .^2,2)' * (size(obj.getP,1)-1)]};
ssqresiduals = {[obj.getQ];[sum(obj.getQContribution,1)]};
description = {['Principal Components Model'];[];[]};
aux=min(obj.getVariableCount * obj.getSampleCount,obj.getBatchCount-1);
li = obj.getLambdaAll;
li = li(1:aux);
aux2 =li * 100 / sum(li);
ssq = [ (1:aux)' ,li, aux2, cumsum(aux2)];
reslim = {{obj.getQLimit}};
tsqlim = {{obj.getT2Limit}};
li = obj.getLambdaAll;
reseig =  [li(obj.getNum+1:aux)];
includ = {1:obj.getBatchCount(); 1:(obj.getVariableCount*obj.getSampleCount)};

if(obj.getNormalizationMethod == 1) % autoscaling
aux11 = obj.getMean;  
aux22 = obj.getStd;
preprocessing = struct('description','Autoscale','calibrate',{{'[data,out{1},out{2}] = auto(data,userdata);'}},'apply',{{'data = scale(data,out{1},out{2},userdata);'}},'undo',...
    {{'data = rescale(data,out{1},out{2},userdata);'}},'out',{{aux11,aux22}},'settingsgui','autoset','settingsonadd',0,'usesdataset',1,'caloutputs',2,'keyword','Autoscale',...
    'tooltip','Mean center and scale each variable to unit standard deviation','category','Scaling and Centering','userdata',struct('offset',0,'badreplacement',0,'stdthreshold',0));
preprocessing2 = preprocessing;
preprocessing2.out = cell(0,0);
else
    if(obj.getNormalizationMethod == 4) %El seu Grup scaling es el meu Bloc Scaling amb blocs de tot el temps!
        aux11 = obj.getMean;
        aux22 = obj.getStd;
        preprocessing = struct('description','Group Scale','calibrate',{{'[data,out{1},out{2}] = gscale(data,userdata);'}},'apply',{{'data = gscaler(data,userdata,out{1},out{2});'}},'undo',...
            {{'data = gscaler(data,userdata,out{1},out{2},1);'}},'out',{{aux11,aux22}},'settingsgui','gscaleset','settingsonadd',0,'usesdataset',1,'caloutputs',2,'keyword','gscale',...
            'tooltip','Scale groups of variables to grand standard deviation','category','Scaling and Centering','userdata',0);
        preprocessing2 = preprocessing;
    else
        error('Metode no suportat');
    end
end
x=unique(obj.getClass)';
lookup = cell(length(x),2);
for i =1: length(x)
lookup{i,1}=double(x(i));
lookup{i,2}=['Class ' num2str(x(i))];
end
detail = struct('data',{cell(1,1)},'res',{cell(1,1)},'ssq',ssq,'rmsec',[],'rmsecv',[],'esterror',struct([]),'means',{cell(1,1)},'stds',{cell(1,1)},...
   'reslim',reslim ,'tsqlim',tsqlim,'reseig',reseig,'eigsnr',[],'cv','','split',[],'iter',[],'includ',{includ},'label',{{'';''}},'labelname',{{'';''}},'axisscale',{cell(2,1)},...
   'axisscalename',{{'';''}},'title',{{'';''}},'titlename',{{'';''}},'class',{{obj.getClass;[]}},'classname',{{'';''}},'classlookup',{{lookup;[]}},'preprocessing',{{preprocessing}},...
   'options', [struct('name', 'options','display', 'off','plots', 'none','outputversion', 3,'preprocessing', {{preprocessing2}},'algorithm', 'svd','blockdetails', 'standard',...
   'confidencelimit', 0.9500,'roptions', [struct('alpha',0.75,'cutoff',[])],'definitions', @optiondefs,'functionname', 'pca','rawmodel', 1)]);

help = struct('predictions',[...
    struct('label','Scores','field','loads{1}','dimension','vector'),...
    struct('label','Hotelling''s T^2','field','tsqs{1}','dimension','scalar'),...
    struct('label','Q Residuals','field','ssqresiduals{1}','dimension','scalar'),...
    struct('label','T Contributions','field','tcon','dimension','vector'),...
    struct('label','Q Contributions','field','qcon','dimension','vector')...
    ]);

pls = struct();
pls.modeltype='PCA';
pls.author='auto generated';
pls.datasource=datasource;
pls.date= [num2str(temps(3)) '/' num2str(temps(2))  '/' num2str(temps(1)) ' - ' num2str(temps(4)) 'h' num2str(temps(5)) 'm'];
pls.time=temps;
pls.info='Scores are in cell 1 of the loads field.';
pls.loads= loads;
pls.pred=pred;
pls.tsqs= tsqs;
pls.ssqresiduals= ssqresiduals;
pls.description= description;
pls.detail= detail;
pls.help= help;
pls.modelversion= '6.5';
end
