%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% C_r_i = compute_reconstruction_correlation_matrix(Xi_i, Xi_res_i)       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FUNCTIONALITY:                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function aim is to compute the matrix that relates the variables   %
% to use when reconstructing a faulty observation for a given fault       %
% direction (Xi_i), using the projection of that fault direction in the   %
% residual subspace (Xi_res_i).                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT PARAMETERS:                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Xi_i:     Column array or matrix with as many rows as variables were    %
%           used to build the PCA model and as many columns as individual %
%           sensor faults are involved for the multisensory fault vector. %
% Xi_res_i: Projection of xi_i in the residual subspace of a PCA model.   %
%           It has the same dimensions as Xi_i.                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OUTPUT PARAMETERS:                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% C_r_i:    Matrix that indicates the weight of each variable included to %
%           build the PCA model when reconstructing using the fault       %
%           direction Xi_i (and its projection in the residual subspace   %
%           Xi_res_i). C_r_i is a square matrix with as many rows and     %
%           columns as variables where used to build the PCA model.       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function C_r_i = compute_reconstruction_correlation_matrix(Xi_i, Xi_res_i)
% We compute the the value of C_r_i for the fault direction Xi_i and its
% projection in the residual subspace Xi_res_i. The resulting value is
% subtracted from an identity matrix in order to have all variables
% affected by fault direction Xi_i have no influence on the reconstruction
% (their value are set to 0).
C_r_i = eye(size(Xi_i, 1)) - Xi_i * ((Xi_res_i' * Xi_res_i)\Xi_res_i');