function out = talla(obje,indexsvars,indextemps)
%Aquesta funci trevalla sobre objectes DATA, PCA i CBR. El que fa es
%modificar les seves dades per un sub conjunt de les actuals. Es pot
%indicar un nou rang de valors pel que fa a les observacins obe per a les
%variables observades.
%
% Aplicat sobre CBR:
%tallem el Model i la Base de casos per separat i reconstruim el
%CBR no guardarem les configuracins de l'objecte actual pel que fa al
%nombre de veins distancies i optimitzacins perqu en realitat s un CBR
%nou sobre un model nou.
%
% Aplicat sobre PCA:
% Noms es pot aplicar sobre models no sobre projeccins. El nou objecte
% PCA portar les configuracions per defecte!
%
%Parametres d'entrada:
%
%   indexvars - Qualsevol tipus d'index valid en matlab (logicals, doubles,
%               caracters,..) indica les variables de l'objecte actual que
%               voldrem en la versi tallada.
%
%               Exemple:
%               Si teni un objecte amb 4 variables i bolem nomes la segona
%               i la terera els valors possibles de indexvars serien:
%
%               [0 1 1 0] o be [2 3] o be [2 ; 3] ...
%
%
%
%   indextemps - Qualsevol tipus d'index valid en matlab (logicals,
%                doubles, caracters,..) indica els instants de temps de
%                l'objecte actual que voldrem en la versi tallada.
%                !!El format ha de ser el mateix que a indexvars!!
%
%               Exemple:
%               Si teni un objecte amb 481 instants de temps i bolem nomes 
%               els instants de temps del 12 al 120 els possibles valors de
%               indextemps que ens permetrien obtenir els resultats
%               desitjats son:
%
%               [12:120] o be [zeros(1,11) ones(1,120-11) zeros(1,481-120)]
%
%
%
%Parametres de sortida:
%
%   out - Un objecte del mateix tipus que obje al qual se li han modificat
%         les variables i el temps segons lo indicat a indexsvars i
%         indextemps. S'hauran modificat tot s els parametres interns en
%         consequencia (Fases, contribucions, mitjes,...)
%

%segons el tipus d'objecte que ens passin farem una cosa o una altre
x=whos('obje');
x=x.class;
switch x
    case 'CBR'
        %tallem el model i la base de casos per separat i reconstruirem el
        %CBR
        M = talla(obje.getModel(),indexsvars,indextemps);
        C = talla(obje.getCaseBase.getDATA(),indexsvars,indextemps);
        
        out= CBR(M,C);
        
    case 'PCA'
        if(obje.isModel)
            %tallem un objecte PCA Model tallan l'objecte dades intern i
            %reconstruint l'objecte
            M = talla(obje.getDATA(),indexsvars,indextemps);
            out = PCA(M);
            
            %Posem la configuraci com avans!
            out.setNormalizationMethod(obje.getNormalizationMethod);
            out.setPCSelectionMethod(obje.getPCSelectionMethod);
            out.setTrajectoryRemoval(obje.getTrajectoryRemoval);
            out.setContributionMethod(obje.getContMetode);
            out.setLimitMethod(obje.getLimitMethod);
            out.setUnfoldingMethod(obje.getUnfoldingMethod);
        else
            %no podem projectar-lo de nou!!
            error('Error/talla:Undefined for non model PCA!');
        end
        
    case 'DATA'
        %Recuperarem les dades en 3D i tallarem segons en indiica l'usuari
        M = obje.getRawData3D;
        out = obje.copy();
        
        if  (ischar(indextemps) && strcmp(indextemps,':'))
            out.setRawData(M(indextemps,indexsvars,:)); %el temps es :
        else
            if max(indextemps) > (size(M,1))
                indextemps(end) = (size(M,1));
            end
            out.setRawData(M(indextemps,indexsvars,:)); %son numeros 0...tems sumem 1
        end
        out.setVariableLabel(out.getVariableLabel(indexsvars));
        
        %Fases
        %falta tallar les fases
        if isempty(obje.getStages) || isempty(obje.getStagesDurations) || ~(ischar(indextemps) && strcmp(indextemps,':'))
            
            if isnumeric(indextemps)
                dura = obje.getStagesDurations;
                fase = obje.getStages;
                csum=cumsum(dura);
                if ~isempty(csum)
                    i = (csum - min(indextemps)) >= 0;
                    dura = dura(i);
                    fase = fase(i);
                    
                    %calculem els desfases
                    z=1:length(i);
                    offset=min(z(i));%posici que ocupava el que ara es ep primer de dura
                    
                    aux = (csum +1 - min(indextemps));
                    dura(1) = aux(offset);%el valor que dura la fase 1 pot variar
                    
                    csum = cumsum(dura);
                    i = (csum - (max(indextemps) - min(indextemps)) -1) <= 0;

                    
                    %calculem els desfases
                    z=1:length(i);
                    offset=max(z(i));%posici que ocupava el que ara es l'ultim de dura
                    
                    
                    
                    
                    
                    if isempty(offset)
                        aux = max(indextemps);
                        offset =1;
                    else
                        aux=(csum - (max(indextemps) - min(indextemps) +1));
                    end
                    
                    if abs(aux(offset)) ~= 0 %hi ha una unltima fase pero no entera
                        %if length(dura)~=length(fase) %i queden de disponibles
                            dura = [dura(i) abs(aux(offset))];
                        %end
                    else
                        dura = dura(i);
                    end
                    
                    
                    fase = fase(1:length(dura));%tallem els noms en consequencia
                    
                    %i les posem al nou objecte!
                    out.setStages(fase);
                    out.setStagesDurations(dura);
                end
            else
                %si no es numeric son els : i ja esta be com esta perque els bolem totes
            end
        end
        %Si les fases estan buides es queden =
        
    otherwise
         error('Unsuported class!');
end

end